/*
 *      MIB table declaration
 *
 *      Authors: David Hsu	<davidhsu@realtek.com.tw>
 *
 *      $Id: mibtbl.c,v 1.12 2005/05/31 11:33:32 richardh Exp $
 *
 */

/* Include files */
#include "apmib.h"
#include "mibtbl.h"

/* Global variable definitions */

/*
 * When using flash (set/get/all) command to access the MIB of below table,
 * it needs append a keyword "HW_" in ahead of mib name.
 */

mib_table_entry_T hwmib_table[]={
{MIB_HW_BOARD_VER,	"BOARD_ID",		BYTE_T,		_OFFSET_HW(boardVer),	_SIZE_HW(boardVer)},
{MIB_HW_NIC0_ADDR,	"NIC0_ADDR",	BYTE6_T, 	_OFFSET_HW(nic0Addr),	_SIZE_HW(nic0Addr)},
{MIB_HW_NIC1_ADDR,	"NIC1_ADDR",	BYTE6_T, 	_OFFSET_HW(nic1Addr),	_SIZE_HW(nic1Addr)},
{0}
};




/*
 * When using flash (set/get/all) command to access the MIB of below table,
 * it needs append a keyword "HW_WLANx_" in ahead of mib name.
 */

mib_table_entry_T hwmib_wlan_table[]={
{MIB_HW_WLAN_ADDR,		"WLAN_ADDR",		BYTE6_T,	_OFFSET_HW_WLAN(macAddr),	_SIZE_HW_WLAN(macAddr)},
{MIB_HW_REG_DOMAIN,		"REG_DOMAIN",		BYTE_T, 	_OFFSET_HW_WLAN(regDomain),	_SIZE_HW_WLAN(regDomain)},
{MIB_HW_RF_TYPE, 		"RF_TYPE",			BYTE_T, 	_OFFSET_HW_WLAN(rfType),	_SIZE_HW_WLAN(rfType)},
{MIB_HW_TX_POWER_CCK,	"TX_POWER_CCK",		BYTE_ARRAY_T, 	_OFFSET_HW_WLAN(txPowerCCK),	_SIZE_HW_WLAN(txPowerCCK)},
{MIB_HW_TX_POWER_OFDM,	"TX_POWER_OFDM",	BYTE_ARRAY_T, 	_OFFSET_HW_WLAN(txPowerOFDM),	_SIZE_HW_WLAN(txPowerOFDM)},
{MIB_HW_ANT_DIVERSITY,	"ANT_DIVERSITY",	BYTE_T, 	_OFFSET_HW_WLAN(antDiversity), 		_SIZE_HW_WLAN(antDiversity)},
{MIB_HW_TX_ANT, 		"TX_ANT",			BYTE_T, 	_OFFSET_HW_WLAN(txAnt),		_SIZE_HW_WLAN(txAnt)},
{MIB_HW_INIT_GAIN, 		"INIT_GAIN",		BYTE_T, 	_OFFSET_HW_WLAN(initGain),	_SIZE_HW_WLAN(initGain)},
{MIB_HW_CCA_MODE, 		"CCA_MODE",			BYTE_T, 	_OFFSET_HW_WLAN(ccaMode),	_SIZE_HW_WLAN(ccaMode)},
{MIB_HW_LED_TYPE, 		"LED_TYPE",			BYTE_T, 	_OFFSET_HW_WLAN(ledType),	_SIZE_HW_WLAN(ledType)},
{0}
};




/*
 * When using flash (set/get/all) command to access the MIB of below table,
 * it needs append a keyword "DEF_" in ahead of mib name when access
 * default setting MIB.
 */

mib_table_entry_T mib_table[]={
#ifdef HOME_GATEWAY
{MIB_WAN_MAC_ADDR, 		"WAN_MAC_ADDR",		BYTE6_T, 	_OFFSET(wanMacAddr), _SIZE(wanMacAddr)},
{MIB_WAN_IP_ADDR,		"WAN_IP_ADDR", 		IA_T,		_OFFSET(wanIpAddr), _SIZE(wanIpAddr)},
{MIB_WAN_SUBNET_MASK,		"WAN_SUBNET_MASK", 	IA_T, 		_OFFSET(wanSubnetMask), _SIZE(wanSubnetMask)},
{MIB_WAN_DEFAULT_GATEWAY,	"WAN_DEFAULT_GATEWAY",	IA_T,		_OFFSET(wanDefaultGateway), _SIZE(wanDefaultGateway)},
{MIB_WAN_DHCP,			"WAN_DHCP",		BYTE_T,		_OFFSET(wanDhcp), _SIZE(wanDhcp)},
{MIB_WAN_DNS_MODE,		"DNS_MODE",		BYTE_T,		_OFFSET(dnsMode), _SIZE(dnsMode)},
{MIB_PPP_USER,			"PPP_USER_NAME",	STRING_T,	_OFFSET(pppUserName), _SIZE(pppUserName)},
{MIB_EXTRA_PPP_USER,	"EXTRA_PPP_USER_NAME",	STRING_T,	_OFFSET(extraPppUserName), _SIZE(extraPppUserName)},
{MIB_PPP_PASSWORD,		"PPP_PASSWORD", 	STRING_T,	_OFFSET(pppPassword), _SIZE(pppPassword)},
{MIB_PPP_IDLE_TIME,		"PPP_IDLE_TIME",	WORD_T,		_OFFSET(pppIdleTime), _SIZE(pppIdleTime)},
{MIB_PPP_CONNECT_TYPE,		"PPP_CONNECT_TYPE",	BYTE_T,		_OFFSET(pppConnectType), _SIZE(pppConnectType)},
{MIB_PPP_MTU_SIZE,		"PPP_MTU_SIZE",		WORD_T,		_OFFSET(pppMtuSize), _SIZE(pppMtuSize)},
{MIB_PPTP_IP,			"PPTP_IP_ADDR", 	IA_T,		_OFFSET(pptpIpAddr), _SIZE(pptpIpAddr)},
{MIB_PPTP_SUBNET_MASK,		"PPTP_SUBNET_MASK", 	IA_T, 		_OFFSET(pptpSubnetMask), _SIZE(pptpSubnetMask)},
{MIB_PPTP_SERVER_IP,		"PPTP_SERVER_IP_ADDR",	IA_T,		_OFFSET(pptpServerIpAddr), _SIZE(pptpServerIpAddr)},
{MIB_PPTP_USER,			"PPTP_USER_NAME",	STRING_T,	_OFFSET(pptpUserName), _SIZE(pptpUserName)},
{MIB_PPTP_PASSWORD,		"PPTP_PASSWORD", 	STRING_T,	_OFFSET(pptpPassword), _SIZE(pptpPassword)},
{MIB_PPTP_MTU_SIZE,		"PPTP_MTU_SIZE",	WORD_T,		_OFFSET(pptpMtuSize), _SIZE(pptpMtuSize)},
{MIB_L2TP_IP,			"L2TP_IP_ADDR", 	IA_T,		_OFFSET(l2tpIpAddr), _SIZE(l2tpIpAddr)},
{MIB_L2TP_SUBNET_MASK,		"L2TP_SUBNET_MASK", 	IA_T, 		_OFFSET(l2tpSubnetMask), _SIZE(l2tpSubnetMask)},
{MIB_L2TP_SERVER_IP,		"L2TP_SERVER_IP_ADDR",	IA_T,		_OFFSET(l2tpServerIpAddr), _SIZE(l2tpServerIpAddr)},
{MIB_L2TP_USER,			"L2TP_USER_NAME",	STRING_T,	_OFFSET(l2tpUserName), _SIZE(l2tpUserName)},
{MIB_L2TP_PASSWORD,		"L2TP_PASSWORD", 	STRING_T,	_OFFSET(l2tpPassword), _SIZE(l2tpPassword)},
{MIB_L2TP_MTU_SIZE,		"L2TP_MTU_SIZE",	WORD_T,		_OFFSET(l2tpMtuSize), _SIZE(l2tpMtuSize)},
{MIB_L2TP_TIMEOUT,		"L2TP_TIMEOUT",	WORD_T,		_OFFSET(l2tpTimeout), _SIZE(l2tpTimeout)},
{MIB_L2TP_IPMODE,		"L2TP_IPMODE",		BYTE_T,		_OFFSET(l2tpIpmode), _SIZE(l2tpIpmode)},
{MIB_NTP_ENABLED, 		"NTP_ENABLED",	BYTE_T, 	_OFFSET(ntpEnabled), _SIZE(ntpEnabled)},
{MIB_NTP_SERVER_ID, 		"NTP_SERVER_ID",	BYTE_T, 	_OFFSET(ntpServerId), _SIZE(ntpServerId)},
{MIB_NTP_TIMEZONE, 		"NTP_TIMEZONE",	STRING_T, 	_OFFSET(ntpTimeZone), _SIZE(ntpTimeZone)},
{MIB_NTP_SERVER_IP1, 		"NTP_SERVER_IP1",	IA_T, 	_OFFSET(ntpServerIp1), _SIZE(ntpServerIp1)},
{MIB_NTP_SERVER_IP2, 		"NTP_SERVER_IP2",	IA_T, 	_OFFSET(ntpServerIp2), _SIZE(ntpServerIp2)},
{MIB_NTP_SERVER_INDEX,	"NTP_SERVER_INDEX",		BYTE_T, 	_OFFSET(ntpsrvIndex), _SIZE(ntpsrvIndex)},
{MIB_Daylight_Saving_Time, 	"Daylight_Saving_Time",	BYTE_T, 	_OFFSET(savingtime), _SIZE(savingtime)},
{MIB_Start_dstMonth, 	"Start_dstMonth",	BYTE_T, 	_OFFSET(dstStartMon), _SIZE(dstStartMon)},
{MIB_Start_dstDay, 	"Start_dstDay",	BYTE_T, 	_OFFSET(dsrStartDay), _SIZE(dsrStartDay)},
{MIB_End_dstMonth, 	"End_dstMonth",	BYTE_T, 	_OFFSET(dstEndMon), _SIZE(dstEndMon)},
{MIB_End_dstDay, 	"End_dstDay",	BYTE_T, 	_OFFSET(dstEndDay), _SIZE(dstEndDay)},
{MIB_DDNS_ENABLED,		"DDNS_ENABLED",		BYTE_T,		_OFFSET(ddnsEnabled), 	_SIZE(ddnsEnabled)},
{MIB_DDNS_TYPE,			"DDNS_TYPE",		BYTE_T,		_OFFSET(ddnsType), 		_SIZE(ddnsType)},
{MIB_DDNS_DOMAIN_NAME,	"DDNS_DOMAIN_NAME",	STRING_T,	_OFFSET(ddnsDomainName),	_SIZE(ddnsDomainName)},
{MIB_DDNS_USER,			"DDNS_USER",		STRING_T,	_OFFSET(ddnsUser), 		_SIZE(ddnsUser)},
{MIB_DDNS_PASSWORD,		"DDNS_PASSWORD",	STRING_T,	_OFFSET(ddnsPassword), 	_SIZE(ddnsPassword)},
{MIB_DDNS_RESULT,		"DDNS_RESULT",		STRING_T,	_OFFSET(ddnsResult), 	_SIZE(ddnsResult)},
{MIB_PORTFW_ENABLED,		"PORTFW_ENABLED",	BYTE_T,		_OFFSET(portFwEnabled), _SIZE(portFwEnabled)},
{MIB_PORTFW_NUM,		"PORTFW_TBL_NUM",	BYTE_T,		_OFFSET(portFwNum), _SIZE(portFwNum)},
{MIB_PORTFW,			"PORTFW_TBL",		PORTFW_ARRAY_T,	_OFFSET(portFwArray), _SIZE(portFwArray)},
{MIB_IPFILTER_ENABLED,		"IPFILTER_ENABLED",	BYTE_T,		_OFFSET(ipFilterEnabled), _SIZE(ipFilterEnabled)},
{MIB_IPFILTER_NUM,		"IPFILTER_TBL_NUM",	BYTE_T,		_OFFSET(ipFilterNum), _SIZE(ipFilterNum)},
{MIB_IPFILTER,			"IPFILTER_TBL",		IPFILTER_ARRAY_T,_OFFSET(ipFilterArray), _SIZE(ipFilterArray)},
{MIB_PORTFILTER_ENABLED,	"PORTFILTER_ENABLED",	BYTE_T,		_OFFSET(portFilterEnabled), _SIZE(portFilterEnabled)},
{MIB_PORTFILTER_NUM,		"PORTFILTER_TBL_NUM",	BYTE_T,		_OFFSET(portFilterNum), _SIZE(portFilterNum)},
{MIB_PORTFILTER,		"PORTFILTER_TBL",	PORTFILTER_ARRAY_T,_OFFSET(portFilterArray), _SIZE(portFilterArray)},
{MIB_MACFILTER_ENABLED,		"MACFILTER_ENABLED",	BYTE_T,		_OFFSET(macFilterEnabled), _SIZE(macFilterEnabled)},
{MIB_MACFILTER_NUM,		"MACFILTER_TBL_NUM",	BYTE_T,		_OFFSET(macFilterNum), _SIZE(macFilterNum)},
{MIB_MACFILTER,			"MACFILTER_TBL",	MACFILTER_ARRAY_T,_OFFSET(macFilterArray), _SIZE(macFilterArray)},
{MIB_URLFILTER_ENABLED,	"URLFILTER_ENABLED",BYTE_T,		_OFFSET(urlFilterEnabled),	_SIZE(urlFilterEnabled)},
{MIB_URLFILTER_NUM,		"URLFILTER_TBL_NUM",BYTE_T,		_OFFSET(urlFilterNum), _SIZE(urlFilterNum)},
{MIB_URLFILTER,			"URLFILTER_TBL",	URLFILTER_ARRAY_T,_OFFSET(urlFilterArray),	_SIZE(urlFilterArray)},
{MIB_TRIGGERPORT_ENABLED,	"TRIGGERPORT_ENABLED",	BYTE_T,		_OFFSET(triggerPortEnabled), _SIZE(triggerPortEnabled)},
{MIB_TRIGGERPORT_NUM,		"TRIGGERPORT_TBL_NUM",	BYTE_T,		_OFFSET(triggerPortNum), _SIZE(triggerPortNum)},
{MIB_TRIGGERPORT,		"TRIGGERPORT_TBL",	TRIGGERPORT_ARRAY_T,_OFFSET(triggerPortArray), _SIZE(triggerPortArray)},
{MIB_DMZ_ENABLED,		"DMZ_ENABLED", 		BYTE_T,		_OFFSET(dmzEnabled), _SIZE(dmzEnabled)},
{MIB_DMZ_HOST,			"DMZ_HOST", 		IA_T,		_OFFSET(dmzHost), _SIZE(dmzHost)},
{MIB_SPAP_ENABLE1,		"SPAP_ENABLE1",		BYTE_T,		_OFFSET(spapenable1), _SIZE(spapenable1)},
{MIB_SPAP_NAME1,		"SPAP_NAME1",		STRING_T,	_OFFSET(apname1), _SIZE(apname1)},
{MIB_SPAP_INTYPE1,		"SPAP_INTYPE1",		BYTE_T,		_OFFSET(intype1), _SIZE(intype1)},
{MIB_SPAP_INSTART1,		"SPAP_INSTART1",	WORD_T,		_OFFSET(instart1), _SIZE(instart1)},
{MIB_SPAP_INEND1,		"SPAP_INEND1",		WORD_T,		_OFFSET(inend1), _SIZE(inend1)},
{MIB_SPAP_TRTYPE1,		"SPAP_TRTYPE1",		BYTE_T,		_OFFSET(trtype1), _SIZE(trtype1)},
{MIB_SPAP_TRSTART1,		"SPAP_TRSTART1",	WORD_T,		_OFFSET(trstart1), _SIZE(trstart1)},
{MIB_SPAP_TREND1,		"SPAP_TREND1",		WORD_T,		_OFFSET(trend1), _SIZE(trend1)},
{MIB_SPAP_ENABLE2,		"SPAP_ENABLE2",		BYTE_T,		_OFFSET(spapenable2), _SIZE(spapenable2)},
{MIB_SPAP_NAME2,		"SPAP_NAME2",		STRING_T,	_OFFSET(apname2), _SIZE(apname2)},
{MIB_SPAP_INTYPE2,		"SPAP_INTYPE2",		BYTE_T,		_OFFSET(intype2), _SIZE(intype2)},
{MIB_SPAP_INSTART2,		"SPAP_INSTART2",	WORD_T,		_OFFSET(instart2), _SIZE(instart2)},
{MIB_SPAP_INEND2,		"SPAP_INEND2",		WORD_T,		_OFFSET(inend2), _SIZE(inend2)},
{MIB_SPAP_TRTYPE2,		"SPAP_TRTYPE2",		BYTE_T,		_OFFSET(trtype2), _SIZE(trtype2)},
{MIB_SPAP_TRSTART2,		"SPAP_TRSTART2",	WORD_T,		_OFFSET(trstart2), _SIZE(trstart2)},
{MIB_SPAP_TREND2,		"SPAP_TREND2",		WORD_T,		_OFFSET(trend2), _SIZE(trend2)},
{MIB_SPAP_ENABLE3,		"SPAP_ENABLE3",		BYTE_T,		_OFFSET(spapenable3), _SIZE(spapenable3)},
{MIB_SPAP_NAME3,		"SPAP_NAME3",		STRING_T,	_OFFSET(apname3), _SIZE(apname3)},
{MIB_SPAP_INTYPE3,		"SPAP_INTYPE3",		BYTE_T,		_OFFSET(intype3), _SIZE(intype3)},
{MIB_SPAP_INSTART3,		"SPAP_INSTART3",	WORD_T,		_OFFSET(instart3), _SIZE(instart3)},
{MIB_SPAP_INEND3,		"SPAP_INEND3",		WORD_T,		_OFFSET(inend3), _SIZE(inend3)},
{MIB_SPAP_TRTYPE3,		"SPAP_TRTYPE3",		BYTE_T,		_OFFSET(trtype3), _SIZE(trtype3)},
{MIB_SPAP_TRSTART3,		"SPAP_TRSTART3",	WORD_T,		_OFFSET(trstart3), _SIZE(trstart3)},
{MIB_SPAP_TREND3,		"SPAP_TREND3",		WORD_T,		_OFFSET(trend3), _SIZE(trend3)},
{MIB_SPAP_ENABLE4,		"SPAP_ENABLE4",		BYTE_T,		_OFFSET(spapenable4), _SIZE(spapenable4)},
{MIB_SPAP_NAME4,		"SPAP_NAME4",		STRING_T,	_OFFSET(apname4), _SIZE(apname4)},
{MIB_SPAP_INTYPE4,		"SPAP_INTYPE4",		BYTE_T,		_OFFSET(intype4), _SIZE(intype4)},
{MIB_SPAP_INSTART4,		"SPAP_INSTART4",	WORD_T,		_OFFSET(instart4), _SIZE(instart4)},
{MIB_SPAP_INEND4,		"SPAP_INEND4",		WORD_T,		_OFFSET(inend4), _SIZE(inend4)},
{MIB_SPAP_TRTYPE4,		"SPAP_TRTYPE4",		BYTE_T,		_OFFSET(trtype4), _SIZE(trtype4)},
{MIB_SPAP_TRSTART4,		"SPAP_TRSTART4",	WORD_T,		_OFFSET(trstart4), _SIZE(trstart4)},
{MIB_SPAP_TREND4,		"SPAP_TREND4",		WORD_T,		_OFFSET(trend4), _SIZE(trend4)},
{MIB_SPAP_ENABLE5,		"SPAP_ENABLE5",		BYTE_T,		_OFFSET(spapenable5), _SIZE(spapenable5)},
{MIB_SPAP_NAME5,		"SPAP_NAME5",		STRING_T,	_OFFSET(apname5), _SIZE(apname5)},
{MIB_SPAP_INTYPE5,		"SPAP_INTYPE5",		BYTE_T,		_OFFSET(intype5), _SIZE(intype5)},
{MIB_SPAP_INSTART5,		"SPAP_INSTART5",	WORD_T,		_OFFSET(instart5), _SIZE(instart5)},
{MIB_SPAP_INEND5,		"SPAP_INEND5",		WORD_T,		_OFFSET(inend5), _SIZE(inend5)},
{MIB_SPAP_TRTYPE5,		"SPAP_TRTYPE5",		BYTE_T,		_OFFSET(trtype5), _SIZE(trtype5)},
{MIB_SPAP_TRSTART5,		"SPAP_TRSTART5",	WORD_T,		_OFFSET(trstart5), _SIZE(trstart5)},
{MIB_SPAP_TREND5,		"SPAP_TREND5",		WORD_T,		_OFFSET(trend5), _SIZE(trend5)},
{MIB_SPAP_ENABLE6,		"SPAP_ENABLE6",		BYTE_T,		_OFFSET(spapenable6), _SIZE(spapenable6)},
{MIB_SPAP_NAME6,		"SPAP_NAME6",		STRING_T,	_OFFSET(apname6), _SIZE(apname6)},
{MIB_SPAP_INTYPE6,		"SPAP_INTYPE6",		BYTE_T,		_OFFSET(intype6), _SIZE(intype6)},
{MIB_SPAP_INSTART6,		"SPAP_INSTART6",	WORD_T,		_OFFSET(instart6), _SIZE(instart6)},
{MIB_SPAP_INEND6,		"SPAP_INEND6",		WORD_T,		_OFFSET(inend6), _SIZE(inend6)},
{MIB_SPAP_TRTYPE6,		"SPAP_TRTYPE6",		BYTE_T,		_OFFSET(trtype6), _SIZE(trtype6)},
{MIB_SPAP_TRSTART6,		"SPAP_TRSTART6",	WORD_T,		_OFFSET(trstart6), _SIZE(trstart6)},
{MIB_SPAP_TREND6,		"SPAP_TREND6",		WORD_T,		_OFFSET(trend6), _SIZE(trend6)},
{MIB_SPAP_ENABLE7,		"SPAP_ENABLE7",		BYTE_T,		_OFFSET(spapenable7), _SIZE(spapenable7)},
{MIB_SPAP_NAME7,		"SPAP_NAME7",		STRING_T,	_OFFSET(apname7), _SIZE(apname7)},
{MIB_SPAP_INTYPE7,		"SPAP_INTYPE7",		BYTE_T,		_OFFSET(intype7), _SIZE(intype7)},
{MIB_SPAP_INSTART7,		"SPAP_INSTART7",	WORD_T,		_OFFSET(instart7), _SIZE(instart7)},
{MIB_SPAP_INEND7,		"SPAP_INEND7",		WORD_T,		_OFFSET(inend7), _SIZE(inend7)},
{MIB_SPAP_TRTYPE7,		"SPAP_TRTYPE7",		BYTE_T,		_OFFSET(trtype7), _SIZE(trtype7)},
{MIB_SPAP_TRSTART7,		"SPAP_TRSTART7",	WORD_T,		_OFFSET(trstart7), _SIZE(trstart7)},
{MIB_SPAP_TREND7,		"SPAP_TREND7",		WORD_T,		_OFFSET(trend7), _SIZE(trend7)},
{MIB_SPAP_ENABLE8,		"SPAP_ENABLE8",		BYTE_T,		_OFFSET(spapenable8), _SIZE(spapenable8)},
{MIB_SPAP_NAME8,		"SPAP_NAME8",		STRING_T,	_OFFSET(apname8), _SIZE(apname8)},
{MIB_SPAP_INTYPE8,		"SPAP_INTYPE8",		BYTE_T,		_OFFSET(intype8), _SIZE(intype8)},
{MIB_SPAP_INSTART8,		"SPAP_INSTART8",	WORD_T,		_OFFSET(instart8), _SIZE(instart8)},
{MIB_SPAP_INEND8,		"SPAP_INEND8",		WORD_T,		_OFFSET(inend8), _SIZE(inend8)},
{MIB_SPAP_TRTYPE8,		"SPAP_TRTYPE8",		BYTE_T,		_OFFSET(trtype8), _SIZE(trtype8)},
{MIB_SPAP_TRSTART8,		"SPAP_TRSTART8",	WORD_T,		_OFFSET(trstart8), _SIZE(trstart8)},
{MIB_SPAP_TREND8,		"SPAP_TREND8",		WORD_T,		_OFFSET(trend8), _SIZE(trend8)},
{MIB_ALG_ENABLE1,		"ALG_ENABLE1",		BYTE_T,		_OFFSET(ALGEnable1), _SIZE(ALGEnable1)},
{MIB_ALG_ENABLE2,		"ALG_ENABLE2",		BYTE_T,		_OFFSET(ALGEnable2), _SIZE(ALGEnable2)},
{MIB_ALG_ENABLE3,		"ALG_ENABLE3",		BYTE_T,		_OFFSET(ALGEnable3), _SIZE(ALGEnable3)},
{MIB_ALG_ENABLE4,		"ALG_ENABLE4",		BYTE_T,		_OFFSET(ALGEnable4), _SIZE(ALGEnable4)},
{MIB_ALG_ENABLE5,		"ALG_ENABLE5",		BYTE_T,		_OFFSET(ALGEnable5), _SIZE(ALGEnable5)},
{MIB_ALG_ENABLE6,		"ALG_ENABLE6",		BYTE_T,		_OFFSET(ALGEnable6), _SIZE(ALGEnable6)},
{MIB_ALG_ENABLE7,		"ALG_ENABLE7",		BYTE_T,		_OFFSET(ALGEnable7), _SIZE(ALGEnable7)},
{MIB_ALG_ENABLE8,		"ALG_ENABLE8",		BYTE_T,		_OFFSET(ALGEnable8), _SIZE(ALGEnable8)},
{MIB_DOS_ENABLED, 		"DOS_ENABLED",		DWORD_T, 	_OFFSET(dosEnabled), 	_SIZE(dosEnabled)},
{MIB_DOS_SYSSYN_FLOOD, 	"DOS_SYSSYN_FLOOD",	WORD_T, 	_OFFSET(syssynFlood), 	_SIZE(syssynFlood)},
{MIB_DOS_SYSFIN_FLOOD, 	"DOS_SYSFIN_FLOOD",	WORD_T, 	_OFFSET(sysfinFlood), 	_SIZE(sysfinFlood)},
{MIB_DOS_SYSUDP_FLOOD, 	"DOS_SYSUDP_FLOOD",	WORD_T, 	_OFFSET(sysudpFlood), 	_SIZE(sysudpFlood)},
{MIB_DOS_SYSICMP_FLOOD, "DOS_SYSICMP_FLOOD",WORD_T, 	_OFFSET(sysicmpFlood), 	_SIZE(sysicmpFlood)},
{MIB_DOS_PIPSYN_FLOOD, 	"DOS_PIPSYN_FLOOD",	WORD_T, 	_OFFSET(pipsynFlood), 	_SIZE(pipsynFlood)},
{MIB_DOS_PIPFIN_FLOOD, 	"DOS_PIPFIN_FLOOD",	WORD_T, 	_OFFSET(pipfinFlood), 	_SIZE(pipfinFlood)},
{MIB_DOS_PIPUDP_FLOOD, 	"DOS_PIPUDP_FLOOD",	WORD_T, 	_OFFSET(pipudpFlood), 	_SIZE(pipudpFlood)},
{MIB_DOS_PIPICMP_FLOOD, "DOS_PIPICMP_FLOOD",WORD_T, 	_OFFSET(pipicmpFlood), 	_SIZE(pipicmpFlood)},
{MIB_DOS_BLOCK_TIME, 	"DOS_BLOCK_TIME",	WORD_T, 	_OFFSET(blockTime), 	_SIZE(blockTime)},
{MIB_UPNP_ENABLED,		"UPNP_ENABLED", 	BYTE_T,		_OFFSET(upnpEnabled), _SIZE(upnpEnabled)},
{VPN_PASSTHRU_IPSEC_ENABLED,"VPN_PASSTHRU_IPSEC_ENABLED",	BYTE_T, 	_OFFSET(vpnPassthruIPsecEnabled), _SIZE(vpnPassthruIPsecEnabled)},
{VPN_PASSTHRU_PPTP_ENABLED,	"VPN_PASSTHRU_PPTP_ENABLED",	BYTE_T, 	_OFFSET(vpnPassthruPPTPEnabled), _SIZE(vpnPassthruPPTPEnabled)},
{VPN_PASSTHRU_L2TP_ENABLED,	"VPN_PASSTHRU_L2TP_ENABLED",	BYTE_T, 	_OFFSET(vpnPassthruL2TPEnabled), _SIZE(vpnPassthruL2TPEnabled)},
{WEB_WAN_ACCESS_ENABLED,"WEB_WAN_ACCESS_ENABLED",	BYTE_T,		_OFFSET(wanAccessEnabled),	_SIZE(wanAccessEnabled)},
{MIB_WEB_WAN_ACCESS_PORT,	"WEB_WAN_ACCESS_PORT",	WORD_T, 	_OFFSET(wanAccessPort),		_SIZE(wanAccessPort)},
{MIB_PING_WAN_ENABLED,	"PING_WAN_ENABLED",	BYTE_T,		_OFFSET(pingWANEnabled),	_SIZE(pingWANEnabled)},
#ifdef VPN_SUPPORT
{MIB_IPSEC_NATT_ENABLED,	"IPSEC_NATT_ENABLED",	BYTE_T,		_OFFSET(ipsecNattEnabled), _SIZE(ipsecNattEnabled)},
{MIB_IPSECTUNNEL_ENABLED,	"IPSECTUNNEL_ENABLED",	BYTE_T,		_OFFSET(ipsecTunnelEnabled), _SIZE(ipsecTunnelEnabled)},
{MIB_IPSECTUNNEL_NUM,		"IPSECTUNNEL_TBL_NUM",	BYTE_T,		_OFFSET(ipsecTunnelNum ), _SIZE(ipsecTunnelNum )},
{MIB_IPSECTUNNEL,		"IPSECTUNNEL_TBL",	IPSECTUNNEL_ARRAY_T,_OFFSET(ipsecTunnelArray), _SIZE(ipsecTunnelArray)},
{MIB_IPSEC_RSA_FILE,	"IPSEC_RSA_FILE",	BYTE_ARRAY_T,		_OFFSET(ipsecRsaKeyFile), _SIZE(ipsecRsaKeyFile)},
#endif
#ifdef ROUTE_SUPPORT
{MIB_STATICROUTE_ENABLED,	"STATICROUTE_ENABLED",	BYTE_T,		_OFFSET(staticRouteEnabled), _SIZE(staticRouteEnabled)},
{MIB_STATICROUTE_NUM,	"STATICROUTE_TBL_NUM",		BYTE_T,		_OFFSET(staticRouteNum), _SIZE(staticRouteNum)},
{MIB_STATICROUTE,		"STATICROUTE_TBL",	STATICROUTE_ARRAY_T,_OFFSET(staticRouteArray), _SIZE(staticRouteArray)},
{MIB_RIP_ENABLED,		"RIP_ENABLED",		BYTE_T,		_OFFSET(ripEnabled),	_SIZE(ripEnabled)},
{MIB_RIP_LAN_TX,		"RIP_LAN_TX",		BYTE_T,		_OFFSET(ripLanTx),	_SIZE(ripLanTx)},
{MIB_RIP_LAN_RX,		"RIP_LAN_RX",		BYTE_T,		_OFFSET(ripLanRx),	_SIZE(ripLanRx)},
{MIB_RIP_WAN_TX,		"RIP_WAN_TX",		BYTE_T,		_OFFSET(ripWanTx),	_SIZE(ripWanTx)},
{MIB_RIP_WAN_RX,		"RIP_WAN_RX",		BYTE_T,		_OFFSET(ripWanRx),	_SIZE(ripWanRx)},
#endif
#endif // end of HOME_GATEWAY
{MIB_WAN_DNS1,			"DNS1",			IA_T,		_OFFSET(dns1), _SIZE(dns1)},
{MIB_WAN_DNS2,			"DNS2", 		IA_T, 		_OFFSET(dns2), _SIZE(dns2)},
{MIB_WAN_DNS3,			"DNS3", 		IA_T,		_OFFSET(dns3), _SIZE(dns3)},
{MIB_DEVICE_NAME,		"DEVICE_NAME",		STRING_T,	_OFFSET(deviceName), _SIZE(deviceName)},
{MIB_ELAN_MAC_ADDR, 	"ELAN_MAC_ADDR",	BYTE6_T, 	_OFFSET(elanMacAddr), _SIZE(elanMacAddr)},
{MIB_IP_ADDR,			"IP_ADDR", 		IA_T,		_OFFSET(ipAddr), _SIZE(ipAddr)},
{MIB_SUBNET_MASK,		"SUBNET_MASK", 		IA_T,		_OFFSET(subnetMask), _SIZE(subnetMask)},
{MIB_DEFAULT_GATEWAY,		"DEFAULT_GATEWAY",	IA_T,		_OFFSET(defaultGateway), _SIZE(defaultGateway)},
{MIB_DHCP,			"DHCP", 		BYTE_T,		_OFFSET(dhcp), _SIZE(dhcp)},
{MIB_DHCP_CLIENT_START,		"DHCP_CLIENT_START", 	IA_T, 		_OFFSET(dhcpClientStart), _SIZE(dhcpClientStart)},
{MIB_DHCP_CLIENT_END,		"DHCP_CLIENT_END", 	IA_T,		_OFFSET(dhcpClientEnd), _SIZE(dhcpClientEnd)},
{MIB_STP_ENABLED,		"STP_ENABLED",		BYTE_T,		_OFFSET(stpEnabled), _SIZE(stpEnabled)},
{MIB_SUPER_NAME,		"SUPER_NAME", 		STRING_T,	_OFFSET(superName), _SIZE(superName)},
{MIB_SUPER_PASSWORD,		"SUPER_PASSWORD",	STRING_T,	_OFFSET(superPassword), _SIZE(superPassword)},
{MIB_USER_NAME,			"USER_NAME",		STRING_T,	_OFFSET(userName), _SIZE(userName)},
{MIB_USER_PASSWORD,		"USER_PASSWORD",	STRING_T,	_OFFSET(userPassword), _SIZE(userPassword)},
{MIB_LOG_ENABLED,		"SCRLOG_ENABLED",	BYTE_T,		_OFFSET(scrlogEnabled), _SIZE(scrlogEnabled)},
{MIB_AUTO_DISCOVERY_ENABLED,	"AUTODISCOVERY_ENABLED",BYTE_T,		_OFFSET(autoDiscoveryEnabled), _SIZE(autoDiscoveryEnabled)},
{MIB_OP_MODE,			"OP_MODE",		BYTE_T,		_OFFSET(opMode), _SIZE(opMode)},
{MIB_WISP_WAN_ID,		"WISP_WAN_ID",		BYTE_T,		_OFFSET(wispWanId), _SIZE(wispWanId)},
{MIB_SHIPPING_MODEL,	"SHIPPING_MODEL",	BYTE_T, 	_OFFSET(shippingModel), _SIZE(shippingModel)},
{MIB_REMOTELOG_ENABLED,	"REMOTELOG_ENABLED",BYTE_T, 	_OFFSET(rtLogEnabled), 		_SIZE(rtLogEnabled)},
{MIB_REMOTELOG_SERVER,	"REMOTELOG_SERVER",	IA_T, 		_OFFSET(rtLogServer), 		_SIZE(rtLogServer)},
#ifdef TLS_CLIENT
{MIB_CERTROOT_NUM,		"CERTROOT_TBL_NUM",	BYTE_T,		_OFFSET(certRootNum), _SIZE(certRootNum)},
{MIB_CERTROOT,			"CERTROOT_TBL",		CERTROOT_ARRAY_T,_OFFSET(certRootArray), _SIZE(certRootArray)},
{MIB_CERTUSER_NUM,		"CERTUSER_TBL_NUM",	BYTE_T,		_OFFSET(certUserNum), _SIZE(certUserNum)},
{MIB_CERTUSER,			"CERTUSER_TBL",		CERTUSER_ARRAY_T,_OFFSET(certUserArray), _SIZE(certUserArray)},
{MIB_ROOT_IDX,			"ROOT_IDX",		BYTE_T,		_OFFSET(rootIdx), _SIZE(rootIdx)},
{MIB_USER_IDX,			"USER_IDX",		BYTE_T,		_OFFSET(userIdx), _SIZE(userIdx)},
#endif
#if INCLUDE_THIS
{MIB_WATCHDOG_ENABLED,		"WATCHDOG_ENABLED",		BYTE_T,		_OFFSET(watchdogEnabled), _SIZE(watchdogEnabled)},
{MIB_WATCHDOG_INTERVAL,		"WATCHDOG_INTERVAL",	WORD_T,		_OFFSET(watchdogInterval), _SIZE(watchdogInterval)},
{MIB_WATCHDOG_IPADDR,			"WATCHDOG_IPADDR", 		IA_T,		_OFFSET(watchdogIpaddr), _SIZE(watchdogIpaddr)},
#endif
{MIB_MODEL_TYPE,	"MODEL_TYPE",	STRING_T,		_OFFSET(modelType),	_SIZE(modelType)},
#ifdef HOME_GATEWAY // add by Eason, 2005/11/4
{MIB_PPP_AUTH_TYPE,		"PPP_AUTH_TYPE",	BYTE_T,		_OFFSET(pppAuthtype), _SIZE(pppAuthtype)},
{MIB_PPTP_AUTH_TYPE,		"PPTP_AUTH_TYPE",	BYTE_T,		_OFFSET(pptpAuthtype), _SIZE(pptpAuthtype)},
{MIB_L2TP_AUTH_TYPE,		"L2TP_AUTH_TYPE",	BYTE_T,		_OFFSET(l2tpAuthtype), _SIZE(l2tpAuthtype)},
#endif
{0}
};




/*
 * When using flash (set/get/all) command to access the MIB of below table,
 * it needs append a keyword "WLANx_" in ahead of mib name.
 * When access default setting, it needs appened a keyword "DEF_" in front of
 * "WLANx_" keyword.
 */

mib_table_entry_T mib_wlan_table[]={
{MIB_WLAN_MAC_ADDR, 		"WLAN_MAC_ADDR",	BYTE6_T, 	_OFFSET_WLAN(wlanMacAddr), _SIZE_WLAN(wlanMacAddr)},
{MIB_WLAN_SSID,			"SSID", 		STRING_T,	_OFFSET_WLAN(ssid), _SIZE_WLAN(ssid)},
{MIB_WLAN_REG_DOMAIN, 		"REG_DOMAIN",		BYTE_T, 	_OFFSET_WLAN(regDomain), _SIZE_WLAN(regDomain)},
{MIB_WLAN_CHAN_NUM,		"CHANNEL", 		BYTE_T, 	_OFFSET_WLAN(channel), _SIZE_WLAN(channel)},
{MIB_WLAN_WEP,			"WEP", 			BYTE_T,		_OFFSET_WLAN(wep), _SIZE_WLAN(wep)},
{MIB_WLAN_WEP64_KEY1,		"WEP64_KEY1", 		BYTE5_T,	_OFFSET_WLAN(wep64Key1), _SIZE_WLAN(wep64Key1)},
{MIB_WLAN_WEP64_KEY2,		"WEP64_KEY2", 		BYTE5_T,	_OFFSET_WLAN(wep64Key2), _SIZE_WLAN(wep64Key2)},
{MIB_WLAN_WEP64_KEY3,		"WEP64_KEY3", 		BYTE5_T,	_OFFSET_WLAN(wep64Key3), _SIZE_WLAN(wep64Key3)},
{MIB_WLAN_WEP64_KEY4,		"WEP64_KEY4", 		BYTE5_T,	_OFFSET_WLAN(wep64Key4), _SIZE_WLAN(wep64Key4)},
{MIB_WLAN_WEP128_KEY1,		"WEP128_KEY1", 		BYTE13_T,	_OFFSET_WLAN(wep128Key1), _SIZE_WLAN(wep128Key1)},
{MIB_WLAN_WEP128_KEY2,		"WEP128_KEY2", 		BYTE13_T,	_OFFSET_WLAN(wep128Key2), _SIZE_WLAN(wep128Key2)},
{MIB_WLAN_WEP128_KEY3,		"WEP128_KEY3", 		BYTE13_T,	_OFFSET_WLAN(wep128Key3), _SIZE_WLAN(wep128Key3)},
{MIB_WLAN_WEP128_KEY4,		"WEP128_KEY4", 		BYTE13_T,	_OFFSET_WLAN(wep128Key4), _SIZE_WLAN(wep128Key4)},
{MIB_WLAN_WEP_DEFAULT_KEY,	"WEP_DEFAULT_KEY", 	BYTE_T,		_OFFSET_WLAN(wepDefaultKey), _SIZE_WLAN(wepDefaultKey)},
{MIB_WLAN_WEP_KEY_TYPE,		"WEP_KEY_TYPE", 	BYTE_T,		_OFFSET_WLAN(wepKeyType), _SIZE_WLAN(wepKeyType)},
{MIB_WLAN_FRAG_THRESHOLD,	"FRAG_THRESHOLD", 	WORD_T,		_OFFSET_WLAN(fragThreshold), _SIZE_WLAN(fragThreshold)},
{MIB_WLAN_SUPPORTED_RATE,	"SUPPORTED_RATES", 	WORD_T,		_OFFSET_WLAN(supportedRates), _SIZE_WLAN(supportedRates)},
{MIB_WLAN_BEACON_INTERVAL,	"BEACON_INTERVAL", 	WORD_T,		_OFFSET_WLAN(beaconInterval), _SIZE_WLAN(beaconInterval)},
{MIB_WLAN_PREAMBLE_TYPE,	"PREAMBLE_TYPE", 	BYTE_T,		_OFFSET_WLAN(preambleType), _SIZE_WLAN(preambleType)},
{MIB_WLAN_BASIC_RATE,		"BASIC_RATES", 		WORD_T, 	_OFFSET_WLAN(basicRates), _SIZE_WLAN(basicRates)},
{MIB_WLAN_RTS_THRESHOLD,	"RTS_THRESHOLD", 	WORD_T, 	_OFFSET_WLAN(rtsThreshold), _SIZE_WLAN(rtsThreshold)},
{MIB_WLAN_AUTH_TYPE,		"AUTH_TYPE", 		BYTE_T,		_OFFSET_WLAN(authType), _SIZE_WLAN(authType)},
{MIB_WLAN_HIDDEN_SSID,		"HIDDEN_SSID", 		BYTE_T,		_OFFSET_WLAN(hiddenSSID), _SIZE_WLAN(hiddenSSID)},
{MIB_WLAN_DISABLED,		"WLAN_DISABLED", 	BYTE_T,		_OFFSET_WLAN(wlanDisabled), _SIZE_WLAN(wlanDisabled)},
{MIB_WLAN_INACTIVITY_TIME,	"INACTIVITY_TIME",	DWORD_T,	_OFFSET_WLAN(inactivityTime), _SIZE_WLAN(inactivityTime)},
{MIB_WLAN_RATE_ADAPTIVE_ENABLED,"RATE_ADAPTIVE_ENABLED",BYTE_T,		_OFFSET_WLAN(rateAdaptiveEnabled), _SIZE_WLAN(rateAdaptiveEnabled)},
{MIB_WLAN_DTIM_PERIOD,		"DTIM_PERIOD", 		BYTE_T,		_OFFSET_WLAN(dtimPeriod), _SIZE_WLAN(dtimPeriod)},
{MIB_WLAN_MODE,			"MODE", 		BYTE_T,		_OFFSET_WLAN(wlanMode), _SIZE_WLAN(wlanMode)},
{MIB_WLAN_NETWORK_TYPE,		"NETWORK_TYPE",		BYTE_T,		_OFFSET_WLAN(networkType), _SIZE_WLAN(networkType)},
{MIB_WLAN_IAPP_DISABLED,	"IAPP_DISABLED",	BYTE_T,		_OFFSET_WLAN(iappDisabled), _SIZE_WLAN(iappDisabled)},
{MIB_WLAN_PROTECTION_DISABLED,	"PROTECTION_DISABLED",	BYTE_T,		_OFFSET_WLAN(protectionDisabled), _SIZE_WLAN(protectionDisabled)},
{MIB_WLAN_DEFAULT_SSID,		"DEFAULT_SSID", 	STRING_T,	_OFFSET_WLAN(defaultSsid), _SIZE_WLAN(defaultSsid)},
{MIB_WLAN_WDS_ENABLED,		"WDS_ENABLED",		BYTE_T,		_OFFSET_WLAN(wdsEnabled), _SIZE_WLAN(wdsEnabled)},
{MIB_WLAN_WDS_NUM,		"WDS_NUM",		BYTE_T,		_OFFSET_WLAN(wdsNum), _SIZE_WLAN(wdsNum)},
{MIB_WLAN_WDS,			"WDS",			WDS_ARRAY_T,	_OFFSET_WLAN(wdsArray), _SIZE_WLAN(wdsArray)},
{MIB_WLAN_WDS_ENCRYPT,		"WDS_ENCRYPT", 		BYTE_T,		_OFFSET_WLAN(wdsEncrypt), _SIZE_WLAN(wdsEncrypt)},
{MIB_WLAN_WDS_WEP_FORMAT,	"WDS_WEP_FORMAT", 	BYTE_T,		_OFFSET_WLAN(wdsWepKeyFormat), _SIZE_WLAN(wdsWepKeyFormat)},
{MIB_WLAN_WDS_WEP_KEY,		"WDS_WEP_KEY",		STRING_T,	_OFFSET_WLAN(wdsWepKey), _SIZE_WLAN(wdsWepKey)},
{MIB_WLAN_WDS_PSK_FORMAT,	"WDS_PSK_FORMAT", 	BYTE_T,		_OFFSET_WLAN(wdsPskFormat), _SIZE_WLAN(wdsPskFormat)},
{MIB_WLAN_WDS_PSK,		"WDS_PSK",		STRING_T,	_OFFSET_WLAN(wdsPsk), _SIZE_WLAN(wdsPsk)},
{MIB_WLAN_ENCRYPT,		"ENCRYPT",		BYTE_T,		_OFFSET_WLAN(encrypt), _SIZE_WLAN(encrypt)},
{MIB_WLAN_ENABLE_SUPP_NONWPA,	"ENABLE_SUPP_NONWPA", 	BYTE_T,		_OFFSET_WLAN(enableSuppNonWpa), _SIZE_WLAN(enableSuppNonWpa)},
{MIB_WLAN_SUPP_NONWPA,		"SUPP_NONWPA", 		BYTE_T,		_OFFSET_WLAN(suppNonWpa), _SIZE_WLAN(suppNonWpa)},
{MIB_WLAN_WPA_AUTH,		"WPA_AUTH", 		BYTE_T,		_OFFSET_WLAN(wpaAuth), _SIZE_WLAN(wpaAuth)},
{MIB_WLAN_WPA_CIPHER_SUITE,	"WPA_CIPHER_SUITE",	BYTE_T,		_OFFSET_WLAN(wpaCipher), _SIZE_WLAN(wpaCipher)},
{MIB_WLAN_WPA_PSK,		"WPA_PSK",		STRING_T,	_OFFSET_WLAN(wpaPSK), _SIZE_WLAN(wpaPSK)},
{MIB_WLAN_WPA_GROUP_REKEY_TIME,	"WPA_GROUP_REKEY_TIME",	DWORD_T,	_OFFSET_WLAN(wpaGroupRekeyTime), _SIZE_WLAN(wpaGroupRekeyTime)},
{MIB_WLAN_ENABLE_MAC_AUTH,	"MAC_AUTH_ENABLED",     BYTE_T,         _OFFSET_WLAN(macAuthEnabled), _SIZE_WLAN(macAuthEnabled)},
{MIB_WLAN_RS_IP,		"RS_IP",		IA_T,		_OFFSET_WLAN(rsIpAddr), _SIZE_WLAN(rsIpAddr)},
{MIB_WLAN_RS_PORT,		"RS_PORT",		WORD_T,		_OFFSET_WLAN(rsPort), _SIZE_WLAN(rsPort)},
{MIB_WLAN_RS_PASSWORD,		"RS_PASSWORD",		STRING_T,	_OFFSET_WLAN(rsPassword), _SIZE_WLAN(rsPassword)},
{MIB_WLAN_RS_RETRY,		"RS_MAXRETRY",		BYTE_T,		_OFFSET_WLAN(rsMaxRetry), _SIZE_WLAN(rsMaxRetry)},
{MIB_WLAN_RS_INTERVAL_TIME,	"RS_INTERVAL_TIME",	WORD_T,		_OFFSET_WLAN(rsIntervalTime), _SIZE_WLAN(rsIntervalTime)},
{MIB_WLAN_ACCOUNT_RS_ENABLED,	"ACCOUNT_RS_ENABLED", 	BYTE_T,		_OFFSET_WLAN(accountRsEnabled), _SIZE_WLAN(accountRsEnabled)},
{MIB_WLAN_ACCOUNT_RS_IP,	"ACCOUNT_RS_IP",	IA_T,		_OFFSET_WLAN(accountRsIpAddr), _SIZE_WLAN(accountRsIpAddr)},
{MIB_WLAN_ACCOUNT_RS_PORT,	"ACCOUNT_RS_PORT",	WORD_T,		_OFFSET_WLAN(accountRsPort), _SIZE_WLAN(accountRsPort)},
{MIB_WLAN_ACCOUNT_RS_PASSWORD,	"ACCOUNT_RS_PASSWORD",	STRING_T,	_OFFSET_WLAN(accountRsPassword), _SIZE_WLAN(accountRsPassword)},
{MIB_WLAN_ACCOUNT_UPDATE_ENABLED,"ACCOUNT_RS_UPDATE_ENABLED",BYTE_T,	_OFFSET_WLAN(accountRsUpdateEnabled), _SIZE_WLAN(accountRsUpdateEnabled)},
{MIB_WLAN_ACCOUNT_UPDATE_DELAY,"ACCOUNT_RS_UPDATE_DELAY",WORD_T,	_OFFSET_WLAN(accountRsUpdateDelay), _SIZE_WLAN(accountRsUpdateDelay)},
{MIB_WLAN_ACCOUNT_RS_RETRY,	"ACCOUNT_RS_MAXRETRY",	BYTE_T,		_OFFSET_WLAN(accountRsMaxRetry), _SIZE_WLAN(accountRsMaxRetry)},
{MIB_WLAN_ACCOUNT_RS_INTERVAL_TIME, "ACCOUNT_RS_INTERVAL_TIME",	WORD_T,	_OFFSET_WLAN(accountRsIntervalTime), _SIZE_WLAN(accountRsIntervalTime)},
{MIB_WLAN_ENABLE_1X,		"ENABLE_1X", 		BYTE_T,		_OFFSET_WLAN(enable1X), _SIZE_WLAN(enable1X)},
{MIB_WLAN_WPA_PSK_FORMAT,	"PSK_FORMAT", 		BYTE_T,		_OFFSET_WLAN(wpaPSKFormat), _SIZE_WLAN(wpaPSKFormat)},
{MIB_WLAN_WPA2_PRE_AUTH,	"WPA2_PRE_AUTH",	BYTE_T,         _OFFSET_WLAN(wpa2PreAuth), _SIZE_WLAN(wpa2PreAuth)},
{MIB_WLAN_WPA2_CIPHER_SUITE,	"WPA2_CIPHER_SUITE",	BYTE_T,		_OFFSET_WLAN(wpa2Cipher), _SIZE_WLAN(wpa2Cipher)},
{MIB_WLAN_AC_NUM, 		"MACAC_NUM",		BYTE_T,		_OFFSET_WLAN(acNum), _SIZE_WLAN(acNum)},
{MIB_WLAN_AC_ENABLED,		"MACAC_ENABLED",	BYTE_T,		_OFFSET_WLAN(acEnabled), _SIZE_WLAN(acEnabled)},
{MIB_WLAN_AC_ADDR,		"MACAC_ADDR",		WLAC_ARRAY_T,  _OFFSET_WLAN(acAddrArray), _SIZE_WLAN(acAddrArray)},
{MIB_WLAN_BLOCK_RELAY,		"BLOCK_RELAY",		BYTE_T,		_OFFSET_WLAN(blockRelay), _SIZE_WLAN(blockRelay)},
{MIB_WLAN_NAT25_MAC_CLONE,	"MACCLONE_ENABLED",	BYTE_T,		_OFFSET_WLAN(maccloneEnabled), _SIZE_WLAN(maccloneEnabled)},
{MIB_WLAN_BAND,			"BAND",			BYTE_T,		_OFFSET_WLAN(wlanBand), _SIZE_WLAN(wlanBand)},
{MIB_WLAN_FIX_RATE,		"FIX_RATE",		WORD_T,		_OFFSET_WLAN(fixedTxRate), _SIZE_WLAN(fixedTxRate)},
{MIB_WLAN_WIFI_SPECIFIC,    "WIFI_SPECIFIC",	BYTE_T, 	_OFFSET_WLAN(wifiSpecific), _SIZE_WLAN(wifiSpecific)},
#ifdef WLAN_EASY_CONFIG
{MIB_WLAN_EASYCFG_ENABLED,	"EASYCFG_ENABLED",	BYTE_T,		_OFFSET_WLAN(acfEnabled), _SIZE_WLAN(acfEnabled)},
{MIB_WLAN_EASYCFG_MODE,		"EASYCFG_MODE",		BYTE_T,		_OFFSET_WLAN(acfMode), _SIZE_WLAN(acfMode)},
{MIB_WLAN_EASYCFG_SSID,		"EASYCFG_SSID",		STRING_T,	_OFFSET_WLAN(acfSSID), _SIZE_WLAN(acfSSID)},
{MIB_WLAN_EASYCFG_KEY,		"EASYCFG_KEY",		STRING_T,	_OFFSET_WLAN(acfKey), _SIZE_WLAN(acfKey)},
{MIB_WLAN_EASYCFG_DIGEST,	"EASYCFG_DIGEST",	STRING_T,	_OFFSET_WLAN(acfDigest), _SIZE_WLAN(acfDigest)},
{MIB_WLAN_EASYCFG_ALG_REQ,	"EASYCFG_ALG_REQ",	BYTE_T,		_OFFSET_WLAN(acfAlgReq), _SIZE_WLAN(acfAlgReq)},
{MIB_WLAN_EASYCFG_ALG_SUPP,	"EASYCFG_ALG_SUPP",	BYTE_T,		_OFFSET_WLAN(acfAlgSupp), _SIZE_WLAN(acfAlgSupp)},
{MIB_WLAN_EASYCFG_ROLE,		"EASYCFG_ROLE",		BYTE_T,		_OFFSET_WLAN(acfRole), _SIZE_WLAN(acfRole)},
{MIB_WLAN_EASYCFG_SCAN_SSID,	"EASYCFG_SCAN_SSID",	STRING_T,	_OFFSET_WLAN(acfScanSSID), _SIZE_WLAN(acfScanSSID)},
{MIB_WLAN_EASYCFG_WLAN_MODE,	"EASYCFG_WLAN_MODE",	BYTE_T,		_OFFSET_WLAN(acfWlanMode), _SIZE_WLAN(acfWlanMode)},
#endif
#if INCLUDE_THIS
{MIB_WLAN_ALIAS,			"ALIAS", 		STRING_T,	_OFFSET_WLAN(alias), _SIZE_WLAN(alias)},
{MIB_WLAN_TX_POWER, 		"TX_POWER_LEVEL",		BYTE_T, 	_OFFSET_WLAN(txPower), _SIZE_WLAN(txPower)},
#endif
{0}
};
