/*
 * security.c -- Security handler
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 */

/******************************** Description *********************************/

/*
 *	This module provides a basic security policy.
 */

/********************************* Includes ***********************************/

#include	"wsIntrn.h"
#include	"um.h"
#ifdef DIGEST_ACCESS_SUPPORT
#include	"websda.h"
#endif

/********************************** Defines ***********************************/
/*
 *	The following #defines change the behaviour of security in the absence
 *	of User Management.
 *	Note that use of User management functions require prior calling of
 *	umInit() to behave correctly
 */

#ifndef USER_MANAGEMENT_SUPPORT
#define umGetAccessMethodForURL(url) AM_FULL
#define umUserExists(userid) 0
#define umUserCanAccessURL(userid, url) 1
#define umGetUserPassword(userid) websGetPassword()
#define umGetAccessLimitSecure(accessLimit) 0
#define umGetAccessLimit(url) NULL
#endif

/******************************** Local Data **********************************/

static char_t	websPassword[WEBS_MAX_PASS];	/* Access password (decoded) */
// marked by david
#if 0
#ifdef _DEBUG
static int		debugSecurity = 1;
#else
static int		debugSecurity = 0;
#endif
#endif
//sc_yang
#ifdef SAFECOM
	#define LOGIN_TIMEOUT (60000*3) //(3 min)
#else
	#define LOGIN_TIMEOUT (60000*5) //(5 min)
#endif
//modified for web access limit by Richard H., 2006/3/23
#if (defined ZYXELRUS) || (defined SAFECOM)
	extern time_t last_access_time;
#else
	static time_t last_access_time = {0};
#endif

int login=0;
extern int logout;
/*********************************** Code *************************************/
/*
 *	Determine if this request should be honored
 */

int websSecurityHandler(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,
						char_t *url, char_t *path, char_t *query)
{
	char_t			*type, *userid, *password, *accessLimit;
	int				flags, nRet;
	accessMeth_t	am;

	a_assert(websValid(wp));
	a_assert(url && *url);
	a_assert(path && *path);
/*
 *	Get the critical request details
 */
	type = websGetRequestType(wp);
	password = websGetRequestPassword(wp);
	userid = websGetRequestUserName(wp);
	flags = websGetRequestFlags(wp);

	//modified for web access limit by Richard H., 2006/3/23
#if (defined ZYXELRUS) || (defined SAFECOM)
	char_t * checkuserpass = umGetUserPassword("admin");
	if((time(0)-last_access_time)*1000 > LOGIN_TIMEOUT) {
    	FILE *plogin;
        system("echo 0 > /var/LOGIN_STATUS");
        plogin = fopen("/var/LOGIN_STATUS","r");
        if( plogin == NULL )
        	printf("can not open this file\n");
        fclose(plogin);
        plogin=NULL;
        login=0;
        logout=0;
	}
#endif

/*
 *	Get the access limit for the URL.  Exit if none found.
 */
	accessLimit = umGetAccessLimit(path);
	if (accessLimit == NULL) {
		return 0;
	}

/*
 *	Check to see if URL must be encrypted
 */
#ifdef WEBS_SSL_SUPPORT
	nRet = umGetAccessLimitSecure(accessLimit);
	if (nRet && ((flags | WEBS_SECURE) == 0)) {
		websStats.access++;
		websError(wp, 200, T("Access Denied\nSecure access is required."));
		trace(3, T("SEC: Non-secure access attempted on <%s>\n"), path);
		return 1;
	}
#endif

/*
 *	Get the access limit for the URL
 */
	am = umGetAccessMethodForURL(accessLimit);
	nRet = 0;

// for debug, david /////////////////////////////////
#if 0
	if ((flags & WEBS_LOCAL_REQUEST) && (debugSecurity == 0)) {
/*
 *		Local access is always allowed (defeat when debugging)
 */
	} else if (am == AM_NONE) {
#endif
	if (am == AM_NONE) {

/////////////////////////////////////////////////////

/*
 *		URL is supposed to be hidden!  Make like it wasn't found.
 */
		websStats.access++;
		websError(wp, 400, T("Page Not Found"));
		last_access_time=time(0);
		nRet = 1;
	} else 	if (userid && *userid) {
		if (!umUserExists(userid)) {
			websStats.access++;
// for debug
//			websError(wp, 200, T("Access Denied\nUnknown User"));
			websError(wp, 401, T("Access Denied\nUnknown User"));
			last_access_time=time(0);

			trace(3, T("SEC: Unknown user <%s> attempted to access <%s>\n"),
			userid, path);
			nRet = 1;
		} else if (!umUserCanAccessURL(userid, accessLimit)) {
			websStats.access++;
			websError(wp, 403, T("Access Denied\nProhibited User"));
			last_access_time=time(0);
			nRet = 1;
		//} else if (password && * password){
		//changed by Richard L. then make sure password can be set as NULL.
		} else if (password){
			char_t * userpass = umGetUserPassword(userid);
			if (userpass) {
				if (gstrcmp(password, userpass) != 0) {
					websStats.access++;
// for debug
//					websError(wp, 200, T("Access Denied\nWrong Password"));
					websError(wp, 401, T("Access Denied\nWrong Password"));
					trace(3, T("SEC: Password fail for user <%s>")
								T("attempt to access <%s>\n"), userid, path);
					last_access_time=time(0);
					nRet = 1;
				}
// marked by Eason, 2006/3/22
#if 0
				else if((login && ((time(0) - last_access_time)*1000 > LOGIN_TIMEOUT)) || logout){
					websError(wp, 401, T("Access Timeout\nPlease login again"));
					login=0;
					logout=0;
					nRet = 1;
				}
				else {
					last_access_time = time(0);
					//login=1;
#else
				// modified by Eason, 2006/3/22
				else if(logout) {
					#if (defined ZYXELRUS) || (defined SAFECOM)
    					FILE *fLogin;
    					system("echo 0 > /var/LOGIN_STATUS");
    					fLogin = fopen("/var/LOGIN_STATUS","r");
    					if( fLogin == NULL )
    						printf("can not open this file\n");
    					fclose(fLogin);
    					fLogin=NULL;
					#endif
					login=0;
					logout=0;
					nRet=1;
					last_access_time=time(0);
					if((time(0)-last_access_time)*1000 > LOGIN_TIMEOUT) {
						websError(wp, 401, T("Access1 Timeout\nPlease login again"));
					}
				}
				else {
					if((int)last_access_time == 0)
						last_access_time=time(0);
					if((time(0)-last_access_time)*1000 > LOGIN_TIMEOUT) {
						websError(wp, 401, T("Access2 Timeout\nPlease login again"));
					}
					last_access_time = time(0);
					//login=1;
#endif
				}
				bfree (B_L, userpass);
			}
#ifdef DIGEST_ACCESS_SUPPORT
		} else if (flags & WEBS_AUTH_DIGEST) {

		char_t *digestCalc;

/*
 *			Check digest for equivalence
 */
			wp->password = umGetUserPassword(userid);

			a_assert(wp->digest);
			a_assert(wp->nonce);
			a_assert(wp->password);

			digestCalc = websCalcDigest(wp);
			a_assert(digestCalc);

			if (gstrcmp(wp->digest, digestCalc) != 0) {
				websStats.access++;
				websError(wp, 200, T("Access Denied\nWrong Password"));
				last_access_time=time(0);
				nRet = 1;
			}

			bfree (B_L, digestCalc);
#endif
		} else {
/*
 *			No password has been specified
 */
#ifdef DIGEST_ACCESS_SUPPORT
			if (am == AM_DIGEST) {
				wp->flags |= WEBS_AUTH_DIGEST;
			}
#endif
#if (defined ZYXELRUS) || (defined SAFECOM)
			if((checkuserpass[0]=='\0') && (!logout)) {
				last_access_time=time(0);
				nRet = 0;
			}
			else if(logout) {
				FILE *fplogin;
				system("echo 0 > /var/LOGIN_STATUS");
				fplogin = fopen("/var/LOGIN_STATUS","r");
				if( fplogin == NULL )
					printf("can not open this file\n");
				fclose(fplogin);
				fplogin=NULL;
				login=0;
				logout=0;
				nRet=1;
			}
			else{
				websStats.errors++;
				websError(wp, 401,T("Access to this document requires a password"));
				last_access_time=time(0);
				nRet = 1;
			}
			bfree(B_L, checkuserpass);
#else
			websStats.errors++;
			websError(wp, 401,T("Access to this document requires a password"));
			last_access_time=time(0);
			nRet = 1;
#endif
		}
	} else if (am != AM_FULL) {
/*
 *		This will cause the browser to display a password / username
 *		dialog
 */
#ifdef DIGEST_ACCESS_SUPPORT
		if (am == AM_DIGEST) {
			wp->flags |= WEBS_AUTH_DIGEST;
		}
#endif
		websStats.errors++;
		websError(wp, 401, T("Access to this document requires a User ID"));
		nRet = 1;
	}

	bfree(B_L, accessLimit);

	return nRet;
}

/******************************************************************************/
/*
 *	Delete the default security handler
 */
#ifndef DOWN_SIZE
void websSecurityDelete()
{
	websUrlHandlerDelete(websSecurityHandler);
}
#endif

/******************************************************************************/
/*
 *	Store the new password, expect a decoded password. Store in websPassword in
 *	the decoded form.
 */

void websSetPassword(char_t *password)
{
	a_assert(password);

	gstrncpy(websPassword, password, TSZ(websPassword));
}

/******************************************************************************/
/*
 *	Get password, return the decoded form
 */

char_t *websGetPassword()
{
	return bstrdup(B_L, websPassword);
}

/******************************************************************************/

