/*
 * FILE NAME rtl_gpio.c
 *
 * BRIEF MODULE DESCRIPTION
 *  GPIO For Flash Reload Default
 *
 *  Author: jimmylin@realtek.com.tw
 *
 * Copyright 2005 Realtek Semiconductor Corp.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE	LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/miscdevice.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/reboot.h>
#include <linux/kmod.h>
#include <linux/proc_fs.h>

#define AUTO_CONFIG


#ifdef CONFIG_RTL_EB8186
	#define RTL_GPIO_PABDIR		0xbd010124
	#define RTL_GPIO_PABDATA	0xbd010120
	#define RESET_BTN_PIN		0
	#define AUTOCFG_BTN_PIN		1
	#define GPIO_A2_PIN			2	//[gw]green, power/reset/EZ-config; [ap]errStatus/booting/EZ-config
	#define GPIO_A3_PIN			3	//[gw]amber, power/reset/EZ-config; [ap]wep/wpa/wpa2(mixed)
	//added for AP mode by Richard H.,2005/4/11 20:37
	#ifdef CONFIG_RTL8186_AP
		#define GPIO_A7_PIN		7	//[ap]wireless access control(mac control)
		#define GPIO_A8_PIN		8	//[ap]wireless wds
	#endif

	#define AUTOCFG_LED_OFF		0
	#define AUTOCFG_LED_ON		1
	#define AUTOCFG_LED_BLINK	2
#else
	#define RTL_GPIO_PABDIR		0xbd010040
	#define RTL_GPIO_PABDATA	0xbd010044
	#define RTL_WLAN_CR9346		0xbd400050
	#define RTL_WLAN_CONFIG0	0xbd400051
	#define RTL_WLAN_PSR		0xbd40005e
	#define RESET_BTN_PIN		10
#endif

#define PROBE_NULL		0
#define PROBE_ACTIVE	1
#define PROBE_RESET		2
#define PROBE_RELOAD	3
#define RTL_R32(addr)		(*(volatile unsigned long *)(addr))
#define RTL_W32(addr, l)	((*(volatile unsigned long*)(addr)) = (l))
#define RTL_R8(addr)		(*(volatile unsigned char*)(addr))
#define RTL_W8(addr, l)		((*(volatile unsigned char*)(addr)) = (l))

//#define  GPIO_DEBUG
#ifdef GPIO_DEBUG
/* note: prints function name for you */
#  define DPRINTK(fmt, args...) printk("%s: " fmt, __FUNCTION__ , ## args)
#else
#  define DPRINTK(fmt, args...)
#endif

static struct timer_list probe_timer;
static unsigned int    probe_counter;
static unsigned int    probe_state;

//realtek 20050624
static char default_flag='0';

#ifndef CONFIG_RTL_EB8186
unsigned int led0enable;
#endif


#if defined(CONFIG_RTL_EB8186) && defined(AUTO_CONFIG)
static unsigned int		AutoCfg_LED_Blink;
static unsigned int		AutoCfg_LED_Toggle;

void autoconfig_gpio_init(void)
{
	// Set GPIOA pin 1 as input pin for auto config button
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) & (~(1 << AUTOCFG_BTN_PIN))));

	// Set GPIOA pin as output pin for auto config led by Richard H.,2005/4/20 22:30
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A2_PIN))));
	#ifdef CONFIG_RTL8186_GW_8M
		RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A3_PIN))));
	#endif

	// turn off auto config led in the beginning by Richad H.,2005/4/20 22:34
	RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
	#ifdef CONFIG_RTL8186_GW_8M
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
	#endif
}


void autoconfig_gpio_off(void)
{
	RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
	#ifdef CONFIG_RTL8186_GW_8M
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
	#endif
	AutoCfg_LED_Blink = 0;
}


void autoconfig_gpio_on(void)
{
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A2_PIN))));
	#ifdef CONFIG_RTL8186_GW_8M
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
	#endif
	AutoCfg_LED_Blink = 0;
}


void autoconfig_gpio_blink(void)
{
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A2_PIN))));
	#ifdef CONFIG_RTL8186_GW_8M
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
	#endif
	AutoCfg_LED_Blink = 1;
	AutoCfg_LED_Toggle = 1;
}
#endif

static int reset_flash_default(void *data)
{
	char *argv[3], *envp[1] = {NULL};
	int i = 0;
	int reset_default=(int)data;

	argv[i++] = "/bin/flash";
	argv[i++] = "reset";
	argv[i] = NULL;

	if(reset_default)
	{
		printk("Going to Reload Default\n");
		if (call_usermodehelper(argv[0], argv, envp,1))
			printk("failed to Reset to default\n");
	}
	machine_restart(0);
	return 0;
}

#define sec_5 (5*16)
#define boot_ready_time 400
static unsigned long delay_reboot;  //maxchang: delay_reboot=1 if reboot operation during boot
static unsigned long delay_facdef;  //maxchang: delay_facdef=1 if factory default operation during boot
static unsigned long timer_count;   //maxchang: start ounting since rtl_gpio_timer is loaded

static void rtl_gpio_timer(unsigned long data)
{
    //printk("tt_%ld\n", timer_count);
    if(timer_count>=boot_ready_time){
	    timer_count = boot_ready_time;
/* maxchang 2006/2/22 10:16
	    if(delay_facdef == 1){
	        del_timer_sync(&probe_timer);
	        delay_facdef = 0;
	        //realtek 20050624
	        default_flag='1';
	        //kernel_thread(reset_flash_default, (void *)1, SIGCHLD);
	        goto stop_here;
	    }else if(delay_reboot == 1){
	        del_timer_sync(&probe_timer);
	        delay_reboot = 0;
	        //realtek 20050624
	        kill_proc(1,SIGTERM,1);
	        //kernel_thread(reset_flash_default, 0, SIGCHLD);
	        goto stop_here;
        }
*/
	}
    timer_count++;

	unsigned int pressed=1;

#ifdef CONFIG_RTL_EB8186
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A2_PIN))));
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A3_PIN))));
	//added for AP mode by Richard H.,2005/4/11 20:37
	#ifdef CONFIG_RTL8186_AP
		RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A7_PIN))));
		RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A8_PIN))));
		//printk("\n[Richard] [AP] RTL_GPIO_PABDATA=0x%08x\n\n", RTL_R32(RTL_GPIO_PABDATA));
	#endif
#endif

	if ((RTL_R32(RTL_GPIO_PABDATA) & (1 << RESET_BTN_PIN))){
		pressed = 0;
#ifdef CONFIG_RTL_EB8186
		//turn off LED0
		//RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | ((1 << GPIO_A2_PIN))));
#endif
	}
	else
	{
		DPRINTK("Key pressed %d!\n", probe_counter+1);
	}

	if (probe_state == PROBE_NULL)
	{
		if (pressed)
		{
			probe_state = PROBE_ACTIVE;
			probe_counter++;
		}
		else
			probe_counter = 0;
	}
	else if (probe_state == PROBE_ACTIVE)
	{
		if (pressed) //press button
		{
			probe_counter++;
			if (probe_counter <= sec_5)
			{
#ifndef CONFIG_RTL_EB8186
				led0enable=1;
				// turn on LED0
				RTL_W8(RTL_WLAN_CR9346, 0xc0);
				RTL_W8(RTL_WLAN_CONFIG0, 0x10);
				RTL_W8(RTL_WLAN_PSR,(RTL_R8(RTL_WLAN_PSR)&0xEF));
#else
				DPRINTK("0-5 turn on led\n");
				//turn on LED0
        #ifndef ZYXELRUS
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
				#ifdef CONFIG_RTL8186_GW_8M
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
				#endif

				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
				#ifdef CONFIG_RTL8186_GW_8M
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | ((1 << GPIO_A3_PIN))));
				#endif
		#endif
#endif

			}
			else if (probe_counter > sec_5)
			{
#ifndef CONFIG_RTL_EB8186
				led0enable=1;
				RTL_W8(RTL_WLAN_CR9346, 0xc0);
				RTL_W8(RTL_WLAN_CONFIG0, 0x10);
				// sparkling LED0
				if (probe_counter & 1)
					RTL_W8(RTL_WLAN_PSR, (RTL_R8(RTL_WLAN_PSR)|0x10));
				else
					RTL_W8(RTL_WLAN_PSR, (RTL_R8(RTL_WLAN_PSR)&0xEF));
#else
				// sparkling LED0
				DPRINTK(">5 \n");
				if (probe_counter & 1)
				{
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
					#ifdef CONFIG_RTL8186_GW_8M
						RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | ((1 << GPIO_A3_PIN))));
					#endif
				}
				else
				{
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
					#ifdef CONFIG_RTL8186_GW_8M
						RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
					#endif
				}
#endif

			}
		}
		else //release button
		{
			if (probe_counter >= sec_5){
	    		if(timer_count >=boot_ready_time){
		            del_timer_sync(&probe_timer);
#ifndef CONFIG_RTL_EB8186
				    led0enable=1;
#endif
				    //reload default
			        //realtek 20050624
				    default_flag='1';
				    //kernel_thread(reset_flash_default, (void *)1, SIGCHLD);
	    	        return;
		        }else{
		            probe_state = PROBE_NULL;
		            pressed=0;
		            //delay_facdef = 1;
		            mod_timer(&probe_timer, jiffies + HZ / 16);	//reload gpio_reset timer
		        }
			}else{
#ifndef ZYXELRUS
			    if(timer_count >=boot_ready_time){
			        del_timer_sync(&probe_timer);

#ifndef CONFIG_RTL_EB8186
				    RTL_W8(RTL_WLAN_PSR , (RTL_R8(RTL_WLAN_PSR)|0x10));
				    RTL_W8(RTL_WLAN_CONFIG0, (RTL_R8(RTL_WLAN_CONFIG0)&(~0x10)));
#endif
				    DPRINTK("0-5 reset\n");
				    //realtek 20050624
				    kill_proc(1,SIGTERM,1);
				    //kernel_thread(reset_flash_default, 0, SIGCHLD);
				    return;
                }else{
                    probe_state = PROBE_NULL;
		            pressed=0;
                    //delay_reboot = 1;
		            mod_timer(&probe_timer, jiffies + HZ / 16);	//reload gpio_reset timer
                }
#endif
			}
		}
	}//probe_state == PROBE_ACTIVE

#if defined(CONFIG_RTL_EB8186) && defined(AUTO_CONFIG)
	if (AutoCfg_LED_Blink)
	{
		if (AutoCfg_LED_Toggle) {
			RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A2_PIN))));
			#ifdef CONFIG_RTL8186_GW_8M
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << GPIO_A3_PIN)));
			#endif
		}
		else {
			RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << GPIO_A2_PIN)));
			#ifdef CONFIG_RTL8186_GW_8M
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << GPIO_A3_PIN))));
			#endif
		}
		AutoCfg_LED_Toggle = AutoCfg_LED_Toggle? 0 : 1;
	}
#endif

	//maxchang
	mod_timer(&probe_timer, jiffies + HZ/16);

stop_here:
}


#ifdef AUTO_CONFIG
static int read_proc(char *page, char **start, off_t off,
				int count, int *eof, void *data)
{
	int len;
	char flag;

	if (RTL_R32(RTL_GPIO_PABDATA) & (1 << AUTOCFG_BTN_PIN))
		flag = '0';
	else
		flag = '1';
	len = sprintf(page, "%c\n", flag);


	if (len <= off+count) *eof = 1;
	*start = page + off;
	len -= off;
	if (len > count) len = count;
	if (len < 0) len = 0;
	return len;
}


static int write_proc(struct file *file, const char *buffer,
				unsigned long count, void *data)
{
	char flag;

	if (count < 2)
		return -EFAULT;

	DPRINTK("file: %08x, buffer: %s, count: %lu, data: %08x\n",
		(unsigned int)file, buffer, count, (unsigned int)data);
	if (buffer && !copy_from_user(&flag, buffer, 1)) {
		if (flag == 'E')
			autoconfig_gpio_init();
		else if (flag == '0')
			autoconfig_gpio_off();
		else if (flag == '1')
			autoconfig_gpio_on();
		else if (flag == '2')
			autoconfig_gpio_blink();
		else
			{}

		return count;
	}
	else
		return -EFAULT;
}
#endif

//realtek 20050624
static int default_read_proc(char *page, char **start, off_t off,
                     int count, int *eof, void *data)
{
      int len;
      len = sprintf(page, "%c\n", default_flag);
      if (len <= off+count) *eof = 1;
          *start = page + off;
      len -= off;
      if (len>count) len = count;
      if (len<0) len = 0;
          return len;

}

static int default_write_proc(struct file *file, const char *buffer,
                      unsigned long count, void *data)
{
      if (count < 2)
         return -EFAULT;
      if (buffer && !copy_from_user(&default_flag, buffer, 1)) {
         return count;
         }
      return -EFAULT;
}


int __init rtl_gpio_init(void)
{
#ifdef AUTO_CONFIG
	struct proc_dir_entry *res=NULL;
#endif

	printk("Realtek GPIO Driver for Flash Reload Default\n");

	// Set GPIOA pin 10(8181)/0(8186) as input pin for reset button
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) & (~(1 << RESET_BTN_PIN))));
#ifdef CONFIG_RTL_EB8186
	// Set GPIOA ping 2 as output pin for reset led
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << GPIO_A2_PIN))));
#endif

#ifdef AUTO_CONFIG
	res = create_proc_entry("gpio", 0, NULL);
	if (res) {
		res->read_proc = read_proc;
		res->write_proc = write_proc;
	}
	else {
		printk("Realtek GPIO Driver, create proc failed!\n");
	}
#endif
	//realtek 20050624
	res = create_proc_entry("load_default", 0, NULL);
    if (res) {
        res->read_proc=default_read_proc;
        res->write_proc=default_write_proc;
    }

    delay_reboot = 0;
    delay_facdef = 0;
	timer_count = 0;

	init_timer (&probe_timer);
	probe_counter = 0;
	probe_state = PROBE_NULL;
	//maxchang
	//probe_timer.expires = jiffies + 100;
	probe_timer.expires = jiffies + HZ/16;
	probe_timer.data = (unsigned long)NULL;
	probe_timer.function = &rtl_gpio_timer;

	//maxchang
	//mod_timer(&probe_timer, jiffies + 100);
	add_timer(&probe_timer);

	return 0;
}


static void __exit rtl_gpio_exit(void)
{
	printk("Unload Realtek GPIO Driver \n");
	del_timer_sync(&probe_timer);
}


module_exit(rtl_gpio_exit);
module_init(rtl_gpio_init);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("GPIO driver for Reload default");
