
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <setjmp.h>
#include <signal.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "cmdedit.h"
#include "busybox.h"

#define HOME_GATEWAY

// apmib header files
#include "../../goahead-2.1.1/LINUX.newnew/apmib.h"
#include "../../goahead-2.1.1/LINUX.newnew/mibtbl.h"
#include "../../goahead-2.1.1/LINUX.newnew/utility.h"


/* -------- sh.h -------- */
/*
 * shell
 */

#define MARK_BY_EASON 1
#undef CONFIG_FEATURE_COMMAND_EDITING

struct termios termbuf;
static char *WLAN_I="wlan0";
static char *ELAN_I="br0";
static char *WAN_I="eth1";
static char *PPP_I="ppp0";

static SS_STATUS_Tp pStatus=NULL;

typedef enum {false = 0, true = 1} bool;
static int PING_COUNT=5;
#define BUFFSIZE					128

#define _DHCPC_PROG_NAME		"udhcpc"
#define _DNRD_PROG_NAME		"dnrd"
#define _DHCPC_SCRIPT_PROG		"dhcpc.sh"
#define _DHCPD_SCRIPT_PROG		"dhcpd.sh"
#define _WLANAPP_SCRIPT_PROG	"dhcpd.sh"
#define _DHCPC_PID_PATH		"/etc/udhcpc"
#define _FIREWALL_SCRIPT_PROG	"firewall.sh"
#define _CONFIG_SCRIPT_PATH	"/bin"
#define _CONFIG_SCRIPT_PROG	"init.sh"
#define _DNS_M_CONFIG			"/etc/resolv.conf"
#define _DNS_A_CONFIG			"/var/udhcpc/resolv.conf"
#define _WEBS_PROG_NAME		"webs"
#define _GW						"gw"
#define _BRIDGE					"bridge"
#define _CLI						"cli"
#define _NO_SCRIPT_MSG 			"1>/dev/null 2>&1"

#define SYS_CMDS				"\tserver passthruvpn upnp\n\n"
#define IP_CMDS					"\taddress dhcp dns ping route status urlfilter ipfilter macfilter nat\n\n"
#define IP_IFCONF_CMDS			"\tiface\n\n"
#define IP_DHCP_CMDS			"\ton off slease status\n\n"
#define IP_DHCP_SLEASE_CMDS	"\tdisp add delete clearall\n\n"
#define IP_DNS_CMDS				"\tserver\n\n"
#define IP_ROUTE_CMDS			"\tstatus add drop\n\n"
#define IP_NAT_CMDS				"\tserver service\n\n"
#define IP_NAT_SERVER_CMDS		"\tdisp delete add clearall\n\n"
#define IP_NAT_SERVICE_CMDS	"\tdisp sap1 sap2 sap3 sap4 sap5 sap6 sap7 sap8\n\n"
#define IP_F_CMDS				"\tcustomize\n\n"
#define IP_F_CUST_CMDS			"\tdisp add delete clearall\n\n"
#define WLAN_CMDS				"\tactive association chid essid bssid version band mode auth preamble iapp survey security acl wds\n\n"
#define WLAN_ACL_CMDS			"\tdisp add delete clearall\n\n"
#define WLAN_SEC_CMDS			"\tencrypt 1x preauth\n\n"
#define WLAN_SEC_ENC_CMDS		"\toff wep wpa wpa2\n\n"
#define WLAN_WDS_CMDS			"\tpeer encrypt\n\n"
#define WLAN_WDS_PEER_CMDS	"\tdisp add delete clearall\n\n"
#define WLAN_WDS_ENC_CMDS	"\toff wep wpa wpa2\n\n"
#define WAN_CMDS				"\tclone dns static dhcp pppoe pptp l2tp\n\n"

#define MAX_NAME_LEN			31

#define	LINELIM	2100
#define	NPUSH	8	/* limit to input nesting */

#define	NOFILE	20	/* Number of open files */
#define	NUFILE	10	/* Number of user-accessible files */
#define	FDBASE	10	/* First file usable by Shell */

/*
 * values returned by wait
 */
#define	WAITSIG(s) ((s)&0177)
#define	WAITVAL(s) (((s)>>8)&0377)
#define	WAITCORE(s) (((s)&0200)!=0)

/*
 * library and system defintions
 */
typedef void xint;	/* base type of jmp_buf, for not broken compilers */

/*
 * shell components
 */

#define	QUOTE	0200

#define	NOBLOCK	((struct op *)NULL)
#define	NOWORD	((char *)NULL)
#define	NOWORDS	((char **)NULL)
#define	NOPIPE	((int *)NULL)

/*
 * Description of a command or an operation on commands.
 * Might eventually use a union.
 */
struct op {
	int	type;	/* operation type, see below */
	char	**words;	/* arguments to a command */
	struct	ioword	**ioact;	/* IO actions (eg, < > >>) */
	struct op *left;
	struct op *right;
	char	*str;	/* identifier for case and for */
};

#define	TCOM	1	/* command */
#define	TPAREN	2	/* (c-list) */
#define	TPIPE	3	/* a | b */
#define	TLIST	4	/* a [&;] b */
#define	TOR	5	/* || */
#define	TAND	6	/* && */
#define	TFOR	7
#define	TDO	8
#define	TCASE	9
#define	TIF	10
#define	TWHILE	11
#define	TUNTIL	12
#define	TELIF	13
#define	TPAT	14	/* pattern in case */
#define	TBRACE	15	/* {c-list} */
#define	TASYNC	16	/* c & */

/*
 * actions determining the environment of a process
 */
#define	BIT(i)	(1<<(i))
#define	FEXEC	BIT(0)	/* execute without forking */

/*
 * flags to control evaluation of words
 */
#define	DOSUB	1	/* interpret $, `, and quotes */
#define	DOBLANK	2	/* perform blank interpretation */
#define	DOGLOB	4	/* interpret [?* */
#define	DOKEY	8	/* move words with `=' to 2nd arg. list */
#define	DOTRIM	16	/* trim resulting string */

#define	DOALL	(DOSUB|DOBLANK|DOGLOB|DOKEY|DOTRIM)

static	char	**dolv;
static	int	dolc;
static	int	exstat;
static  char	gflg;
static  int	interactive;	/* Is this an interactive shell */
static  int	execflg;
static  int	multiline;	/* \n changed to ; */
static  struct	op	*outtree;	/* result from parser */

static	xint	*failpt;
static	xint	*errpt;
static	struct brkcon	*brklist;
static	int	isbreak;
//static int newfile(char *s);
static char *findeq(char *cp);
static char *cclass(char *p, int sub);
static void initarea(void);
extern int cli_main(int argc, char **argv);


struct	brkcon {
	jmp_buf	brkpt;
	struct	brkcon	*nextlev;
} ;

/*
 * redirection
 */
struct ioword {
	short	io_unit;	/* unit affected */
	short	io_flag;	/* action (below) */
	char	*io_name;	/* file name */
};
#define	IOREAD	1	/* < */
#define	IOHERE	2	/* << (here file) */
#define	IOWRITE	4	/* > */
#define	IOCAT	8	/* >> */
#define	IOXHERE	16	/* ${}, ` in << */
#define	IODUP	32	/* >&digit */
#define	IOCLOSE	64	/* >&- */

#define	IODEFAULT (-1)	/* token for default IO unit */

static	struct	wdblock	*wdlist;
static	struct	wdblock	*iolist;

/*
 * parsing & execution environment
 */
static struct	env {
	char	*linep;
	struct	io	*iobase;
	struct	io	*iop;
	xint	*errpt;
	int	iofd;
	struct	env	*oenv;
} e;

/*
 * flags:
 * -e: quit on error
 * -k: look for name=value everywhere on command line
 * -n: no execution
 * -t: exit after reading and executing one command
 * -v: echo as read
 * -x: trace
 * -u: unset variables net diagnostic
 */
static	char	*flag;

static	char	*null;	/* null value for variable */
static	int	intr;	/* interrupt pending */

static	char	*trap[_NSIG+1];
static	char	ourtrap[_NSIG+1];
static	int	trapset;	/* trap pending */

static	int	heedint;	/* heed interrupt signals */

static	int	yynerrs;	/* yacc */

static	char	line[LINELIM];
static	char	*elinep;

/*
 * other functions
 */
static int (*inbuilt(char *s ))(void);

static char *space (int n );
static char *strsave (char *s, int a );
static char *evalstr (char *cp, int f );
static char *putn (int n );
static char *itoa (unsigned u, int n );
static char *unquote (char *as );
static struct var *lookup (char *n );
static int rlookup (char *n );
static struct wdblock *glob (char *cp, struct wdblock *wb );
static int my_getc( int ec);
static int subgetc (int ec, int quoted );
static char **eval (char **ap, int f );
static int setstatus (int s );
static int waitfor (int lastpid, int canintr );

static void onintr (int s ); /* SIGINT handler */

static int newenv (int f );
static void quitenv (void);
static void err (char *s );
static int anys (char *s1, char *s2 );
static int any (int c, char *s );
static void next (int f );
static void setdash (void);
static void onecommand (void);
static void runtrap (int i );
static int gmatch (char *s, char *p );

/*
 * error handling
 */
static void leave (void); /* abort shell (or fail in subshell) */
static void fail (void);	 /* fail but return to process next command */
static void warn (char *s );
static void sig (int i );	 /* default signal handler */



/* -------- area stuff -------- */

#define	REGSIZE		sizeof(struct region)
#define GROWBY		256
//#define	SHRINKBY   64
#undef	SHRINKBY
#define FREE 32767
#define BUSY 0
#define	ALIGN (sizeof(int)-1)


struct region {
	struct	region *next;
	int	area;
};



/* -------- grammar stuff -------- */
typedef union {
	char	*cp;
	char	**wp;
	int	i;
	struct	op *o;
} YYSTYPE;
#define	WORD	256
#define	LOGAND	257
#define	LOGOR	258
#define	BREAK	259
#define	IF	260
#define	THEN	261
#define	ELSE	262
#define	ELIF	263
#define	FI	264
#define	CASE	265
#define	ESAC	266
#define	FOR	267
#define	WHILE	268
#define	UNTIL	269
#define	DO	270
#define	DONE	271
#define	IN	272
#define	YYERRCODE 300

/* flags to yylex */
#define	CONTIN	01	/* skip new lines to complete command */

#define	SYNTAXERR	zzerr()
static struct op *pipeline(int cf );
static struct op *andor(void);
static struct op *c_list(void);
static int synio(int cf );
static void musthave (int c, int cf );
static struct op *simple(void);
//static struct op *nested(int type, int mark );
static struct op *command(int cf );
/*static struct op *dogroup(int onlydone );
static struct op *thenpart(void);
static struct op *elsepart(void);
static struct op *caselist(void);
static struct op *casepart(void);
static char **pattern(void);*/
static char **wordlist(void);
static struct op *list(struct op *t1, struct op *t2 );
static struct op *block(int type, struct op *t1, struct op *t2, char **wp );
static struct op *newtp(void);
static struct op *namelist(struct op *t );
static char **copyw(void);
static void word(char *cp );
static struct ioword **copyio(void);
static struct ioword *io (int u, int f, char *cp );
static void zzerr(void);
static void yyerror(char *s );
static int yylex(int cf );
static int collect(int c, int c1 );
static int dual(int c );
static void diag(int ec );
static char *tree(unsigned size );

/* -------- var.h -------- */

struct	var {
	char	*value;
	char	*name;
	struct	var	*next;
	char	status;
};
#define	COPYV	1	/* flag to setval, suggesting copy */
#define	RONLY	01	/* variable is read-only */
#define	EXPORT	02	/* variable is to be exported */
#define	GETCELL	04	/* name & value space was got with getcell */

static	struct	var	*vlist;		/* dictionary */

static	struct	var	*homedir;	/* home directory */
static	struct	var	*prompt;	/* main prompt */
static	struct	var	*cprompt;	/* continuation prompt */
static	struct	var	*path;		/* search path for commands */
static	struct	var	*shell;		/* shell to interpret command files */
static	struct	var	*ifs;		/* field separators */

static int yyparse (void);
static struct var *lookup (char *n );
static void setval (struct var *vp, char *val );
static void nameval (struct var *vp, char *val, char *name );
static void ronly (struct var *vp );
static int isassign (char *s );
static int checkname (char *cp );
static int assign (char *s, int cf );
static void putvlist (int f, int out );
static int eqname (char *n1, char *n2 );

static int execute (struct op *t, int *pin, int *pout, int act );

/* -------- io.h -------- */
/* io buffer */
struct iobuf {
  unsigned id;				/* buffer id */
  char buf[512];			/* buffer */
  char *bufp;				/* pointer into buffer */
  char *ebufp;				/* pointer to end of buffer */
};

/* possible arguments to an IO function */
struct ioarg {
	char	*aword;
	char	**awordlist;
	int	afile;		/* file descriptor */
	unsigned afid;		/* buffer id */
	long	afpos;		/* file position */
	struct iobuf *afbuf;	/* buffer for this file */
};
//static struct ioarg ioargstack[NPUSH];
#define AFID_NOBUF	(~0)
#define AFID_ID		0

/* an input generator's state */
struct	io {
	int	(*iofn)();
	struct	ioarg	*argp;
	int	peekc;
	char	prev;		/* previous character read by readc() */
	char	nlcount;	/* for `'s */
	char	xchar;		/* for `'s */
	char	task;		/* reason for pushed IO */
};
//static	struct	io	iostack[NPUSH];
#define	XOTHER	0	/* none of the below */
#define	XDOLL	1	/* expanding ${} */
#define	XGRAVE	2	/* expanding `'s */
#define	XIO	3	/* file IO */

/* in substitution */
#define	INSUB()	(e.iop->task == XGRAVE || e.iop->task == XDOLL)

/*
 * input generators for IO structure
 */
static int nlchar (struct ioarg *ap );
static int strchar (struct ioarg *ap );
static int qstrchar (struct ioarg *ap );
static int filechar (struct ioarg *ap );
static int herechar (struct ioarg *ap );
static int linechar (struct ioarg *ap );
static int gravechar (struct ioarg *ap, struct io *iop );
static int qgravechar (struct ioarg *ap, struct io *iop );
static int dolchar (struct ioarg *ap );
static int wdchar (struct ioarg *ap );
static void scraphere (void);
static void freehere (int area );
static void gethere (void);
static void markhere (char *s, struct ioword *iop );
static int herein (char *hname, int xdoll );
static int run (struct ioarg *argp, int (*f)());

/*
 * IO functions
 */
static int eofc (void);
static int readc (void);
static void unget (int c );
static void ioecho (int c );
static void prs (char *s );
static void prn (unsigned u );
static void closef (int i );
static void closeall (void);

/*
 * IO control
 */
static void pushio (struct ioarg *argp, int (*fn)());
static int remap (int fd );
static int openpipe (int *pv );
static void closepipe (int *pv );
static struct io *setbase (struct io *ip );

static	struct	ioarg	temparg;	/* temporary for PUSHIO */
#define	PUSHIO(what,arg,gen) ((temparg.what = (arg)),pushio(&temparg,(gen)))
#define	RUN(what,arg,gen) ((temparg.what = (arg)), run(&temparg,(gen)))

/* -------- word.h -------- */

#define	NSTART	16	/* default number of words to allow for initially */

struct	wdblock {
	short	w_bsize;
	short	w_nword;
	/* bounds are arbitrary */
	char	*w_words[1];
};

static struct wdblock *addword (char *wd, struct wdblock *wb );
static struct wdblock *newword (int nw );
static char **getwords (struct wdblock *wb );

/* -------- area.h -------- */

/*
 * storage allocation
 */
static char *getcell (unsigned nbytes );
static void garbage (void);
static void setarea (char *cp, int a );
static int getarea (char *cp );
static void freearea (int a );
static void freecell (char *cp );
static	int	areanum;	/* current allocation area */

#define	NEW(type) (type *)getcell(sizeof(type))
#define	DELETE(obj)	freecell((char *)obj)


/* -------- misc stuff -------- */

static int iosetup (struct ioword *iop, int pipein, int pipeout );
static struct op **find1case (struct op *t, char *w );
static struct op *findcase (struct op *t, char *w );
static void brkset(struct brkcon *bc );

static int forkexec (struct op *t, int *pin, int *pout, int act, char **wp, int *pforked );

#ifndef MARK_BY_EASON
static char **makenv (void);

static void export (struct var *vp );
static void echo(char **wp );

static char *rexecve (char *c , char **v, char **envp );

static int dolabel(void);
static int dochdir(struct op *t );
static int doshift(struct op *t );
static int dologin(struct op *t );
static int doumask(struct op *t );
static int dodot(struct op *t );
static int dowait(struct op *t );
static int doread(struct op *t );
static int doeval(struct op *t );
static int dotrap(struct op *t );
static int dobreak(struct op *t );
static int docontinue(struct op *t );
static int doexport(struct op *t );
static int doreadonly(struct op *t );
static int doset(struct op *t );
static int dotimes(void);
#endif
static int doexec(struct op *t );
static int dosys(struct op *t );
static int dowan(struct op *t );
static int doip(struct op *t );
static int dowlan(struct op *t );
static int dohelp(void);
static int doexit(struct op *t );
static int doreload(struct op *t );

static int getsig(char *s );
static void setsig (int n, void (*f)());
static int getn(char *as );
//static int brkcontin (char *cp, int val );

//static void rdexp (char **wp, void (*f)(), int key);
//static void badid(char *s );

//static void varput (char *s, int out );
static int expand (char *cp, struct wdblock **wbp, int f );
static char *blank(int f );
static int dollar(int quoted );
static int grave(int quoted );
static void globname (char *we, char *pp );
static char *generate (char *start1, char *end1, char *middle, char *end );
static int anyspcl(struct wdblock *wb );
static int xstrcmp (char *p1, char *p2 );
static void glob0 (char *a0, unsigned int a1, int a2, int (*a3)(char *, char *));
static void glob1 (char *base, char *lim );
static void glob2 (char *i, char *j );
static void glob3 (char *i, char *j, char *k );
static void readhere (char **name, char *s, int ec );
static void pushio(struct ioarg *argp, int (*fn)());
static int xxchar(struct ioarg *ap );

struct	here {
	char	*h_tag;
	int	h_dosub;
	struct	ioword *h_iop;
	struct	here	*h_next;
};

static	char	*signame[] = {
	"Signal 0",
	"Hangup",
	(char *)NULL,	/* interrupt */
	"Quit",
	"Illegal instruction",
	"Trace/BPT trap",
	"Abort",
	"Bus error",
	"Floating Point Exception",
	"Killed",
	"SIGUSR1",
	"SIGSEGV",
	"SIGUSR2",
	(char *)NULL,	/* broken pipe */
	"Alarm clock",
	"Terminated",
};
#define	NSIGNAL (sizeof(signame)/sizeof(signame[0]))

struct res {
	char *r_name;
	int	  r_val;
};
#ifndef MARK_BY_EASON
static struct res restab[] = {
    {"for",		FOR},
    {"case",	CASE},
    {"esac",	ESAC},
    {"while",	WHILE},
    {"do",		DO},
    {"done",	DONE},
    {"if",		IF},
    {"in",		IN},
    {"then",	THEN},
    {"else",	ELSE},
    {"elif",	ELIF},
    {"until",	UNTIL},
    {"fi",		FI},
    {";;",		BREAK},
    {"||",		LOGOR},
    {"&&",		LOGAND},
    {"{",		'{'},
    {"}",		'}'},
    {0,		0},
};
#endif

struct builtincmd {
	const char *name;
	int (*builtinfunc)();
};
static const struct	builtincmd	builtincmds[] = {
#ifndef MARK_BY_EASON
    {".",		dodot},
    {":",		dolabel},
    {"break",	dobreak},
    {"cd",		dochdir},
    {"continue",docontinue},
    {"eval",	doeval},
    {"exec",	doexec},
    {"export",	doexport},
    {"login",	dologin},
    {"newgrp",	dologin},
    {"read",	doread},
    {"readonly",doreadonly},
    {"set",		doset},
    {"shift",	doshift},
    {"times",	dotimes},
    {"trap",	dotrap},
    {"umask",	doumask},
    {"wait",	dowait},
#endif
    {"?",	dohelp},
    {"reload",	doreload},
    {"help",	dohelp},
    {"sys",	dosys},
    {"ip",	doip},
    {"wlan",	dowlan},
    {"wan",	dowan},
    {"exit",	doexit},
    {0,0}
};

/* Globals */
extern	char	**environ;	/* environment pointer */
static char	**dolv;
static int	dolc;
static int	exstat;
static char	gflg;
static int	interactive;	/* Is this an interactive shell */
static int	execflg;
static int	multiline;	/* \n changed to ; */
static struct	op	*outtree;	/* result from parser */
static xint	*failpt;
static xint	*errpt;
static struct	brkcon	*brklist;
static int	isbreak;
static struct	wdblock	*wdlist;
static struct	wdblock	*iolist;
static char	*trap[_NSIG+1];
static char	ourtrap[_NSIG+1];
static int	trapset;	/* trap pending */
static int	yynerrs;	/* yacc */
static char	line[LINELIM];
static struct	var	*vlist;		/* dictionary */
static struct	var	*homedir;	/* home directory */
static struct	var	*prompt;	/* main prompt */
static struct	var	*cprompt;	/* continuation prompt */
static struct	var	*path;		/* search path for commands */
static struct	var	*shell;		/* shell to interpret command files */
static struct	var	*ifs;		/* field separators */
static struct	ioarg ioargstack[NPUSH];
static struct	io	iostack[NPUSH];
static int	areanum;	/* current allocation area */
static int	intr;
static int	inparse;
static char	flags['z'-'a'+1];
static char	*flag = flags-'a';
static char	*elinep = line+sizeof(line)-5;
static char	*null	= "";
static int	heedint =1;
static struct env e ={line, iostack, iostack-1, (xint *)NULL, FDBASE, (struct env *)NULL};
static void (*qflag)(int) = SIG_IGN;
static char	shellname[] = "/bin/cli";
//static char	search[] = ":/bin:/usr/bin";
static char	search[] = ":/";
static	int	startl;
static	int	peeksym;
static	int	nlseen;
static	int	iounit = IODEFAULT;
static	YYSTYPE	yylval;
static struct iobuf sharedbuf = {AFID_NOBUF};
static struct iobuf mainbuf = {AFID_NOBUF};
static unsigned bufid = AFID_ID;	/* buffer id counter */
static struct ioarg temparg = {0, 0, 0, AFID_NOBUF, 0};
static	struct here *inhere;		/* list of hear docs while parsing */
static	struct here *acthere;		/* list of active here documents */
static	struct region *areabot;		/* bottom of area */
static	struct region *areatop;		/* top of area */
static	struct region *areanxt;		/* starting point of scan */
static void * brktop;
static void * brkaddr;


#ifdef CONFIG_FEATURE_COMMAND_EDITING
static char * current_prompt;
#endif

static void disable_ttyecho() {
	// disable ECHO at tty
	tcgetattr(0, &termbuf);
	termbuf.c_lflag &= ~(ECHO|ECHOK);
	tcsetattr(0, TCSANOW, &termbuf);
}

static void enable_ttyecho() {
	// re-enable ECHO at tty
	tcgetattr(0, &termbuf);
	termbuf.c_lflag |= ECHO|ECHOK;
	tcsetattr(0, TCSANOW, &termbuf);	
}

static int loginauth() {
	register char passwd[10];
	register char real_pwd[10];
	int retry=0;

retry_a:
	disable_ttyecho();

	printf("Password: ");
	scanf("%s", passwd);

	enable_ttyecho();

	if(!apmib_get(MIB_USER_PASSWORD, (void *)real_pwd)) {
		printf("Get MIB error\n");
		return 0;
	}

	if(strcmp(passwd,real_pwd) != 0) {
		retry++;
		if(retry > 3)
			return 0;
		else {
			printf("\nInvalid Password !!!\n\n");
			goto retry_a;
		}
	}
	else
		return 1;
}

/* -------- sh.c -------- */
/*
 * shell
 */

extern int cli_main(int argc, char **argv)
{
	register int f;
	register char *s;
	int cflag;
	char *name, **ap;
	int (*iof)();

	signal(SIGTTOU,SIG_IGN); 
	signal(SIGTTIN,SIG_IGN); 
	signal(SIGTSTP,SIG_IGN); 
	signal(SIGHUP ,SIG_IGN);

	initarea();
	if ((ap = environ) != NULL) {
		while (*ap)
			assign(*ap++, !COPYV);
#ifndef MARK_BY_EASON
		for (ap = environ; *ap;)
			export(lookup(*ap++));
#endif
	}
	closeall();
	areanum = 1;
#ifndef MARK_BY_EASON
	shell = lookup("SHELL");
	if (shell->value == null)
		setval(shell, shellname);
	export(shell);

	homedir = lookup("HOME");
	if (homedir->value == null)
		setval(homedir, "/");
	export(homedir);

	setval(lookup("$"), itoa(getpid(), 5));

	path = lookup("PATH");
	if (path->value != null)
	//if (path->value == null)
		setval(path, search);
	export(path);
#endif

	ifs = lookup("IFS");
	if (ifs->value == null)
		setval(ifs, " \t\n");

	prompt = lookup("PS1");
#ifdef CONFIG_FEATURE_SH_FANCY_PROMPT
	if (prompt->value == null)
#endif
		setval(prompt, "$ ");
	if (geteuid() == 0) {
		setval(prompt, "P-330W> ");
		prompt->status &= ~EXPORT;
	}
	cprompt = lookup("PS2");
#ifdef CONFIG_FEATURE_SH_FANCY_PROMPT
	if (cprompt->value == null)
#endif
		setval(cprompt, "> ");

	iof = filechar;
	cflag = 0;
	name = *argv++;
	if (--argc >= 1) {
		if(argv[0][0] == '-' && argv[0][1] != '\0') {
			for (s = argv[0]+1; *s; s++)
				switch (*s) {
				case 'c':
					prompt->status &= ~EXPORT;
					cprompt->status &= ~EXPORT;
					setval(prompt, "");
					setval(cprompt, "");
					cflag = 1;
					if (--argc > 0)
						PUSHIO(aword, *++argv, iof = nlchar);
					break;
	
				case 'q':
					qflag = SIG_DFL;
					break;

				case 's':
					/* standard input */
					break;

				case 't':
					prompt->status &= ~EXPORT;
					setval(prompt, "");
					iof = linechar;
					break;
	
				case 'i':
					interactive++;
				default:
					if (*s>='a' && *s<='z')
						flag[(int)*s]++;
				}
		} else {
			argv--;
			argc++;
		}
		if (iof == filechar && --argc > 0) {
			setval(prompt, "");
			setval(cprompt, "");
			prompt->status &= ~EXPORT;
			cprompt->status &= ~EXPORT;
			//if (newfile(name = *++argv))
				//exit(1);
		}
	}
	setdash();
	
	// apmib control
	if (!apmib_init()) {
		printf("Initialize AP MIB failed!\n");
		return 0;
	}
	if(!loginauth()) {
		printf("Login failure!\n");
		exit(0);
	}
	
	if (e.iop < iostack) {
		PUSHIO(afile, 0, iof);
		if (isatty(0) && isatty(1) && !cflag) {
			interactive++;
			/*printf( "\n\n" BB_BANNER " Built-in shell (msh)\n");
			printf( "Enter 'help' for a list of built-in commands.\n\n");*/
			printf("\n\n Welcome !!! \n\n");
		}
	}
	signal(SIGQUIT, qflag);
	if (name && name[0] == '-') {
		interactive++;
		if ((f = open(".profile", 0)) >= 0)
			next(remap(f));
		if ((f = open("/etc/profile", 0)) >= 0)
			next(remap(f));
	}
	if (interactive)
		signal(SIGTERM, sig);
	if (signal(SIGINT, SIG_IGN) != SIG_IGN)
		signal(SIGINT, onintr);
	dolv = argv;
	dolc = argc;
	dolv[0] = name;
	if (dolc > 1) {
		for (ap = ++argv; --argc > 0;) {
			if (assign(*ap = *argv++, !COPYV)) {
				dolc--;	/* keyword */
			} else {
				ap++;
			}
		}
	}	
	setval(lookup("#"), putn((--dolc < 0) ? (dolc = 0) : dolc));

	for (;;) {
		if (interactive && e.iop <= iostack) {
#ifdef CONFIG_FEATURE_COMMAND_EDITING
			current_prompt=prompt->value;
#else
			prs(prompt->value);
#endif
		}
		onecommand();
	}
}

static void
setdash()
{
	register char *cp;
	register int c;
	char m['z'-'a'+1];

	cp = m;
	for (c='a'; c<='z'; c++)
		if (flag[c])
			*cp++ = c;
	*cp = 0;
	setval(lookup("-"), m);
}

/*static int
newfile(s)
register char *s;
{
	register int f;

	if (strcmp(s, "-") != 0) {
		f = open(s, 0);
		if (f < 0) {
			prs(s);
			err(": cannot open");
			return(1);
		}
	} else
		f = 0;
	next(remap(f));
	return(0);
}*/

static void
onecommand()
{
	register int i;
	jmp_buf m1;

	while (e.oenv)
		quitenv();
	areanum = 1;
	freehere(areanum);
	freearea(areanum);
	garbage();
	wdlist = 0;
	iolist = 0;
	e.errpt = 0;
	e.linep = line;
	yynerrs = 0;
	multiline = 0;
	inparse = 1;
	intr = 0;
	execflg = 0;
	setjmp(failpt = m1);	/* Bruce Evans' fix */
	if (setjmp(failpt = m1) || yyparse() || intr) {
		while (e.oenv)
			quitenv();
		scraphere();
		if (!interactive && intr)
			leave();
		inparse = 0;
		intr = 0;
		return;
	}
	inparse = 0;
	brklist = 0;
	intr = 0;
	execflg = 0;
	if (!flag['n'])
		execute(outtree, NOPIPE, NOPIPE, 0);
	if (!interactive && intr) {
		execflg = 0;
		leave();
	}
	if ((i = trapset) != 0) {
		trapset = 0;
		runtrap(i);
	}
}

static void
fail()
{
	longjmp(failpt, 1);
	/* NOTREACHED */
}

static void
leave()
{
	if (execflg)
		fail();
	scraphere();
	freehere(1);
	runtrap(0);
	exit(exstat);
	/* NOTREACHED */
}

static void
warn(s)
register char *s;
{
	if(*s) {
		prs(s);
		exstat = -1;
	}
	prs("\n");
	if (flag['e'])
		leave();
}

static void
err(s)
char *s;
{
	warn(s);
	if (flag['n'])
		return;
	if (!interactive)
		leave();
	if (e.errpt)
		longjmp(e.errpt, 1);
	closeall();
	e.iop = e.iobase = iostack;
}

static int
newenv(f)
int f;
{
	register struct env *ep;

	if (f) {
		quitenv();
		return(1);
	}
	ep = (struct env *) space(sizeof(*ep));
	if (ep == NULL) {
		while (e.oenv)
			quitenv();
		fail();
	}
	*ep = e;
	e.oenv = ep;
	e.errpt = errpt;
	return(0);
}

static void
quitenv()
{
	register struct env *ep;
	register int fd;

	if ((ep = e.oenv) != NULL) {
		fd = e.iofd;
		e = *ep;
		/* should close `'d files */
		DELETE(ep);
		while (--fd >= e.iofd)
			close(fd);
	}
}

/*
 * Is any character from s1 in s2?
 */
static int
anys(s1, s2)
register char *s1, *s2;
{
	while (*s1)
		if (any(*s1++, s2))
			return(1);
	return(0);
}

/*
 * Is character c in s?
 */
static int
any(c, s)
register int c;
register char *s;
{
	while (*s)
		if (*s++ == c)
			return(1);
	return(0);
}

static char *
putn(n)
register int n;
{
	return(itoa(n, -1));
}

static char *
itoa(u, n)
register unsigned u;
int n;
{
	register char *cp;
	static char s[20];
	int m;

	m = 0;
	if (n < 0 && (int) u < 0) {
		m++;
		u = -u;
	}
	cp = s+sizeof(s);
	*--cp = 0;
	do {
		*--cp = u%10 + '0';
		u /= 10;
	} while (--n > 0 || u);
	if (m)
		*--cp = '-';
	return(cp);
}

static void
next(f)
int f;
{
	PUSHIO(afile, f, filechar);
}

static void
onintr(s)
int s;				/* ANSI C requires a parameter */
{
	signal(SIGINT, onintr);
	intr = 1;
	if (interactive) {
		if (inparse) {
			prs("\n");
			fail();
		}
	}
	else if (heedint) {
		execflg = 0;
		leave();
	}
}

static char *
space(n)
int n;
{
	register char *cp;

	if ((cp = getcell(n)) == 0)
		err("out of string space");
	return(cp);
}

static char *
strsave(s, a)
register char *s;
int a;
{
	register char *cp, *xp;

	if ((cp = space(strlen(s)+1)) != NULL) {
		setarea((char *)cp, a);
		for (xp = cp; (*xp++ = *s++) != '\0';)
			;
		return(cp);
	}
	return("");
}

/*
 * trap handling
 */
static void
sig(i)
register int i;
{
	trapset = i;
	signal(i, sig);
}

static void runtrap(i)
int i;
{
	char *trapstr;

	if ((trapstr = trap[i]) == NULL)
		return;
	if (i == 0)
		trap[i] = 0;
	RUN(aword, trapstr, nlchar);
}

/* -------- var.c -------- */

/*
 * Find the given name in the dictionary
 * and return its value.  If the name was
 * not previously there, enter it now and
 * return a null value.
 */
static struct var *
lookup(n)
register char *n;
{
	register struct var *vp;
	register char *cp;
	register int c;
	static struct var dummy;

	if (isdigit(*n)) {
		dummy.name = n;
		for (c = 0; isdigit(*n) && c < 1000; n++)
			c = c*10 + *n-'0';
		dummy.status = RONLY;
		dummy.value = c <= dolc? dolv[c]: null;
		return(&dummy);
	}
	for (vp = vlist; vp; vp = vp->next)
		if (eqname(vp->name, n))
			return(vp);
	cp = findeq(n);
	vp = (struct var *)space(sizeof(*vp));
	if (vp == 0 || (vp->name = space((int)(cp-n)+2)) == 0) {
		dummy.name = dummy.value = "";
		return(&dummy);
	}
	for (cp = vp->name; (*cp = *n++) && *cp != '='; cp++)
		;
	if (*cp == 0)
		*cp = '=';
	*++cp = 0;
	setarea((char *)vp, 0);
	setarea((char *)vp->name, 0);
	vp->value = null;
	vp->next = vlist;
	vp->status = GETCELL;
	vlist = vp;
	return(vp);
}

/*
 * give variable at `vp' the value `val'.
 */
static void
setval(vp, val)
struct var *vp;
char *val;
{
	nameval(vp, val, (char *)NULL);
}

/*
 * if name is not NULL, it must be
 * a prefix of the space `val',
 * and end with `='.
 * this is all so that exporting
 * values is reasonably painless.
 */
static void
nameval(vp, val, name)
register struct var *vp;
char *val, *name;
{
	register char *cp, *xp;
	char *nv;
	int fl;

	if (vp->status & RONLY) {
		for (xp = vp->name; *xp && *xp != '=';)
			putc(*xp++, stderr);
		err(" is read-only");
		return;
	}
	fl = 0;
	if (name == NULL) {
		xp = space(strlen(vp->name)+strlen(val)+2);
		if (xp == 0)
			return;
		/* make string:  name=value */
		setarea((char *)xp, 0);
		name = xp;
		for (cp = vp->name; (*xp = *cp++) && *xp!='='; xp++)
			;
		if (*xp++ == 0)
			xp[-1] = '=';
		nv = xp;
		for (cp = val; (*xp++ = *cp++) != '\0';)
			;
		val = nv;
		fl = GETCELL;
	}
	if (vp->status & GETCELL)
		freecell(vp->name);    /* form new string `name=value' */
	vp->name = name;
	vp->value = val;
	vp->status |= fl;
}



static void
ronly(vp)
struct var *vp;
{
	if (isalpha(vp->name[0]) || vp->name[0] == '_')	/* not an internal symbol */
		vp->status |= RONLY;
}

static int
isassign(s)
register char *s;
{
	if (!isalpha((int)*s) && *s != '_')
		return(0);
	for (; *s != '='; s++)
		if (*s == 0 || (!isalnum(*s) && *s != '_'))
			return(0);
	return(1);
}

static int
assign(s, cf)
register char *s;
int cf;
{
	register char *cp;
	struct var *vp;

	if (!isalpha(*s) && *s != '_')
		return(0);
	for (cp = s; *cp != '='; cp++)
		if (*cp == 0 || (!isalnum(*cp) && *cp != '_'))
			return(0);
	vp = lookup(s);
	nameval(vp, ++cp, cf == COPYV? (char *)NULL: s);
	if (cf != COPYV)
		vp->status &= ~GETCELL;
	return(1);
}

static int
checkname(cp)
register char *cp;
{
	if (!isalpha(*cp++) && *(cp-1) != '_')
		return(0);
	while (*cp)
		if (!isalnum(*cp++) && *(cp-1) != '_')
			return(0);
	return(1);
}

static void
putvlist(f, out)
register int f, out;
{
	register struct var *vp;

	for (vp = vlist; vp; vp = vp->next)
		if (vp->status & f && (isalpha(*vp->name) || *vp->name == '_')) {
#ifndef MARK_BY_EASON
			if (vp->status & EXPORT)
				write(out, "export ", 7);
#endif
			if (vp->status & RONLY)
				write(out, "readonly ", 9);
			write(out, vp->name, (int)(findeq(vp->name) - vp->name));
			write(out, "\n", 1);
		}
}

static int
eqname(n1, n2)
register char *n1, *n2;
{
	for (; *n1 != '=' && *n1 != 0; n1++)
		if (*n2++ != *n1)
			return(0);
	return(*n2 == 0 || *n2 == '=');
}

static char *
findeq(cp)
register char *cp;
{
	while (*cp != '\0' && *cp != '=')
		cp++;
	return(cp);
}

/* -------- gmatch.c -------- */
/*
 * int gmatch(string, pattern)
 * char *string, *pattern;
 *
 * Match a pattern as in sh(1).
 */

#define	CMASK	0377
#define	QUOTE	0200
#define	QMASK	(CMASK&~QUOTE)
#define	NOT	'!'	/* might use ^ */

static int
gmatch(s, p)
register char *s, *p;
{
	register int sc, pc;

	if (s == NULL || p == NULL)
		return(0);
	while ((pc = *p++ & CMASK) != '\0') {
		sc = *s++ & QMASK;
		switch (pc) {
		case '[':
			if ((p = cclass(p, sc)) == NULL)
				return(0);
			break;

		case '?':
			if (sc == 0)
				return(0);
			break;

		case '*':
			s--;
			do {
				if (*p == '\0' || gmatch(s, p))
					return(1);
			} while (*s++ != '\0');
			return(0);

		default:
			if (sc != (pc&~QUOTE))
				return(0);
		}
	}
	return(*s == 0);
}

static char *
cclass(p, sub)
register char *p;
register int sub;
{
	register int c, d, not, found;

	if ((not = *p == NOT) != 0)
		p++;
	found = not;
	do {
		if (*p == '\0')
			return((char *)NULL);
		c = *p & CMASK;
		if (p[1] == '-' && p[2] != ']') {
			d = p[2] & CMASK;
			p++;
		} else
			d = c;
		if (c == sub || (c <= sub && sub <= d))
			found = !not;
	} while (*++p != ']');
	return(found? p+1: (char *)NULL);
}


/* -------- area.c -------- */

/*
 * All memory between (char *)areabot and (char *)(areatop+1) is
 * exclusively administered by the area management routines.
 * It is assumed that sbrk() and brk() manipulate the high end.
 */

#define sbrk(X) ({ void * __q = (void *)-1; if (brkaddr + (int)(X) < brktop) { __q = brkaddr; brkaddr+=(int)(X); } __q;})

static void
initarea()
{
	brkaddr = malloc(65000);
	brktop = brkaddr + 65000;

	while ((int)sbrk(0) & ALIGN)
		sbrk(1);
	areabot = (struct region *)sbrk(REGSIZE);

	areabot->next = areabot;
	areabot->area = BUSY;
	areatop = areabot;
	areanxt = areabot;
}

char *
getcell(nbytes)
unsigned nbytes;
{
	register int nregio;
	register struct region *p, *q;
	register int i;

	if (nbytes == 0) {
		puts("getcell(0)");
		abort();
	}	/* silly and defeats the algorithm */
	/*
	 * round upwards and add administration area
	 */
	nregio = (nbytes+(REGSIZE-1))/REGSIZE + 1;
	for (p = areanxt;;) {
		if (p->area > areanum) {
			/*
			 * merge free cells
			 */
			while ((q = p->next)->area > areanum && q != areanxt)
				p->next = q->next;
			/*
			 * exit loop if cell big enough
			 */
			if (q >= p + nregio)
				goto found;
		}
		p = p->next;
		if (p == areanxt)
			break;
	}
	i = nregio >= GROWBY ? nregio : GROWBY;
	p = (struct region *)sbrk(i * REGSIZE);
	if (p == (struct region *)-1)
		return((char *)NULL);
	p--;
	if (p != areatop) {
		puts("not contig");
		abort();	/* allocated areas are contiguous */
	}
	q = p + i;
	p->next = q;
	p->area = FREE;
	q->next = areabot;
	q->area = BUSY;
	areatop = q;
found:
	/*
	 * we found a FREE area big enough, pointed to by 'p', and up to 'q'
	 */
	areanxt = p + nregio;
	if (areanxt < q) {
		/*
		 * split into requested area and rest
		 */
		if (areanxt+1 > q) {
			puts("OOM");
			abort();	/* insufficient space left for admin */
		}
		areanxt->next = q;
		areanxt->area = FREE;
		p->next = areanxt;
	}
	p->area = areanum;
	return((char *)(p+1));
}

static void
freecell(cp)
char *cp;
{
	register struct region *p;

	if ((p = (struct region *)cp) != NULL) {
		p--;
		if (p < areanxt)
			areanxt = p;
		p->area = FREE;
	}
}

static void
freearea(a)
register int a;
{
	register struct region *p, *top;

	top = areatop;
	for (p = areabot; p != top; p = p->next)
		if (p->area >= a)
			p->area = FREE;
}

static void
setarea(cp,a)
char *cp;
int a;
{
	register struct region *p;

	if ((p = (struct region *)cp) != NULL)
		(p-1)->area = a;
}

int
getarea(cp)
char *cp;
{
	return ((struct region*)cp-1)->area;
}

static void
garbage()
{
	register struct region *p, *q, *top;

	top = areatop;
	for (p = areabot; p != top; p = p->next) {
		if (p->area > areanum) {
			while ((q = p->next)->area > areanum)
				p->next = q->next;
			areanxt = p;
		}
	}
#ifdef SHRINKBY
	if (areatop >= q + SHRINKBY && q->area > areanum) {
		brk((char *)(q+1));
		q->next = areabot;
		q->area = BUSY;
		areatop = q;
	}
#endif
}

/* -------- csyn.c -------- */
/*
 * shell: syntax (C version)
 */


int
yyparse()
{
	startl  = 1;
	peeksym = 0;
	yynerrs = 0;
	outtree = c_list();
	musthave('\n', 0);
	return(yynerrs!=0);
}

static struct op *
pipeline(cf)
int cf;
{
	register struct op *t, *p;
	register int c;

	t = command(cf);
	if (t != NULL) {
		while ((c = yylex(0)) == '|') {
			if ((p = command(CONTIN)) == NULL)
				SYNTAXERR;
			if (t->type != TPAREN && t->type != TCOM) {
				/* shell statement */
				t = block(TPAREN, t, NOBLOCK, NOWORDS);
			}
			t = block(TPIPE, t, p, NOWORDS);
		}
		peeksym = c;
	}
	return(t);
}

static struct op *
andor()
{
	register struct op *t, *p;
	register int c;

	t = pipeline(0);
	if (t != NULL) {
		while ((c = yylex(0)) == LOGAND || c == LOGOR) {
			if ((p = pipeline(CONTIN)) == NULL)
				SYNTAXERR;
			t = block(c == LOGAND? TAND: TOR, t, p, NOWORDS);
		}
		peeksym = c;
	}
	return(t);
}

static struct op *
c_list()
{
	register struct op *t, *p;
	register int c;

	t = andor();
	if (t != NULL) {
		if((peeksym = yylex(0)) == '&')
			t = block(TASYNC, t, NOBLOCK, NOWORDS);
		while ((c = yylex(0)) == ';' || c == '&' || (multiline && c == '\n')) {
			if ((p = andor()) == NULL)
				return(t);
			if((peeksym = yylex(0)) == '&')
				p = block(TASYNC, p, NOBLOCK, NOWORDS);
			t = list(t, p);
		}
		peeksym = c;
	}
	return(t);
}


static int
synio(cf)
int cf;
{
	register struct ioword *iop;
	register int i;
	register int c;

	if ((c = yylex(cf)) != '<' && c != '>') {
		peeksym = c;
		return(0);
	}
	i = yylval.i;
	musthave(WORD, 0);
	iop = io(iounit, i, yylval.cp);
	iounit = IODEFAULT;
	if (i & IOHERE)
		markhere(yylval.cp, iop);
	return(1);
}

static void
musthave(c, cf)
int c, cf;
{
	if ((peeksym = yylex(cf)) != c)
		SYNTAXERR;
	peeksym = 0;
}

static struct op *
simple()
{
	register struct op *t;

	t = NULL;
	for (;;) {
		switch (peeksym = yylex(0)) {
		case '<':
		case '>':
			(void) synio(0);
			break;

		case WORD:
			if (t == NULL) {
				t = newtp();
				t->type = TCOM;
			}
			peeksym = 0;
			word(yylval.cp);
			break;

		default:
			return(t);
		}
	}
}

/*static struct op *
nested(type, mark)
int type, mark;
{
	register struct op *t;

	multiline++;
	t = c_list();
	musthave(mark, 0);
	multiline--;
	return(block(type, t, NOBLOCK, NOWORDS));
}*/

static struct op *
command(cf)
int cf;
{
	register struct op *t;
	struct wdblock *iosave;
	register int c;

	iosave = iolist;
	iolist = NULL;
	if (multiline)
		cf |= CONTIN;
	while (synio(cf))
		cf = 0;

	c = yylex(cf);
	// defaults
	peeksym = c;
	if ((t = simple()) == NULL) {
		if (iolist == NULL)
			return((struct op *)NULL);
		t = newtp();
		t->type = TCOM;
	}
	/*switch (c = yylex(cf)) {
	default:
		peeksym = c;
		if ((t = simple()) == NULL) {
			if (iolist == NULL)
				return((struct op *)NULL);
			t = newtp();
			t->type = TCOM;
		}
		break;

	case '(':
		t = nested(TPAREN, ')');
		break;

	case '{':
		t = nested(TBRACE, '}');
		break;

	case FOR:
		t = newtp();
		t->type = TFOR;
		musthave(WORD, 0);
		startl = 1;
		t->str = yylval.cp;
		multiline++;
		t->words = wordlist();
		if ((c = yylex(0)) != '\n' && c != ';')
			peeksym = c;
		t->left = dogroup(0);
		multiline--;
		break;

	case WHILE:
	case UNTIL:
		multiline++;
		t = newtp();
		t->type = c == WHILE? TWHILE: TUNTIL;
		t->left = c_list();
		t->right = dogroup(1);
		t->words = NULL;
		multiline--;
		break;

	case CASE:
		t = newtp();
		t->type = TCASE;
		musthave(WORD, 0);
		t->str = yylval.cp;
		startl++;
		multiline++;
		musthave(IN, CONTIN);
		startl++;
		t->left = caselist();
		musthave(ESAC, 0);
		multiline--;
		break;

	case IF:
		multiline++;
		t = newtp();
		t->type = TIF;
		t->left = c_list();
		t->right = thenpart();
		musthave(FI, 0);
		multiline--;
		break;
	}*/
	while (synio(0))
		;
	t = namelist(t);
	iolist = iosave;
	return(t);
}

/*static struct op *
dogroup(onlydone)
int onlydone;
{
	register int c;
	register struct op *mylist;

	c = yylex(CONTIN);
	if (c == DONE && onlydone)
		return((struct op *)NULL);
	if (c != DO)
		SYNTAXERR;
	mylist = c_list();
	musthave(DONE, 0);
	return(mylist);
}

static struct op *
thenpart()
{
	register int c;
	register struct op *t;

	if ((c = yylex(0)) != THEN) {
		peeksym = c;
		return((struct op *)NULL);
	}
	t = newtp();
	t->type = 0;
	t->left = c_list();
	if (t->left == NULL)
		SYNTAXERR;
	t->right = elsepart();
	return(t);
}

static struct op *
elsepart()
{
	register int c;
	register struct op *t;

	switch (c = yylex(0)) {
	case ELSE:
		if ((t = c_list()) == NULL)
			SYNTAXERR;
		return(t);

	case ELIF:
		t = newtp();
		t->type = TELIF;
		t->left = c_list();
		t->right = thenpart();
		return(t);

	default:
		peeksym = c;
		return((struct op *)NULL);
	}
}

static struct op *
caselist()
{
	register struct op *t;

	t = NULL;
	while ((peeksym = yylex(CONTIN)) != ESAC)
		t = list(t, casepart());
	return(t);
}

static struct op *
casepart()
{
	register struct op *t;

	t = newtp();
	t->type = TPAT;
	t->words = pattern();
	musthave(')', 0);
	t->left = c_list();
	if ((peeksym = yylex(CONTIN)) != ESAC)
		musthave(BREAK, CONTIN);
	return(t);
}

static char **
pattern()
{
	register int c, cf;

	cf = CONTIN;
	do {
		musthave(WORD, cf);
		word(yylval.cp);
		cf = 0;
	} while ((c = yylex(0)) == '|');
	peeksym = c;
	word(NOWORD);
	return(copyw());
}*/

static char **
wordlist()
{
	register int c;

	if ((c = yylex(0)) != IN) {
		peeksym = c;
		return((char **)NULL);
	}
	startl = 0;
	while ((c = yylex(0)) == WORD)
		word(yylval.cp);
	word(NOWORD);
	peeksym = c;
	return(copyw());
}

/*
 * supporting functions
 */
static struct op *
list(t1, t2)
register struct op *t1, *t2;
{
	if (t1 == NULL)
		return(t2);
	if (t2 == NULL)
		return(t1);
	return(block(TLIST, t1, t2, NOWORDS));
}

static struct op *
block(type, t1, t2, wp)
int type;
struct op *t1, *t2;
char **wp;
{
	register struct op *t;

	t = newtp();
	t->type = type;
	t->left = t1;
	t->right = t2;
	t->words = wp;
	return(t);
}

static int
rlookup(n)
register char *n;
{
#ifndef MARK_BY_EASON
	register struct res *rp;

	for (rp = restab; rp->r_name; rp++)
		if (strcmp(rp->r_name, n) == 0)
			return(rp->r_val);
#endif
	return(0);
}

static struct op *
newtp()
{
	register struct op *t;

	t = (struct op *)tree(sizeof(*t));
	t->type = 0;
	t->words = NULL;
	t->ioact = NULL;
	t->left = NULL;
	t->right = NULL;
	t->str = NULL;
	return(t);
}

static struct op *
namelist(t)
register struct op *t;
{
	if (iolist) {
		iolist = addword((char *)NULL, iolist);
		t->ioact = copyio();
	} else
		t->ioact = NULL;
	if (t->type != TCOM) {
		if (t->type != TPAREN && t->ioact != NULL) {
			t = block(TPAREN, t, NOBLOCK, NOWORDS);
			t->ioact = t->left->ioact;
			t->left->ioact = NULL;
		}
		return(t);
	}
	word(NOWORD);
	t->words = copyw();
	return(t);
}

static char **
copyw()
{
	register char **wd;

	wd = getwords(wdlist);
	wdlist = 0;
	return(wd);
}

static void
word(cp)
char *cp;
{
	wdlist = addword(cp, wdlist);
}

static struct ioword **
copyio()
{
	register struct ioword **iop;

	iop = (struct ioword **) getwords(iolist);
	iolist = 0;
	return(iop);
}

static struct ioword *
io(u, f, cp)
int u;
int f;
char *cp;
{
	register struct ioword *iop;

	iop = (struct ioword *) tree(sizeof(*iop));
	iop->io_unit = u;
	iop->io_flag = f;
	iop->io_name = cp;
	iolist = addword((char *)iop, iolist);
	return(iop);
}

static void
zzerr()
{
	yyerror("syntax error");
}

static void
yyerror(s)
char *s;
{
	yynerrs++;
	if (interactive && e.iop <= iostack) {
		multiline = 0;
		while (eofc() == 0 && yylex(0) != '\n')
			;
	}
	err(s);
	fail();
}

static int
yylex(cf)
int cf;
{
	register int c, c1;
	int atstart;

	if ((c = peeksym) > 0) {
		peeksym = 0;
		if (c == '\n')
			startl = 1;
		return(c);
	}
	nlseen = 0;
	e.linep = line;
	atstart = startl;
	startl = 0;
	yylval.i = 0;

loop:
	while ((c = my_getc(0)) == ' ' || c == '\t')
		;
	switch (c) {
	default:
		if (any(c, "0123456789")) {
			unget(c1 = my_getc(0));
			if (c1 == '<' || c1 == '>') {
				iounit = c - '0';
				goto loop;
			}
			*e.linep++ = c;
			c = c1;
		}
		break;

	case '#':
		while ((c = my_getc(0)) != 0 && c != '\n')
			;
		unget(c);
		goto loop;

	case 0:
		return(c);

	case '$':
		*e.linep++ = c;
		if ((c = my_getc(0)) == '{') {
			if ((c = collect(c, '}')) != '\0')
				return(c);
			goto pack;
		}
		break;

	case '`':
	case '\'':
	case '"':
		/*if ((c = collect(c, c)) != '\0')
			return(c);*/
		printf("Invalid charsets !!!\n");
		goto pack;

	case '|':
	case '&':
	case ';':
		/*if ((c1 = dual(c)) != '\0') {
			startl = 1;
			return(c1);
		}
		startl = 1;*/
		printf("Invalid charsets !!!\n");
		//return(c);
		goto pack;
	case '^':
		//startl = 1;
		printf("Invalid charsets !!!\n");
		//return('|');
		goto pack;
	case '>':
	case '<':
		//diag(c);
		//return(c);
		printf("Invalid charsets !!!\n");
		goto pack;

	case '\n':
		nlseen++;
		gethere();
		startl = 1;
		if (multiline || cf & CONTIN) {
			if (interactive && e.iop <= iostack) {
#ifdef CONFIG_FEATURE_COMMAND_EDITING
			current_prompt=cprompt->value;
#else
			prs(cprompt->value);
#endif
			}
			if (cf & CONTIN)
				goto loop;
		}
		return(c);

	case '(':
	case ')':
		//startl = 1;
		//return(c);
		printf("Invalid charsets !!!\n");
		goto pack;
	}

	unget(c);

pack:
	while ((c = my_getc(0)) != 0 && !any(c, "`$ '\"\t;&<>()|^\n"))
		if (e.linep >= elinep)
			err("word too long");
		else
			*e.linep++ = c;
	unget(c);
	if(any(c, "\"'`$"))
		goto loop;
	*e.linep++ = '\0';
	if (atstart && (c = rlookup(line))!=0) {
		startl = 1;
		return(c);
	}
	yylval.cp = strsave(line, areanum);
	return(WORD);
}

static int
collect(c, c1)
register int c, c1;
{
	char s[2];

	*e.linep++ = c;
	while ((c = my_getc(c1)) != c1) {
		if (c == 0) {
			unget(c);
			s[0] = c1;
			s[1] = 0;
			prs("no closing "); yyerror(s);
			return(YYERRCODE);
		}
		if (interactive && c == '\n' && e.iop <= iostack) {
#ifdef CONFIG_FEATURE_COMMAND_EDITING
		    current_prompt=cprompt->value;
#else
		    prs(cprompt->value);
#endif
		}
		*e.linep++ = c;
	}
	*e.linep++ = c;
	return(0);
}

static int
dual(c)
register int c;
{
	char s[3];
	register char *cp = s;

	*cp++ = c;
	*cp++ = my_getc(0);
	*cp = 0;
	if ((c = rlookup(s)) == 0)
		unget(*--cp);
	return(c);
}

static void
diag(ec)
register int ec;
{
	register int c;

	c = my_getc(0);
	if (c == '>' || c == '<') {
		if (c != ec)
			zzerr();
		yylval.i = ec == '>'? IOWRITE|IOCAT: IOHERE;
		c = my_getc(0);
	} else
		yylval.i = ec == '>'? IOWRITE: IOREAD;
	if (c != '&' || yylval.i == IOHERE)
		unget(c);
	else
		yylval.i |= IODUP;
}

static char *
tree(size)
unsigned size;
{
	register char *t;

	if ((t = getcell(size)) == NULL) {
		prs("command line too complicated\n");
		fail();
		/* NOTREACHED */
	}
	return(t);
}

/* VARARGS1 */
/* ARGSUSED */

/* -------- exec.c -------- */

/*
 * execute tree
 */


static int
execute(t, pin, pout, act)
register struct op *t;
int *pin, *pout;
int act;
{
	register struct op *t1;
	volatile int i, rv, a;
	char *cp, **wp, **wp2;
	struct var *vp;
	struct brkcon bc;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &wp;
#endif	


	if (t == NULL)
		return(0);
	rv = 0;
	a = areanum++;
	wp = (wp2 = t->words) != NULL
	     ? eval(wp2, t->type == TCOM ? DOALL : DOALL & ~DOKEY)
	     : NULL;

	switch(t->type) {
	case TPAREN:
	case TCOM:
		{
			int child;
			rv = forkexec(t, pin, pout, act, wp, &child);
			if (child) {
				exstat = rv;
				leave();
			}
		}
		break;

	case TPIPE:
		{
		    int pv[2];
		    if ((rv = openpipe(pv)) < 0)
			break;
		    pv[0] = remap(pv[0]);
		    pv[1] = remap(pv[1]);
		    (void) execute(t->left, pin, pv, 0);
		    rv = execute(t->right, pv, pout, 0);
		}
		break;

	case TLIST:
		(void) execute(t->left, pin, pout, 0);
		rv = execute(t->right, pin, pout, 0);
		break;

	case TASYNC:
	{
		int hinteractive = interactive;

		i = vfork();
		if (i != 0) {
			interactive = hinteractive;
			if (i != -1) {
				setval(lookup("!"), putn(i));
				if (pin != NULL)
					closepipe(pin);
				if (interactive) {
					prs(putn(i));
					prs("\n");
				}
			} else
				rv = -1;
			setstatus(rv);
		} else {
			signal(SIGINT, SIG_IGN);
			signal(SIGQUIT, SIG_IGN);
			if (interactive)
				signal(SIGTERM, SIG_DFL);
			interactive = 0;
			if (pin == NULL) {
				close(0);
				open("/dev/null", 0);
			}
			exit(execute(t->left, pin, pout, FEXEC));
		}
	}
		break;

	case TOR:
	case TAND:
		rv = execute(t->left, pin, pout, 0);
		if ((t1 = t->right)!=NULL && (rv == 0) == (t->type == TAND))
			rv = execute(t1, pin, pout, 0);
		break;

	case TFOR:
		if (wp == NULL) {
			wp = dolv+1;
			if ((i = dolc) < 0)
				i = 0;
		} else {
			i = -1;
			while (*wp++ != NULL)
				;			
		}
		vp = lookup(t->str);
		while (setjmp(bc.brkpt))
			if (isbreak)
				goto broken;
		brkset(&bc);
		for (t1 = t->left; i-- && *wp != NULL;) {
			setval(vp, *wp++);
			rv = execute(t1, pin, pout, 0);
		}
		brklist = brklist->nextlev;
		break;

	case TWHILE:
	case TUNTIL:
		while (setjmp(bc.brkpt))
			if (isbreak)
				goto broken;
		brkset(&bc);
		t1 = t->left;
		while ((execute(t1, pin, pout, 0) == 0) == (t->type == TWHILE))
			rv = execute(t->right, pin, pout, 0);
		brklist = brklist->nextlev;
		break;

	case TIF:
	case TELIF:
	 	if (t->right != NULL) {
		rv = !execute(t->left, pin, pout, 0) ?
			execute(t->right->left, pin, pout, 0):
			execute(t->right->right, pin, pout, 0);
		}
		break;

	case TCASE:
		if ((cp = evalstr(t->str, DOSUB|DOTRIM)) == 0)
			cp = "";
		if ((t1 = findcase(t->left, cp)) != NULL)
			rv = execute(t1, pin, pout, 0);
		break;

	case TBRACE:
/*
		if (iopp = t->ioact)
			while (*iopp)
				if (iosetup(*iopp++, pin!=NULL, pout!=NULL)) {
					rv = -1;
					break;
				}
*/
		if (rv >= 0 && (t1 = t->left))
			rv = execute(t1, pin, pout, 0);
		break;
	}

broken:
	t->words = wp2;
	isbreak = 0;
	freehere(areanum);
	freearea(areanum);
	areanum = a;
	if (interactive && intr) {
		closeall();
		fail();
	}
	if ((i = trapset) != 0) {
		trapset = 0;
		runtrap(i);
	}
	return(rv);
}


/*
 * 0< 1> are ignored as required
 * within pipelines.
 */
static int
iosetup(iop, pipein, pipeout)
register struct ioword *iop;
int pipein, pipeout;
{
	register int u = -1;
	char *cp=NULL, *msg;

	if (iop->io_unit == IODEFAULT)	/* take default */
		iop->io_unit = iop->io_flag&(IOREAD|IOHERE)? 0: 1;
	if (pipein && iop->io_unit == 0)
		return(0);
	if (pipeout && iop->io_unit == 1)
		return(0);
	msg = iop->io_flag&(IOREAD|IOHERE)? "open": "create";
	if ((iop->io_flag & IOHERE) == 0) {
		cp = iop->io_name;
		if ((cp = evalstr(cp, DOSUB|DOTRIM)) == NULL)
			return(1);
	}
	if (iop->io_flag & IODUP) {
		if (cp[1] || (!isdigit(*cp) && *cp != '-')) {
			prs(cp);
			err(": illegal >& argument");
			return(1);
		}
		if (*cp == '-')
			iop->io_flag = IOCLOSE;
		iop->io_flag &= ~(IOREAD|IOWRITE);
	}
	switch (iop->io_flag) {
	case IOREAD:
		u = open(cp, 0);
		break;

	case IOHERE:
	case IOHERE|IOXHERE:
		u = herein(iop->io_name, iop->io_flag&IOXHERE);
		cp = "here file";
		break;

	case IOWRITE|IOCAT:
		if ((u = open(cp, 1)) >= 0) {
			lseek(u, (long)0, 2);
			break;
		}
	case IOWRITE:
		u = creat(cp, 0666);
		break;

	case IODUP:
		u = dup2(*cp-'0', iop->io_unit);
		break;

	case IOCLOSE:
		close(iop->io_unit);
		return(0);
	}
	if (u < 0) {
		prs(cp);
		prs(": cannot ");
		warn(msg);
		return(1);
	} else {
		if (u != iop->io_unit) {
			dup2(u, iop->io_unit);
			close(u);
		}
	}
	return(0);
}

static struct op **
find1case(t, w)
struct op *t;
char *w;
{
	register struct op *t1;
	struct op **tp;
	register char **wp, *cp;

	if (t == NULL)
		return((struct op **)NULL);
	if (t->type == TLIST) {
		if ((tp = find1case(t->left, w)) != NULL)
			return(tp);
		t1 = t->right;	/* TPAT */
	} else
		t1 = t;
	for (wp = t1->words; *wp;)
		if ((cp = evalstr(*wp++, DOSUB)) && gmatch(w, cp))
			return(&t1->left);
	return((struct op **)NULL);
}

static struct op *
findcase(t, w)
struct op *t;
char *w;
{
	register struct op **tp;

	return((tp = find1case(t, w)) != NULL? *tp: (struct op *)NULL);
}

/*
 * Enter a new loop level (marked for break/continue).
 */
static void
brkset(bc)
struct brkcon *bc;
{
	bc->nextlev = brklist;
	brklist = bc;
}

/*
 * Wait for the last process created.
 * Print a message for each process found
 * that was killed by a signal.
 * Ignore interrupt signals while waiting
 * unless `canintr' is true.
 */
static int
waitfor(lastpid, canintr)
register int lastpid;
int canintr;
{
	register int pid, rv;
	int s;
	int oheedint = heedint;

	heedint = 0;
	rv = 0;
	do {
		pid = wait(&s);
		if (pid == -1) {
			if (errno != EINTR || canintr)
				break;
		} else {
			if ((rv = WAITSIG(s)) != 0) {
				if (rv < NSIGNAL) {
					if (signame[rv] != NULL) {
						if (pid != lastpid) {
							prn(pid);
							prs(": ");
						}
						prs(signame[rv]);
					}
				} else {
					if (pid != lastpid) {
						prn(pid);
						prs(": ");
					}
					prs("Signal "); prn(rv); prs(" ");
				}
				if (WAITCORE(s))
					prs(" - core dumped");
				if (rv >= NSIGNAL || signame[rv])
					prs("\n");
				rv = -1;
			} else
				rv = WAITVAL(s);
		}
	} while (pid != lastpid);
	heedint = oheedint;
	if (intr) {
		if (interactive) {
			if (canintr)
				intr = 0;
		} else {
			if (exstat == 0) exstat = rv;
			onintr(0);
		}
	}
	return(rv);
}

static int
setstatus(s)
register int s;
{
	exstat = s;
	setval(lookup("?"), putn(s));
	return(s);
}

/*
 * Run the command produced by generator `f'
 * applied to stream `arg'.
 */
static int
run(argp, f)
struct ioarg *argp;
int (*f)();
{
	struct op *otree;
	struct wdblock *swdlist;
	struct wdblock *siolist;
	jmp_buf ev, rt;
	xint *ofail;
	int rv;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &rv;
#endif

	areanum++;
	swdlist = wdlist;
	siolist = iolist;
	otree = outtree;
	ofail = failpt;
	rv = -1;
	if (newenv(setjmp(errpt = ev)) == 0) {
		wdlist = 0;
		iolist = 0;
		pushio(argp, f);
		e.iobase = e.iop;
		yynerrs = 0;
		if (setjmp(failpt = rt) == 0 && yyparse() == 0)
			rv = execute(outtree, NOPIPE, NOPIPE, 0);
		quitenv();
	}
	wdlist = swdlist;
	iolist = siolist;
	failpt = ofail;
	outtree = otree;
	freearea(areanum--);
	return(rv);
}

static void run_script(char *str) {
	int pid;
	char *tmpBuf;
	//disable_ttyecho();
	tmpBuf=(char *)malloc(BUFFSIZE);
	snprintf(tmpBuf, BUFFSIZE, "%s/%s %s %s %s %s", _CONFIG_SCRIPT_PATH, _CONFIG_SCRIPT_PROG,
		_GW, str, _CLI, _NO_SCRIPT_MSG);
	system(tmpBuf);
	free(tmpBuf);
	//enable_ttyecho();
}
static int checkRemotePort(int portnum) {
	if (portnum < 0 || portnum > 65535 ) {
		printf("Valid port number: 0 ~ 65535\n");  
		return(0);
	}
	return(1);
}
static int isConnectPPP()
{
	struct stat status;

	if ( stat("/etc/ppp/link", &status) < 0)
		return(0);

	return(1);
}
static int isDhcpClientExist(char *name)
{
	char *tmpBuf;
	struct in_addr intaddr;

	tmpBuf=(char *)malloc(BUFFSIZE);
	if ( getInAddr(name, IP_ADDR, (void *)&intaddr ) ) {
		snprintf(tmpBuf, BUFFSIZE, "%s/%s-%s.pid", _DHCPC_PID_PATH, _DHCPC_PROG_NAME, name);
		if ( getPid(tmpBuf) > 0)
			return(1);
	}
	free(tmpBuf);
	return(0);
}

static int getPid(char *filename)
{
	struct stat status;
	char *buff;
	FILE *fp;

	buff=(char *)malloc(BUFFSIZE);

	if ( stat(filename, &status) < 0)
		return -1;
	fp = fopen(filename, "r");
	if (!fp) {
        	//error(E_L, E_LOG, T("Read pid file error!\n"));
		return -1;
   	}
	fgets(buff, BUFFSIZE, fp);
	fclose(fp);
	free(buff);

	return (atoi(buff));
}

static bool _is_hex(char c)
{
    return (((c >= '0') && (c <= '9')) ||
            ((c >= 'A') && (c <= 'F')) ||
            ((c >= 'a') && (c <= 'f')));
}

// Validate digit
static bool _isdigit(char c) {
    return ((c >= '0') && (c <= '9'));
}

static int __inline__ string_to_hex(char *string, unsigned char *key, int len)
{
	char tmpBuf[4];
	int idx, ii=0;
	for (idx=0; idx<len; idx+=2) {
		tmpBuf[0] = string[idx];
		tmpBuf[1] = string[idx+1];
		tmpBuf[2] = 0;
		if ( !_is_hex(tmpBuf[0]) || !_is_hex(tmpBuf[1]))
			return 0;

		key[ii++] = (unsigned char) strtol(tmpBuf, (char**)NULL, 16);
	}
	return 1;
}

static int __inline__ string_to_dec(char *string, int *val)
{
	int idx;
	int len = strlen(string);

	for (idx=0; idx<len; idx++) {
		if ( !_isdigit(string[idx]))
			return 0;
	}

	*val = strtol(string, (char**)NULL, 10);
	return 1;
}

static void invalid_subcmds(char *str) {
	printf("valid subcommands are: \n");
	printf("-------------------\n");
	printf(str);
}

static void dns_dnrd(int dn, struct op *t) {
	char *dnrdbuff;
	dnrdbuff=(char *)malloc(BUFFSIZE);
	switch(dn) {
		case 1:
			sprintf(dnrdbuff,"%s -s %s",_DNRD_PROG_NAME,t->words[2]);
			break;
		case 2:
			sprintf(dnrdbuff,"%s -s %s -s %s",_DNRD_PROG_NAME,t->words[2],t->words[3]);
			break;
		case 3:
			sprintf(dnrdbuff,"%s -s %s -s %s -s %s",_DNRD_PROG_NAME,t->words[2],t->words[3],t->words[4]);
			break;
	}
	system(dnrdbuff);
	free(dnrdbuff);
}

static void ppp_format(char *strn) {
	if(!strcmp(strn,"pppoe"))
		printf("format: wan pppoe <username> <password> <auth_type> <mppe> <mtu_values> <wan_ip> <wan_netmask> [static <ip_addr> <remote_ip_addr> <remote_netmask>|dynamic]\n");
	else
		printf("format: wan %s <ip [server_ip]|url [server_url]> <username> <password> <auth_type> <mppe> <mtu [value]> [static <ip_addr> <netmask> <gateway>|dynamic]\n",strn);
	printf("\tauth_type:\tmppe:\tmtu_values: 1400~1492\n");
	printf("\t0. PAP     \t0. None\n");
	printf("\t1. CHAP   \t1. 40bits\n");
	printf("\t2. MSCHAP\t2. 56bits\n");
	printf("\t3. MSCHAP2\t3. 128bits\n");
	wan_status();
}

static void ifconfig_format() {
	printf("format: ip ifconfig [iface] [ipaddr] [broadcast <addr> | mtu <value> | dynamic]\n");
	printf("\tiface:\n");
	printf("\t1. WAN\n");
	printf("\t2. LAN\n");
}

static void nat_service_edit_format(char *strn) {
	printf("format: ip nat service %s edit <name> <in_proto> <in_from_port> <in_to_port> <trig_proto> <trig_from_port> <trig_to_port>\n",strn);
	printf("\tprotocol:\n");
	printf("\t1. TCP+UDP\n");
	printf("\t2. TCP\n");
	printf("\t3. UDP\n");
}

static void nat_server_add_format() {
	printf("format: ip nat server add <ip addr> <server> [protocol <from port [to port]>]\n");
	printf("\tserver:\t\tprotocol:\n");
	printf("\t0. Customize\n");
	printf("\t1. WEB\t\t1. TCP+UDP\n");
	printf("\t2. FTP\t\t2. TCP\n");
	printf("\t3. POP3\t\t3. UDP\n");
	printf("\t4. SMTP\n");
	printf("\t5. DNS\n");
	printf("\t6. telnet\n");
}

static void restart_daemon(char *name) {
	char *buff;
	buff=(char *)malloc(BUFFSIZE);
	sprintf(buff,"%s %s &",name,_NO_SCRIPT_MSG);
	system(buff);
	free(buff);
}

static void tunnel_control(struct op *t,int nc) {

	struct in_addr eIp,eMask,eGateway;
	int auth,mppe,mtu,server_mode=0,connect_type=0,ipmode=0,wan_dhcp;
	int uname_mib,upwd_mib,mtu_mib,auth_mib,ser_info_mib,ser_ip_mib,ser_url_mib,mppe_mib,ipmode_mib,ip_mib,subnet_mib,gw_mib;
	char *uname,*upwd,*ser_url;
	unsigned long v1, v2, v3;

	if(nc < 9) {
		ppp_format(t->words[0]);
		return(0);
	}

	if(!strcmp(t->words[0],"pptp")) {
		wan_dhcp=4;
		mppe_mib=MIB_MPPE_LEVEL;
		gw_mib=MIB_WAN_DEFAULT_GATEWAY;
		ip_mib=MIB_PPTP_IP;
		subnet_mib=MIB_PPTP_SUBNET_MASK;
		uname_mib=MIB_PPTP_USER;
		upwd_mib=MIB_PPTP_PASSWORD;
		mtu_mib=MIB_PPTP_MTU_SIZE;
		auth_mib=MIB_PPTP_AUTH_TYPE;
		ser_ip_mib=MIB_PPTP_SERVER_IP;
#if INCLUDE_SPECIAL_PPP
		ipmode_mib=MIB_PPTP_IPMODE;
		ser_url_mib=MIB_PPTP_SERVER_URL;
		ser_info_mib=MIB_PPTP_SERVER_INFO;
#else
		ipmode_mib=NULL;
		ser_url_mib=NULL;
		ser_info_mib=NULL;
#endif
	}
	else if(!strcmp(t->words[0],"l2") || !strcmp(t->words[0],"l2tp")) {
		wan_dhcp=5;
		mppe_mib=MIB_MPPE_LEVEL;
		gw_mib=MIB_WAN_DEFAULT_GATEWAY;
		ip_mib=MIB_L2TP_IP;
		subnet_mib=MIB_L2TP_SUBNET_MASK;
		ipmode_mib=MIB_L2TP_IPMODE;
		uname_mib=MIB_L2TP_USER;
		upwd_mib=MIB_L2TP_PASSWORD;
		mtu_mib=MIB_L2TP_MTU_SIZE;
		auth_mib=MIB_L2TP_AUTH_TYPE;
		ser_ip_mib=MIB_L2TP_SERVER_IP;
		// note that, the url style server information has not implemented on l2tp now.
		ser_url_mib=NULL;
		ser_info_mib=NULL;
	}
	
	if(!strcmp(t->words[nc-(nc-1)],"ip")) {
		if (!inet_aton(t->words[nc-(nc-2)], &eIp)) {
			printf("Invalid IP address input !!!\n");
			return(0);
		}
		if(!apmib_set(ser_ip_mib, (void *)&eIp)) {
			printf("\tSet Server IP address MIB Error\n");
			return(0);
		}
#if INCLUDE_SPECIAL_PPP
		if(wan_dhcp==4) {
			if(!apmib_set(ser_info_mib, (void *)&server_mode)) {
				printf("\tSet Server Info MIB Error\n");
				return(0);
			}
		}
#endif
	}
	else if(!strcmp(t->words[nc-(nc-1)],"url")) {
#if INCLUDE_SPECIAL_PPP
		if(wan_dhcp==4) {
			ser_url=t->words[nc-(nc-2)];
			if(!apmib_set(ser_url_mib, (void *)ser_url)) {
				printf("\tSet Server URL MIB Error\n");
				return(0);
			}
			server_mode=1;
			if(!apmib_set(ser_info_mib, (void *)&server_mode)) {
				printf("\tSet Server Info MIB Error\n");
				return(0);
			}
		}
#endif
	}
	else {
		if(wan_dhcp==4)
			ppp_format("pptp");
		else if(wan_dhcp==5)
			ppp_format("l2tp");
		return(0);
	}

	uname=t->words[nc-(nc-3)];
	if(strlen(uname) > MAX_NAME_LEN) {
		printf("Invalid username length, max lenght is 31 characters !!!\n");
		return(0);
	}
	upwd=t->words[nc-(nc-4)];
	auth=atoi(t->words[nc-(nc-5)]);
	if(auth<0 || auth>3) {
		printf("Invalid auth type, It should be set at 0 ~ 3 !!!\n");
		return(0);
	}
	mppe=atoi(t->words[nc-(nc-6)]);
	if(mppe<0 || mppe>3) {
		printf("Invalid mppe level, It should be set at 0 ~ 3 !!!\n");
		return(0);
	}
	mtu=strtol(t->words[nc-(nc-7)], (char**)NULL, 10);
	if(mtu<1400 || mtu>1492) {
		printf("Invalid mtu values, It should be set at 1400 ~ 1492 !!!\n");
		return(0);
	}

	if(!apmib_set(uname_mib, (void *)uname)) { // user name
		printf("\tSet USER MIB Error\n");
		return(0);
	}
	if(!apmib_set(upwd_mib, (void *)upwd)) { // password
		printf("\tSet PASSWORD MIB Error\n");
		return(0);
	}
	if(!apmib_set(mtu_mib, (void *)&mtu)) { // mtu
		printf("\tSet MTU_SIZE MIB Error\n");
		return(0);
	}
	if(!apmib_set(auth_mib, (void *)&auth)) { // auth type
		printf("\tSet AUTH_TYPE MIB Error\n");
		return(0);
	}
	if(!apmib_set(mppe_mib, (void *)&mppe)) { // mppe
		printf("\tSet MPPE_LEVEL MIB Error\n");
		return(0);
	}

	if(!strcmp(t->words[nc-(nc-8)],"dyn") || !strcmp(t->words[nc-(nc-8)],"dynam") || !strcmp(t->words[nc-(nc-8)],"dynamic")) {
#if INCLUDE_SPECIAL_PPP
		if(!apmib_set(ipmode_mib, (void *)&ipmode)) { // ip mode
			printf("\tSet IPMODE MIB Error\n");
			return(0);
		}
#else
		if(wan_dhcp == 5) {
			if(!apmib_set(ipmode_mib, (void *)&ipmode)) { // ip mode
				printf("\tSet IPMODE MIB Error\n");
				return(0);
			}
		}
#endif
	}
	else if(!strcmp(t->words[nc-(nc-8)],"stat") || !strcmp(t->words[nc-(nc-8)],"static")) {
		if(nc==10 || nc==12) {
			if (!inet_aton(t->words[nc-(nc-9)], &eIp)) {
				printf("Invalid IP address input !!!\n");
				return(0);
			}
			if(nc == 12) {
				if (!inet_aton(t->words[nc-(nc-10)], &eMask)) {
					printf("Invalid Netmask input !!!\n");
					return(0);
				}
				if (!inet_aton(t->words[nc-(nc-11)], &eGateway)) {
					printf("Invalid Gateway address input !!!\n");
					return(0);
				}
				v1 = *((unsigned long *)&eGateway);
				v2 = *((unsigned long *)&eIp);
				v3 = *((unsigned long *)&eMask);
				if ( (v1 & v3) != (v2 & v3) )	{
					printf("Invalid IP or Gateway address! It should be set within the same subnet !!!\n");
					return(0);
				}
				if (!apmib_set(subnet_mib, (void *)&eMask)) {
					printf("\tSet SUBNET_MASK MIB Error\n");
					return(0);
				}
				if (!apmib_set(gw_mib, (void *)&eGateway)) {
					printf("\tSet WAN_DEFAULT_GATEWAY MIB Error\n");
					return(0);
				}
			}
			if (!apmib_set(ip_mib, (void *)&eIp)) {
				printf("\tSet IP address MIB Error\n");
				return(0);
			}
#if INCLUDE_SPECIAL_PPP
			ipmode=1;
			if(!apmib_set(ipmode_mib, (void *)&ipmode)) { // ip mode
				printf("\tSet IPMODE MIB Error\n");
				return(0);
			}
#else
			if(wan_dhcp == 5) {
				ipmode=1;
				if(!apmib_set(ipmode_mib, (void *)&ipmode)) { // ip mode
					printf("\tSet IPMODE MIB Error\n");
					return(0);
				}
			}
#endif
		}
		else {
			ppp_format(t->words[0]);
			return(0);
		}
	}
	else {
		ppp_format(t->words[0]);
		return(0);
	}

	if(!apmib_set(MIB_WAN_DHCP, (void *)&wan_dhcp)) {
		printf("\tSet WAN_DHCP MIB Error\n");
		return(0);
	}
	
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
}

static void pppoe_control(struct op *t,int nc) {
	char *buff,*uname,*upwd;
	int auth,mppe,mtu,connect_type=0,ipmode=0,wan_dhcp=3;
	struct in_addr eIp,eIp_remote,eMask_remote,wan_Ip,wan_Mask;
	unsigned long v1, v2, v3;

	if(nc < 9) {
		ppp_format("pppoe");
		return(0);
	}
	
	uname=t->words[nc-(nc-1)];
	if(strlen(uname) > MAX_NAME_LEN) {
		printf("Invalid username length, max lenght is %d characters !!!\n",MAX_NAME_LEN);
		return(0);
	}
	upwd=t->words[nc-(nc-2)];
	auth=atoi(t->words[nc-(nc-3)]);
	if(auth<0 || auth>3) {
		printf("Invalid auth type, It should be set at 0 ~ 3 !!!\n");
		return(0);
	}
	mppe=atoi(t->words[nc-(nc-4)]);
	if(mppe<0 || mppe>3) {
		printf("Invalid mppe level, It should be set at 0 ~ 3 !!!\n");
		return(0);
	}
	mtu=strtol(t->words[nc-(nc-5)], (char**)NULL, 10);
	if(mtu<1400 || mtu>1492) {
		printf("Invalid mtu values, It should be set at 1400 ~ 1492 !!!\n");
		return(0);
	}

	if(!apmib_set(MIB_PPP_USER, (void *)uname)) { // user name
		printf("\tSet PPP_USER MIB Error\n");
		return(0);
	}
	if(!apmib_set(MIB_PPP_PASSWORD, (void *)upwd)) { // password
		printf("\tSet PPP_PASSWORD MIB Error\n");
		return(0);
	}
	if(!apmib_set(MIB_PPP_CONNECT_TYPE, (void *)&connect_type)) { // connect type
		printf("\tSet CONNECT_TYPE MIB Error\n");
		return(0);
	}
	if(!apmib_set(MIB_PPP_MTU_SIZE, (void *)&mtu)) { // mtu
		printf("\tSet PPP_MTU_SIZE MIB Error\n");
		return(0);
	}
	if(!apmib_set(MIB_PPP_AUTH_TYPE, (void *)&auth)) { // auth type
		printf("\tSet PPP_AUTH_TYPE MIB Error\n");
		return(0);
	}
	if(!apmib_set(MIB_MPPE_LEVEL, (void *)&mppe)) { // mppe
		printf("\tSet MPPE_LEVEL MIB Error\n");
		return(0);
	}

	if (!inet_aton(t->words[nc-(nc-6)], &wan_Ip)) {
		printf("Invalid WAN IP address input !!!\n");
		return(0);
	}

	if ( !apmib_set(MIB_PHY_WAN_IP_ADDR, (void *)&wan_Ip)) {
		printf("\tSet PHY_WAN_IP_ADDR MIB Error\n");
		return(0);
	}

	if (!inet_aton(t->words[nc-(nc-7)], &wan_Mask)) {
		printf("Invalid Remote Netmask input !!!\n");
		return(0);
	}

	if ( !apmib_set(MIB_PHY_WAN_SUBNET_MASK, (void *)&wan_Mask)) {
		printf("\tSet PHY_WAN_SUBNET_MASK MIB Error\n");
		return(0);
	}
	
	if(!strcmp(t->words[nc-(nc-8)],"dyn") || !strcmp(t->words[nc-(nc-8)],"dynam") || !strcmp(t->words[nc-(nc-8)],"dynamic")) {
#if INCLUDE_SPECIAL_PPP		
		if(!apmib_set(MIB_PPP_IPMODE, (void *)&ipmode)) { // ip mode
			printf("\tSet PPP_IPMODE MIB Error\n");
			return(0);
		}
#else
		printf("PPPoE IPMODE not supported in this model !!!\n");
		return(0);
#endif
	}
	else if(!strcmp(t->words[nc-(nc-8)],"stat") || !strcmp(t->words[nc-(nc-8)],"static")) {
#if INCLUDE_SPECIAL_PPP	
		if(nc==10 || nc==12) {
			if (!inet_aton(t->words[nc-(nc-9)], &eIp)) {
				printf("Invalid IP address input !!!\n");
				return(0);
			}
			if(nc == 12) {
				if (!inet_aton(t->words[nc-(nc-10)], &eIp_remote)) {
					printf("Invalid Remote IP address input !!!\n");
					return(0);
				}
				if (!inet_aton(t->words[nc-(nc-11)], &eMask_remote)) {
					printf("Invalid Remote Netmask input !!!\n");
					return(0);
				}
				v1 = *((unsigned long *)&eIp);
				v2 = *((unsigned long *)&eIp_remote);
				v3 = *((unsigned long *)&eMask_remote);
				if ( (v1 & v3) != (v2 & v3) )	{
					printf("Invalid IP or Remote IP address! It should be set within the same subnet !!!\n");
					return(0);
				}
				if (!apmib_set(MIB_PPP_REMOTE_IP, (void *)&eIp_remote)) {
					printf("\tSet PPP_REMOTE_IP MIB Error\n");
					return(0);
				}
				if (!apmib_set(MIB_PPP_REMOTE_SUBNET_MASK, (void *)&eMask_remote)) {
					printf("\tSet PPP_REMOTE_SUBNET_MASK MIB Error\n");
					return(0);
				}
			}
			if (!apmib_set(MIB_PPP_IP, (void *)&eIp)) {
				printf("\tSet PPP_IP MIB Error\n");
				return(0);
			}
			ipmode=1;
			if(!apmib_set(MIB_PPP_IPMODE, (void *)&ipmode)) { // ip mode
				printf("\tSet PPP_IPMODE MIB Error\n");
				return(0);
			}
		}
		else {
			ppp_format("pppoe");
			return(0);
		}
#else
		printf("PPPoE Static not supported in this model !!!\n");
		return(0);
#endif
	}
	else {
		ppp_format("pppoe");
		return(0);
	}

	if(!apmib_set(MIB_WAN_DHCP, (void *)&wan_dhcp)) {
		printf("\tSet WAN_DHCP MIB Error\n");
		return(0);
	}
	
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();

}


static void sap_disp() {
	int spap_num=8;
	int spap_id[spap_num],i;
	char *spap_name[spap_num];

	for(i=0;i<spap_num;i++)
		spap_name[i]=(char *)malloc(BUFFSIZE);
	
	if (!apmib_get( MIB_SPAP_ENABLE1, (void *)&spap_id[0])) {
		printf("\tGet SPAP_ENABLE1 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME1, (void *)spap_name[0])) {
		printf("\tGet SPAP_NAME1 MIB Error\n");
		return(0);
	}
	
	if (!apmib_get( MIB_SPAP_ENABLE2, (void *)&spap_id[1])) {
		printf("\tGet SPAP_ENABLE2 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME2, (void *)spap_name[1])) {
		printf("\tGet SPAP_NAME2 MIB Error\n");
		return(0);
	}
	
	if (!apmib_get( MIB_SPAP_ENABLE3, (void *)&spap_id[2])) {
		printf("\tGet SPAP_ENABLE3 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME3, (void *)spap_name[2])) {
		printf("\tGet SPAP_NAME3 MIB Error\n");
		return(0);
	}

	if (!apmib_get( MIB_SPAP_ENABLE4, (void *)&spap_id[3])) {
		printf("\tGet SPAP_ENABLE4 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME4, (void *)spap_name[3])) {
		printf("\tGet SPAP_NAME4 MIB Error\n");
		return(0);
	}

	if (!apmib_get( MIB_SPAP_ENABLE5, (void *)&spap_id[4])) {
		printf("\tGet SPAP_ENABLE5 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME5, (void *)spap_name[4])) {
		printf("\tGet SPAP_NAME5 MIB Error\n");
		return(0);
	}

	if (!apmib_get( MIB_SPAP_ENABLE6, (void *)&spap_id[5])) {
		printf("\tGet SPAP_ENABLE6 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME6, (void *)spap_name[5])) {
		printf("\tGet SPAP_NAME6 MIB Error\n");
		return(0);
	}

	if (!apmib_get( MIB_SPAP_ENABLE7, (void *)&spap_id[6])) {
		printf("\tGet SPAP_ENABLE7 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME7, (void *)spap_name[6])) {
		printf("\tGet SPAP_NAME7 MIB Error\n");
		return(0);
	}

	if (!apmib_get( MIB_SPAP_ENABLE8, (void *)&spap_id[7])) {
		printf("\tGet SPAP_ENABLE8 MIB Error\n");
		return(0);
	}
	if (!apmib_get( MIB_SPAP_NAME8, (void *)spap_name[7])) {
		printf("\tGet SPAP_NAME8 MIB Error\n");
		return(0);
	}

	printf("=========================================\n");
	printf("ID\tName\t\tStatus\n");
	for(i=0;i<spap_num;i++) {
		if(spap_id[i])
			printf("sap%d\t%s\t\ton\n",i+1,spap_name[i]);
		else
			printf("sap%d\t%s\t\toff\n",i+1,spap_name[i]);
		free(spap_name[i]);
	}
	printf("=========================================\n");
}

static void sap_control(struct op *t,int nc) {
	int idx,enable,doenable,t_port;;
	char *buff,*iname;
	int spap_enable,spap_name,spap_intype,spap_trtype,spap_instart,spap_inend,spap_trstart,spap_trend;

	int iin_p,iin_fp,iin_tp,itr_p,itr_fp,itr_tp;
	
	idx=(int)t->words[2][3]-48;
	buff=(char *)malloc(BUFFSIZE);
	
	switch(idx) {
		case 1:
			spap_enable=MIB_SPAP_ENABLE1;
			spap_name=MIB_SPAP_NAME1;
			spap_intype=MIB_SPAP_INTYPE1;
			spap_trtype=MIB_SPAP_TRTYPE1;
			spap_instart=MIB_SPAP_INSTART1;
			spap_inend=MIB_SPAP_INEND1;
			spap_trstart=MIB_SPAP_TRSTART1;
			spap_trend=MIB_SPAP_TREND1;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE1 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME1 MIB Error\n");
				return(0);
			}
			break;
		case 2:
			spap_enable=MIB_SPAP_ENABLE2;
			spap_name=MIB_SPAP_NAME2;
			spap_intype=MIB_SPAP_INTYPE2;
			spap_trtype=MIB_SPAP_TRTYPE2;
			spap_instart=MIB_SPAP_INSTART2;
			spap_inend=MIB_SPAP_INEND2;
			spap_trstart=MIB_SPAP_TRSTART2;
			spap_trend=MIB_SPAP_TREND2;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE2 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME2 MIB Error\n");
				return(0);
			}
			break;
		case 3:
			spap_enable=MIB_SPAP_ENABLE3;
			spap_name=MIB_SPAP_NAME3;
			spap_intype=MIB_SPAP_INTYPE3;
			spap_trtype=MIB_SPAP_TRTYPE3;
			spap_instart=MIB_SPAP_INSTART3;
			spap_inend=MIB_SPAP_INEND3;
			spap_trstart=MIB_SPAP_TRSTART3;
			spap_trend=MIB_SPAP_TREND3;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE3 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME3 MIB Error\n");
				return(0);
			}
			break;
		case 4:
			spap_enable=MIB_SPAP_ENABLE4;
			spap_name=MIB_SPAP_NAME4;
			spap_intype=MIB_SPAP_INTYPE4;
			spap_trtype=MIB_SPAP_TRTYPE4;
			spap_instart=MIB_SPAP_INSTART4;
			spap_inend=MIB_SPAP_INEND4;
			spap_trstart=MIB_SPAP_TRSTART4;
			spap_trend=MIB_SPAP_TREND4;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE4 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME4 MIB Error\n");
				return(0);
			}
			break;
		case 5:
			spap_enable=MIB_SPAP_ENABLE5;
			spap_name=MIB_SPAP_NAME5;
			spap_intype=MIB_SPAP_INTYPE5;
			spap_trtype=MIB_SPAP_TRTYPE5;
			spap_instart=MIB_SPAP_INSTART5;
			spap_inend=MIB_SPAP_INEND5;
			spap_trstart=MIB_SPAP_TRSTART5;
			spap_trend=MIB_SPAP_TREND5;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE5 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME5 MIB Error\n");
				return(0);
			}
			break;
		case 6:
			spap_enable=MIB_SPAP_ENABLE6;
			spap_name=MIB_SPAP_NAME6;
			spap_intype=MIB_SPAP_INTYPE6;
			spap_trtype=MIB_SPAP_TRTYPE6;
			spap_instart=MIB_SPAP_INSTART6;
			spap_inend=MIB_SPAP_INEND6;
			spap_trstart=MIB_SPAP_TRSTART6;
			spap_trend=MIB_SPAP_TREND6;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE6 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME6 MIB Error\n");
				return(0);
			}
			break;
		case 7:
			spap_enable=MIB_SPAP_ENABLE7;
			spap_name=MIB_SPAP_NAME7;
			spap_intype=MIB_SPAP_INTYPE7;
			spap_trtype=MIB_SPAP_TRTYPE7;
			spap_instart=MIB_SPAP_INSTART7;
			spap_inend=MIB_SPAP_INEND7;
			spap_trstart=MIB_SPAP_TRSTART7;
			spap_trend=MIB_SPAP_TREND7;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE7 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME7 MIB Error\n");
				return(0);
			}
			break;
		case 8:
			spap_enable=MIB_SPAP_ENABLE8;
			spap_name=MIB_SPAP_NAME8;
			spap_intype=MIB_SPAP_INTYPE8;
			spap_trtype=MIB_SPAP_TRTYPE8;
			spap_instart=MIB_SPAP_INSTART8;
			spap_inend=MIB_SPAP_INEND8;
			spap_trstart=MIB_SPAP_TRSTART8;
			spap_trend=MIB_SPAP_TREND8;
			if (!apmib_get( spap_enable, (void *)&enable)) {
				printf("\tGet SPAP_ENABLE8 MIB Error\n");
				return(0);
			}
			if (!apmib_get( spap_name, (void *)buff)) {
				printf("\tGet SPAP_NAME8 MIB Error\n");
				return(0);
			}
			break;
	}
	
	if(!strcmp(t->words[3],"on")) {
		if(strcmp(buff,"")) {
			if(enable)
				printf("Now: %s [%s] on\n",t->words[2],buff);
			else {
				doenable=1;
				if (!apmib_set( spap_enable, (void *)&doenable)) {
					printf("\tSet SPAP MIB Error\n");
					return(0);
				}
				apmib_update(CURRENT_SETTING);
				restart_daemon(_WEBS_PROG_NAME);
				restart_daemon(_FIREWALL_SCRIPT_PROG);
				printf("Turn on %s [%s]\n",t->words[2],buff);
			}
		}
		else
			printf("No Service Setup at %s !!!\n",t->words[2]);
	}
	else if(!strcmp(t->words[3],"off")) {
		if(strcmp(buff,"")) {
			if(enable) {
				doenable=0;
				if (!apmib_set( spap_enable, (void *)&doenable)) {
					printf("\tSet SPAP MIB Error\n");
					return(0);
				}
				apmib_update(CURRENT_SETTING);
				restart_daemon(_WEBS_PROG_NAME);
				restart_daemon(_FIREWALL_SCRIPT_PROG);
				printf("Turn off %s [%s]\n",t->words[2],buff);
			}
			else
				printf("Now: %s [%s] off\n",t->words[2],buff);
		}
		else
			printf("No Service Setup at %s !!!\n",t->words[2]);
	}
	else if(!strcmp(t->words[3],"cle") || !strcmp(t->words[3],"clear")) { //clear up
		doenable=0;
		iname="";
		iin_p=PROTO_TCP;
		itr_p=PROTO_TCP;
		iin_fp=0;
		iin_tp=0;
		itr_fp=0;
		itr_tp=0;
		if(!apmib_set( spap_name, (void *)iname) || !apmib_set( spap_intype, (void *)&iin_p) || !apmib_set( spap_trtype, (void *)&itr_p) || !apmib_set( spap_instart, (void *)&iin_fp) || !apmib_set( spap_inend, (void *)&iin_tp) || !apmib_set( spap_trstart, (void *)&itr_fp) || !apmib_set( spap_trend, (void *)&itr_tp)) {
			printf("\tSet SPAP MIB Error\n");
			return(0);
		}
		if(!apmib_set( spap_enable, (void *)&doenable)) {
			printf("\tSet SPAP MIB Error\n");
			return(0);
		}
		apmib_update(CURRENT_SETTING);
		restart_daemon(_WEBS_PROG_NAME);
		restart_daemon(_FIREWALL_SCRIPT_PROG);
		printf("Clearup %s Successfully !!!\n",t->words[2]);
	}
	else if(!strcmp(t->words[3],"ed") || !strcmp(t->words[3],"edit")) {
		if(nc == 11) {
			switch(atoi(t->words[5])) { // incoming protocol
				case 1:
					iin_p=PROTO_BOTH;
					break;
				case 2:
					iin_p=PROTO_TCP;
					break;
				case 3:
					iin_p=PROTO_UDP;
					break;
				default:
					printf("Invalid Protocol number, It should be set 1:TCP+UDP, 2:TCP or 3:UDP !!!\n");
					return(0);
					break;
			}
			switch(atoi(t->words[8])) { // trigger protocol
				case 1:
					itr_p=PROTO_BOTH;
					break;
				case 2:
					itr_p=PROTO_TCP;
					break;
				case 3:
					itr_p=PROTO_UDP;
					break;
				default:
					printf("Invalid Protocol number, It should be set 1:TCP+UDP, 2:TCP or 3:UDP !!!\n");
					return(0);
					break;
			}
			
			if(!string_to_dec(t->words[6], &t_port) || t_port<1 || t_port>65535) { // incoming from port
				printf("Invalid value of from-port\n");
				return(0);
			}
			iin_fp=t_port;
			if(!string_to_dec(t->words[7], &t_port) || t_port<iin_fp || t_port>65535) { // incoming to port
				printf("Invalid value of from-port\n");
				return(0);
			}
			iin_tp=t_port;

			if(!string_to_dec(t->words[9], &t_port) || t_port<1 || t_port>65535) { // trigger from port
				printf("Invalid value of from-port\n");
				return(0);
			}
			itr_fp=t_port;
			if(!string_to_dec(t->words[10], &t_port) || t_port<itr_fp || t_port>65535) { // trigger to port
				printf("Invalid value of from-port\n");
				return(0);
			}
			itr_tp=t_port;

			if (!apmib_set( spap_name, (void *)t->words[4]) || !apmib_set( spap_intype, (void *)&iin_p) || !apmib_set( spap_trtype, (void *)&itr_p) || !apmib_set( spap_instart, (void *)&iin_fp) || !apmib_set( spap_inend, (void *)&iin_tp) || !apmib_set( spap_trstart, (void *)&itr_fp) || !apmib_set( spap_trend, (void *)&itr_tp)) {
				printf("\tSet SPAP MIB Error\n");
				return(0);
			}

			doenable=1;
			if(!apmib_set( spap_enable, (void *)&doenable)) {
				printf("\tSet SPAP MIB Error\n");
				return(0);
			}
			apmib_update(CURRENT_SETTING);
			restart_daemon(_WEBS_PROG_NAME);
			restart_daemon(_FIREWALL_SCRIPT_PROG);
			printf("Edit %s [%s] Successfully !!!\n",t->words[2],t->words[4]);
		}
		else
			nat_service_edit_format(t->words[2]);
	}
	else {
		if(strcmp(buff,"")) {
			printf("format: ip nat service %s [on|off|clear|edit <name> <in_proto> <in_from_port> <in_to_port> <trig_proto> <trig_from_port> <trig_to_port>]\n\n",t->words[2]);
			if(enable)
				printf("Now: %s [%s] on\n",t->words[2],buff);
			else
				printf("Now: %s [%s] off\n",t->words[2],buff);
		}
		else
			printf("No Service Setup at %s !!!\n",t->words[2]);
	}
	free(buff);
}

static void wlan_encrypt_control(struct op *t,int nc) {
	int enc=0,wep_length=0, key_lt, key_type,psk_gklt,wpa_auth=2,enable1x=0,wpa_cipher=0,wpa2_cipher=0,keyLen=0,keyNum=0;
	char key[64];

	if(!strcmp(t->words[2],"wep")) {
		if(nc<7) {
			printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
			return(0);
		}
		enc=1;
		keyNum=atoi(t->words[5]);
		if(!strcmp(t->words[3],"64")) {
			wep_length=1;
			keyLen=WEP64_KEY_LEN;
			switch(keyNum) {
				case 1:
					key_lt=MIB_WLAN_WEP64_KEY1;
					break;
				case 2:
					key_lt=MIB_WLAN_WEP64_KEY2;
					break;
				case 3:
					key_lt=MIB_WLAN_WEP64_KEY3;
					break;
				case 4:
					key_lt=MIB_WLAN_WEP64_KEY4;
					break;
				default:
					printf("Invalid WEP key Number, It should be key1 ~ key 4 !!!\n");
					return(0);
			}
		}
		else if(!strcmp(t->words[3],"128")) {
			wep_length=2;
			keyLen=WEP128_KEY_LEN;
			switch(keyNum) {
				case 1:
					key_lt=MIB_WLAN_WEP128_KEY1;
					break;
				case 2:
					key_lt=MIB_WLAN_WEP128_KEY2;
					break;
				case 3:
					key_lt=MIB_WLAN_WEP128_KEY3;
					break;
				case 4:
					key_lt=MIB_WLAN_WEP128_KEY4;
					break;
				default:
					printf("Invalid WEP key Number, It should be key1 ~ key 4 !!!\n");
					return(0);
			}
		}
		else {
			printf("Invalid WEP key length, It should be 64 or 128 !!!\n");
			printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
			return(0);
		}
		keyNum--; // 0 ~ 3 are valid values in flash
		
		if(!strcmp(t->words[4],"asc") || !strcmp(t->words[4],"ascii")) {
			key_type=0;
			if(strlen(t->words[6]) != keyLen) {
				printf("Error, the key length should be %d characters using hex type !!!\n",keyLen);
				printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
				return(0);
			}
			strcpy(key, t->words[6]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			key_type=1;
			keyLen*=2;
			if(strlen(t->words[6]) != keyLen) {
				printf("Error, the key length should be %d characters using hex type !!!\n",keyLen);
				printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
				return(0);
			}
			if (!string_to_hex(t->words[6], key, keyLen)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid key type, It should be ascii or hex !!!\n");
			printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
			return(0);
		}
	
		if (!apmib_set(MIB_WLAN_ENCRYPT, (void *)&enc)) { // wlan encrypt = 1 (wep mode)
			printf("\tSet WLAN_ENCRYPT MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WEP, (void *)&wep_length)) { // wep key length
			printf("\tSet WLAN WEP MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WEP_KEY_TYPE, (void *)&key_type)) { // wep key type, should be ASCII(5 characters) or Hex(10 characters)
			printf("\tSet WLAN_WEP_KEY_TYPE MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WEP_DEFAULT_KEY, (void *)&keyNum)) { // wep default key, key1 ~ key4
			printf("\tSet WLAN WEP_DEFAULT_KEY MIB Error\n");
			return(0);
		}
		if (!apmib_set(key_lt, (void *)key)) { // key of wep
			printf("\tSet WLAN_KEY MIB Error\n");
			return(0);
		}
	
	}
	else if(!strcmp(t->words[2],"wpa")) {
		if(nc<6) {
			printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}
		enc=2;
		if(!strcmp(t->words[3],"tk") || !strcmp(t->words[3],"tkip"))
			wpa_cipher=1;
		else if(!strcmp(t->words[3],"aes"))
			wpa_cipher=2;
		else {
			printf("Invalid wpa cipher suite, It should be tkip or aes !!!\n");
			printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}

		if(!strcmp(t->words[4],"pass")) {
			key_type=0;
			if(strlen(t->words[5]) < 8) {
				printf("Error, the psk should be set at least 8 characters in passphrease format !!!\n");
				printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
				return(0);
			}
			strcpy(key, t->words[5]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			key_type=1;
			if(strlen(t->words[5]) != 64) {
				printf("Error, the psk should be 64 characters in hex format !!!\n");
				printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
				return(0);
			}
			if (!string_to_hex(t->words[5], key, MAX_PSK_LEN)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid psk type, It should be passphrase or hex !!!\n");
			printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}
		
		if(nc == 7)
			psk_gklt=atoi(t->words[6]);
		else
			psk_gklt=86400;

		if (!apmib_set(MIB_WLAN_ENCRYPT, (void *)&enc)) { // wlan encrypt = 2 (WPA mode)
			printf("\tSet WLAN_ENCRYPT MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_PSK_FORMAT, (void *)&key_type)) { // wpa psk format, Passphrase or Hex
			printf("\tSet WLAN_WPA_PSK_FORMAT MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_CIPHER_SUITE, (void *)&wpa_cipher)) { // wpa cipher suit, TKIP or AES
			printf("\tSet WLAN_WPA_CIPHER_SUITE MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA2_CIPHER_SUITE, (void *)&wpa2_cipher)) { // wpa2 cipher suit, only None(0) can be used on wpa
			printf("\tSet WLAN_WPA2_CIPHER_SUITE MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_GROUP_REKEY_TIME, (void *)&psk_gklt)) { // group key life time, default is 86400 sec
			printf("\tSet WLAN_WPA_GROUP_REKEY_TIME MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_PSK, (void *)key)) { // key of wpa
			printf("\tSet WLAN_WPA_PSK MIB Error\n");
			return(0);
		}
		
	}
	else if(!strcmp(t->words[2],"wpa2")) {
		if(nc<6) {
			printf("format: wlan security encrypt wpa2 [mixed|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}
		enc=4;
		wpa2_cipher=2; // note that, only aes supported for wap2 cipher suite on rtl8186 series
		if(!strcmp(t->words[3],"mix") || !strcmp(t->words[3],"mixed")) {
			enc=6;  // 6 = wpa2-mixed moed
			wpa_cipher=1; // note that, wpa cipher suite should be set as tkip for wap2-mixed on rtl8186 series
		}
		else if(!strcmp(t->words[3],"aes"))
			wpa_cipher=0;
		else {
			printf("Invalid wpa cipher suit, It should be tkip or aes !!!\n");
			printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}

		if(!strcmp(t->words[4],"pass")) {
			key_type=0;
			if(strlen(t->words[5]) < 8) {
				printf("Error, the psk should be set at least 8 characters in passphrease format !!!\n");
				printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
				return(0);
			}
			strcpy(key, t->words[5]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			key_type=1;
			if(strlen(t->words[5]) != 64) {
				printf("Error, the psk should be 64 characters in hex format !!!\n");
				printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
				return(0);
			}
			if (!string_to_hex(t->words[5], key, MAX_PSK_LEN)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid psk type, It should be passphrase or hex !!!\n");
			printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
			return(0);
		}

		if(nc == 7)
			psk_gklt=atoi(t->words[6]);
		else
			psk_gklt=86400;

		if (!apmib_set(MIB_WLAN_ENCRYPT, (void *)&enc)) { // wlan encrypt = 2 (WPA mode)
			printf("\tSet WLAN_ENCRYPT MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_PSK_FORMAT, (void *)&key_type)) { // wpa psk format, Passphrase or Hex
			printf("\tSet WLAN_WPA_PSK_FORMAT MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_CIPHER_SUITE, (void *)&wpa_cipher)) { // wpa cipher suit, TKIP(1) or None(0)
			printf("\tSet WLAN_WPA_CIPHER_SUITE MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA2_CIPHER_SUITE, (void *)&wpa2_cipher)) { // wpa2 cipher suit, only AES(2) supported on rtl8186 series
			printf("\tSet WLAN_WPA2_CIPHER_SUITE MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_GROUP_REKEY_TIME, (void *)&psk_gklt)) { // group key life time, default is 86400 sec
			printf("\tSet WLAN_WPA_GROUP_REKEY_TIME MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_WLAN_WPA_PSK, (void *)key)) { // key of wpa
			printf("\tSet WLAN_WPA_PSK MIB Error\n");
			return(0);
		}
	}
	else {
		invalid_subcmds(WLAN_SEC_ENC_CMDS);
		return(0);
	}

	if (!apmib_set(MIB_WLAN_WPA_AUTH, (void *)&wpa_auth)) { // wpa auth type, using PSK
		printf("\tSet WLAN_WPA_AUTH MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_WLAN_ENABLE_1X, (void *)&enable1x)) { // disable 802.1x flag
		printf("\tSet ENABLE_1X MIB Error\n");
		return(0);
	}

	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
}

static void wlan_1x_control(struct op *t,int nc) {
	int enc=0,wep_length=0,port,enable1x=1,wpa_auth=1,wpa_cipher=0,wpa2_cipher=2;
	char *passwd;
	struct in_addr eIp;
	
	if(nc<8 && nc != 6) {
		printf("format: wlan security 1x [off|on <port> <ip_addr> <password> [wep <64|128>|wpa <tkip|aes>|wpa2 <mixed|aes>]]\n");
		return(0);
	}

	/*if (!apmib_get( MIB_WLAN_MODE, (void *)&mode)) {
		printf("\tGet WLAN MODE MIB Error\n");
		return(0);
	}

	if(mode == 1) {
		printf("Error, 1x authentication can't be used in Client mode !!!\n");
		return(0);
	}*/

	if(!string_to_dec(t->words[3], &port) || port<1 || port>65535) {
		printf("Invalid value of 1x port !!!\n");
		return(0);
	}

	if (!inet_aton(t->words[4], &eIp)) {
		printf("Invalid IP address input !!!\n");
		return(0);
	}

	passwd=t->words[5];

	if(nc != 6) { // if nc == 6 meas encrypt is none

		if(!strcmp(t->words[6],"wep")) {
			enc=1;
			//key_length=strtol(t->words[7], (char**)NULL, 10);
			if(!strcmp(t->words[7],"64"))
				wep_length=1;
			else if(!strcmp(t->words[7],"128"))
				wep_length=2;
			else {
				printf("Invalid WEP key length, It should be 64 or 128 !!!\n");
				return(0);
			}
		}
		else if(!strcmp(t->words[6],"wpa")) {
			enc=2;
			if(!strcmp(t->words[7],"tk") || !strcmp(t->words[7],"tkip"))
				wpa_cipher=1; // wpa-tkip
			else if(!strcmp(t->words[7],"aes"))
				wpa_cipher=2; // wpa-aes
			else {
				printf("Invalid wpa mode, It should be aes or tkip !!!\n");
				return(0);
			}
		}
		else if(!strcmp(t->words[6],"wpa2")) {
			enc=4; // 4 , means wpa2-aes mode
			if(!strcmp(t->words[7],"mix") || !strcmp(t->words[7],"mixed")) {
				enc=6; // 6, means wpa2-mixed mode
				wpa_cipher=1;
			}
			else if(!strcmp(t->words[7],"aes"))
				wpa_cipher=0;
			else {
				printf("Invalid wpa2 mode, It should be aes or mixed !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid key mode, It should be wep,wpa,wpa2 or none !!!\n");
			return(0);
		}
	}

	if (!apmib_set(MIB_WLAN_RS_PORT, (void *)&port)) { // Radius server port number
		printf("\tSet WLAN_RS_PORT MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_RS_IP, (void *)&eIp)) { // Radius server ip address
		printf("\tSet WLAN_RS_IP MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_RS_PASSWORD, (void *)passwd)) { // password for authentication at Radius server
		printf("\tSet WLAN_RS_PASSWORD MIB Error\n");
		return(0);
	}
	
	if (!apmib_set(MIB_WLAN_ENABLE_1X, (void *)&enable1x)) { // enable 802.1x flag
		printf("\tSet ENABLE_1X MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_WEP, (void *)&wep_length)) { // wep key length
		printf("\tSet WLAN WEP MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_WPA_AUTH, (void *)&wpa_auth)) { // wpa auth type, using WPA-Radius if selected WPA/WPA2
		printf("\tSet WLAN_WPA_AUTH MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_WPA_CIPHER_SUITE, (void *)&wpa_cipher)) { // wpa cipher suit
		printf("\tSet WLAN_WPA_CIPHER_SUITE MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_WLAN_WPA2_CIPHER_SUITE, (void *)&wpa2_cipher)) { // wpa2 cipher suit
		printf("\tSet WLAN_WPA2_CIPHER_SUITE MIB Error\n");
		return(0);
	}

	if (!apmib_set(MIB_WLAN_ENCRYPT, (void *)&enc)) { // wlan encrypt mode, wep, wpa, wpa2 or wpa2-mixed
		printf("\tSet WLAN_ENCRYPT MIB Error\n");
		return(0);
	}

	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
}

static void wan_status() {
	struct in_addr tIp,eIp,eMask,eGateway;
	unsigned long v1, v2, v3;
	int intVal;
	char *tgIp;
	
	if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
		printf("\tGet WAN_DHCP MIB Error\n");
		return(0);
	}
	if(intVal) {
		switch(intVal) {
			case 1:
				if(!isDhcpClientExist(WAN_I))
					printf("Now: Getting WAN IP from DHCP server...\n");
				else {
					if(getInAddr(WAN_I, IP_ADDR, (void *)&tIp )) {
						tgIp=inet_ntoa(tIp);
						printf("\Now: DHCP %s\n", tgIp);
					}
				}
				break;
			case 3:
				if (isConnectPPP()) {
					if(getInAddr(PPP_I, IP_ADDR, (void *)&eIp))
						printf("Now: PPPoE Connected %s\n",inet_ntoa(eIp));
					else
						printf("Now: PPPoE Connected 0.0.0.0\n");
				}
				else
					printf("Now: PPPoE Disconnected\n");
				break;
			case 4:
				if (isConnectPPP()) {
					if(getInAddr(PPP_I, IP_ADDR, (void *)&eIp))
						printf("Now: PPTP Connected %s\n",inet_ntoa(eIp));
					else
						printf("Now: PPTP Connected 0.0.0.0\n");
				}
				else
					printf("Now: PPTP Disconnected\n");
				break;
			case 5:
				if (isConnectPPP()) {
					if(getInAddr(PPP_I, IP_ADDR, (void *)&eIp))
						printf("Now: L2TP Connected %s\n",inet_ntoa(eIp));
					else
						printf("Now: L2TP Connected 0.0.0.0\n");
				}
				else
					printf("Now: L2TP Disconnected\n");
				break;
		}
	}
	else {
		/*if (!apmib_get(MIB_WAN_IP_ADDR, (void *)&eIp)) {
			printf("\tGet WAN_IP_ADDR MIB Error\n");
			return(0);
		}
		if (!apmib_get(MIB_WAN_SUBNET_MASK, (void *)&eMask)) {
			printf("\tGet WAN_SUBNET_MASK MIB Error\n");
			return(0);
		}
		if (!apmib_get(MIB_WAN_DEFAULT_GATEWAY, (void *)&eGateway)) {
			printf("\tGet WAN_DEFAULT_GATEWAY MIB Error\n");
			return(0);
		}
		printf("Now: Static IP=%s, Subnet Mask=%s, Gateway=%s\n",inet_ntoa(eIp),inet_ntoa(eMask),inet_ntoa(eGateway));*/
		printf("Now: Static IP\n");
	}
}
static void wds_encrypt_control(struct op *t,int nc) {
	int wds_encrypt,keyLen,keyFormat,keyFormat_value,keyType;
	char key[64];
	
	if(nc < 6 || nc > 6) {
		printf("Insufficient arguments !!!\n");
		return(0);
	}
		
	if(!strcmp(t->words[2],"wep")) {
		keyFormat=MIB_WLAN_WDS_WEP_FORMAT;
		keyType=MIB_WLAN_WDS_WEP_KEY;
		if(!strcmp(t->words[3],"64")) {
			keyLen=WEP64_KEY_LEN;
			wds_encrypt=WDS_ENCRYPT_WEP64;
		}
		else if(!strcmp(t->words[3],"128")) {
			keyLen=WEP128_KEY_LEN;
			wds_encrypt=WDS_ENCRYPT_WEP128;
		}
		else {
			printf("format: wlan wds encrypt wep [64|128] <ascii|hex> <key>\n");
			return(0);
		}

		if(!strcmp(t->words[4],"asc") || !strcmp(t->words[4],"ascii")) {
			keyFormat_value=0;
			if(strlen(t->words[5]) != keyLen) {
				printf("Error, the key length should be %d characters using hex type !!!\n",keyLen);
				printf("format: wlan wds encrypt wep [64|128] <ascii|hex> <key>\n");
				return(0);
			}
			strcpy(key, t->words[5]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			keyFormat_value=1;
			keyLen*=2;
			if(strlen(t->words[5]) != keyLen) {
				printf("Error, the key length should be %d characters using hex type !!!\n",keyLen);
				printf("format: wlan wds encrypt wep [64|128] <ascii|hex> <key>\n");
				return(0);
			}
			if (!string_to_hex(t->words[5], key, keyLen)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid key type, It should be ascii or hex !!!\n");
			printf("format: wlan wds encrypt wep [64|128] <ascii|hex> <key>\n");
			return(0);
		}
	}
	else if(!strcmp(t->words[2],"wpa")) {
		keyFormat=MIB_WLAN_WDS_PSK_FORMAT;
		keyType=MIB_WLAN_WDS_PSK;
		if(!strcmp(t->words[3],"tkip")) {
			wds_encrypt=WDS_ENCRYPT_TKIP;
		}
		else if(!strcmp(t->words[3],"aes")) {
			; //not yet implement
		}
		else {
			printf("format: wlan wds encrypt wpa [tkip|aes] <pass|hex> <key>\n");
			return(0);
		}

		if(!strcmp(t->words[4],"pass")) {
			keyFormat_value=0;
			if(strlen(t->words[5]) < 8) {
				printf("Error, the psk should be set at least 8 characters in passphrease format !!!\n");
				printf("format: wlan wds encrypt wpa [tkip|aes] <pass|hex> <key>\n");
				return(0);
			}
			strcpy(key, t->words[5]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			keyFormat_value=1;
			if(strlen(t->words[5]) != 64) {
				printf("Error, the psk should be 64 characters in hex format !!!\n");
				printf("format: wlan wds encrypt wpa [tkip|aes] <pass|hex> <key>\n");
				return(0);
			}
			if (!string_to_hex(t->words[5], key, MAX_PSK_LEN)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid psk type, It should be passphrase or hex !!!\n");
			printf("format: wlan wds encrypt wpa [tkip|aes] <pass|hex> <key>\n");
			return(0);
		}
	}
	else if(!strcmp(t->words[2],"wpa2")) {
		keyFormat=MIB_WLAN_WDS_PSK_FORMAT;
		keyType=MIB_WLAN_WDS_PSK;
		if(!strcmp(t->words[3],"aes")) {
			wds_encrypt=WDS_ENCRYPT_AES;
		}
		else if(!strcmp(t->words[3],"mixed")) {
			; //not yet implement
		}
		else {
			printf("format: wlan wds encrypt wpa2 [mixed|aes] <pass|hex> <key>\n");
			return(0);
		}

		if(!strcmp(t->words[4],"pass")) {
			keyFormat_value=0;
			if(strlen(t->words[5]) < 8) {
				printf("Error, the psk should be set at least 8 characters in passphrease format !!!\n");
				printf("format: wlan wds encrypt wpa2 [mixed|aes] <pass|hex> <key>\n");
				return(0);
			}
			strcpy(key, t->words[5]);
		}
		else if(!strcmp(t->words[4],"hex")) {
			keyFormat_value=1;
			if(strlen(t->words[5]) != 64) {
				printf("Error, the psk should be 64 characters in hex format !!!\n");
				printf("format: wlan wds encrypt wpa2 [mixed|aes] <pass|hex> <key>\n");
				return(0);
			}
			if (!string_to_hex(t->words[5], key, MAX_PSK_LEN)) {
				printf("Set wep-key error !!!\n");
				return(0);
			}
		}
		else {
			printf("Invalid psk type, It should be passphrase or hex !!!\n");
			printf("format: wlan wds encrypt wpa2 [mixed|aes] <pass|hex> <key>\n");
			return(0);
		}
	}
	else
		invalid_subcmds(WLAN_WDS_ENC_CMDS);

	if (!apmib_set(MIB_WLAN_WDS_ENCRYPT, (void *)&wds_encrypt)) {
		printf("\tSet WLAN_WDS_ENCRYPT MIB Error\n");
		return(0);
	}

	if (!apmib_set(keyFormat, (void *)&keyFormat_value)) {
		printf("\tSet WLAN_WDS_KEY_FORMAT MIB Error\n");
		return(0);
	}

	if (!apmib_set(keyType, (void *)&key)) {
		printf("\tSet WLAN_WDS_KEY MIB Error\n");
		return(0);
	}

	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
}

static void wds_peer_add(char *mac) {
	WDS_T *entry;
	int enable;

	entry=(WDS_T *)malloc(sizeof(*entry));
	
	if (strlen(mac)!=12 || (mac[0] != '0' && mac[1] != '0') || !string_to_hex(mac, entry->macAddr, 12)) {
		printf("Invalid MAC address input !!!\n");
		return(0);
	}
	strcpy(entry->comment, "");

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_WLAN_WDS_DEL, (void *)entry);
	if (!apmib_set(MIB_WLAN_WDS_ADD,(void *)entry)) {
		printf("\tSet WLAN_WDS_ADD MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_WLAN_WDS_ENABLED, (void *)&enable)) {
		printf("\tGet WLAN_WDS_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_WLAN_WDS_ENABLED, (void *)&enable)) {
			printf("\tSet WLAN_WDS_ENABLED MIB Error\n");
			return(0);
		}
	}
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	printf("Add one WDS peer %s\n",mac);
	free(entry);
}
static void wds_peer_del(int idx) {
	WDS_T *entry;
	int entryNum,enable=0;
	
	entry=(WDS_T *)malloc(sizeof(*entry));
	*((char *)entry) = (char)idx;
	if (!apmib_get(MIB_WLAN_WDS, (void *)entry)) {
		printf("\tGet WLAN_WDS MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_WLAN_WDS_DEL, (void *)entry)) {
		printf("\tSet WLAN_WDS_DEL MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_WLAN_WDS_NUM, (void *)&entryNum)) {
  		printf("\tGet WLAN_WDS_NUM MIB Error\n");
		return(0);
	}

	if(!entryNum) {
		if (!apmib_set(MIB_WLAN_WDS_ENABLED, (void *)&enable)) {
			printf("\tSet WLAN_WDS_ENABLED MIB Error\n");
			return(0);
		}
	}
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	printf("Del WDS peer ID %d\n",idx);
	free(entry);
}
static void wds_peer_clearall() {
	WDS_T *entry;
	int intVal;
	entry=(WDS_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_WLAN_WDS_DELALL, (void *)entry)) {
		printf("\tSet WLAN_WDS_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable wds
	if (!apmib_set(MIB_WLAN_WDS_ENABLED, (void *)&intVal)) {
		printf("\tSet WLAN_WDS_ENABLED MIB Error\n");
		return(0);
	}
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	printf("Clear All wlan wds peer !!!\n");
	free(entry);
}
static void wds_peer_disp() {
	WDS_T *entry;
	int entryNum, i, enable;
	if (!apmib_get(MIB_WLAN_WDS_NUM, (void *)&entryNum)) {
  		printf("\tGet WLAN_WDS_NUM MIB Error\n");
		return(0);
	}
	entry=(WDS_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_WLAN_WDS_ENABLED, (void *)&enable)) {
		printf("\tGet WLAN_WDS_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: wlan wds on\n");
	else
		printf("Now: wlan wds off\n");
	
	printf("=========================================\n");
	printf("ID\tMAC Address\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_WLAN_WDS, (void *)entry)) {
			printf("\tGet WLAN_WDS MIB Error\n");
			return(0);
		}
		printf("%d\t%02x:%02x:%02x:%02x:%02x:%02x\n",i,entry->macAddr[0],entry->macAddr[1],entry->macAddr[2],entry->macAddr[3],entry->macAddr[4],entry->macAddr[5]);
	}
	printf("=========================================\n");
	free(entry);
}


static void acl_add(char *mac) {
	MACFILTER_T *entry;
	int enable;

	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	
	if (strlen(mac)!=12 || (mac[0] != '0' && mac[1] != '0') || !string_to_hex(mac, entry->macAddr, 12)) {
		printf("Invalid MAC address input !!!\n");
		return(0);
	}
	strcpy(entry->comment, "");

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_WLAN_AC_ADDR_DEL, (void *)entry);
	if (!apmib_set(MIB_WLAN_AC_ADDR_ADD,(void *)entry)) {
		printf("\tSet WLAN_AC_ADDR_ADD MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_WLAN_AC_ENABLED, (void *)&enable)) {
		printf("\tGet WLAN_AC_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_WLAN_AC_ENABLED, (void *)&enable)) {
			printf("\tSet WLAN_AC_ENABLED MIB Error\n");
			return(0);
		}
	}
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	printf("Add one acl %s\n",mac);
	free(entry);
}


static void acl_del(int idx) {
	MACFILTER_T *entry;
	int entryNum,enable=0;
	
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	*((char *)entry) = (char)idx;
	if (!apmib_get(MIB_WLAN_AC_ADDR, (void *)entry)) {
		printf("\tGet WLAN_AC_ADDR MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_WLAN_AC_ADDR_DEL, (void *)entry)) {
		printf("\tSet WLAN_AC_ADDR_DEL MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_WLAN_AC_NUM, (void *)&entryNum)) {
  		printf("\tGet WLAN_AC_NUM MIB Error\n");
		return(0);
	}

	if(!entryNum) {
		if (!apmib_set(MIB_WLAN_AC_ENABLED, (void *)&enable)) {
			printf("\tSet WLAN_AC_ENABLED MIB Error\n");
			return(0);
		}
	}
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	printf("Del filter ID %d\n",idx);
	free(entry);
}

static void acl_clearall() {
	MACFILTER_T *entry;
	int intVal;
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_WLAN_AC_ADDR_DELALL, (void *)entry)) {
		printf("\tSet WLAN_AC_ADDR_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable macFilter
	if (!apmib_set(MIB_WLAN_AC_ENABLED, (void *)&intVal)) {
		printf("\tSet WLAN_AC_ENABLED MIB Error\n");
		return(0);
	}
	printf("Clear All wlan acl !!!\n");
	//apmib_update(CURRENT_SETTING);
	//printf("\n\tplease wait some minutes...\n\n");
	//run_script();
	free(entry);
}

static void acl_disp() {
	MACFILTER_T *entry;
	int entryNum, i, enable;
	if (!apmib_get(MIB_WLAN_AC_NUM, (void *)&entryNum)) {
  		printf("\tGet WLAN_AC_NUM MIB Error\n");
		return(0);
	}
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_WLAN_AC_ENABLED, (void *)&enable)) {
		printf("\tGet WLAN_AC_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: wlan acl on\n");
	else
		printf("Now: wlan acl off\n");
	
	printf("=========================================\n");
	printf("ID\tMAC Address\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_WLAN_AC_ADDR, (void *)entry)) {
			printf("\tGet WLAN_AC_ADDR MIB Error\n");
			return(0);
		}
		printf("%d\t%02x:%02x:%02x:%02x:%02x:%02x\n",i,entry->macAddr[0],entry->macAddr[1],entry->macAddr[2],entry->macAddr[3],entry->macAddr[4],entry->macAddr[5]);
	}
	printf("=========================================\n");
	free(entry);
}

static void macfilter_add(char *mac) {
	MACFILTER_T *entry;
	int enable;

	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	
	if (strlen(mac)!=12 || (mac[0] != '0' && mac[1] != '0') || !string_to_hex(mac, entry->macAddr, 12)) {
		printf("Invalid MAC address input !!!\n");
		return(0);
	}
	strcpy(entry->comment, "");

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_MACFILTER_DEL, (void *)entry);
	if (!apmib_set(MIB_MACFILTER_ADD,(void *)entry)) {
		printf("\tSet MACFILTER_ADD MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_MACFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet MACFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_MACFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet MACFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add one macfilter %s\n",mac);
	free(entry);
}

static void macfilter_del(int idx) {
	MACFILTER_T *entry;
	int entryNum,enable=0;
	
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	*((char *)entry) = (char)idx;
	if (!apmib_get(MIB_MACFILTER, (void *)entry)) {
		printf("\tGet MACFILTER MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_MACFILTER_DEL, (void *)entry)) {
		printf("\tSet MACFILTER_DEL MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_MACFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet MACFILTER_NUM MIB Error\n");
		return(0);
	}

	if(!entryNum) {
		if (!apmib_set(MIB_MACFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet MACFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Del filter ID %d\n",idx);
	free(entry);
}

static void macfilter_clearall() {
	MACFILTER_T *entry;
	int intVal;
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_MACFILTER_DELALL, (void *)entry)) {
		printf("\tSet MACFILTER_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable macFilter
	if (!apmib_set(MIB_MACFILTER_ENABLED, (void *)&intVal)) {
		printf("\tSet MACFILTER_ENABLED MIB Error\n");
		return(0);
	}
	printf("Clear All macfilter list !!!\n");
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	free(entry);
}

static void macfilter_disp() {
	MACFILTER_T *entry;
	int entryNum, i, enable;
	if (!apmib_get(MIB_MACFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet URLFILTER_NUM MIB Error\n");
		return(0);
	}
	entry=(MACFILTER_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_MACFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet MACFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: macfilter on\n");
	else
		printf("Now: macfilter off\n");
	
	printf("=========================================\n");
	printf("ID\tMAC Address\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_MACFILTER, (void *)entry)) {
			printf("\tGet MACFILTER MIB Error\n");
			return(0);
		}
		printf("%d\t%02x:%02x:%02x:%02x:%02x:%02x\n",i,entry->macAddr[0],entry->macAddr[1],entry->macAddr[2],entry->macAddr[3],entry->macAddr[4],entry->macAddr[5]);
	}
	printf("=========================================\n");
	free(entry);
}

static void ipfilter_add(char *ip,int prot) {
	IPFILTER_T *entry;
	unsigned long v1, v2, v3;
	struct in_addr curIpAddr, curSubnet;
	int enable;

	entry=(IPFILTER_T *)malloc(sizeof(*entry));
	inet_aton(ip, (struct in_addr *)entry->ipAddr);
	getInAddr(ELAN_I, IP_ADDR, (void *)&curIpAddr);
	getInAddr(ELAN_I, SUBNET_MASK, (void *)&curSubnet);

	v1 = *((unsigned long *)entry->ipAddr);
	v2 = *((unsigned long *)&curIpAddr);
	v3 = *((unsigned long *)&curSubnet);

	if ( (v1 & v3) != (v2 & v3) ) {
		printf("Invalid IP address! It should be set within the current subnet\n");
		free(entry);
		return(0);
	}
	strcpy(entry->comment, "");

	switch(prot) {
		case 1:
			entry->protoType=PROTO_BOTH;
			break;
		case 2:
			entry->protoType=PROTO_TCP;
			break;
		case 3:
			entry->protoType=PROTO_UDP;
			break;
		default:
			printf("Invalid Protocol number, It should be set 1:TCP+UDP, 2:TCP or 3:UDP !!!\n");
			return(0);
			break;
	}

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_IPFILTER_DEL, (void *)entry);
	if (!apmib_set(MIB_IPFILTER_ADD,(void *)entry)) {
		printf("\tSet IPFILTER_ADD MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_IPFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet IPFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_IPFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet IPFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add one filter %s\n",ip);
	free(entry);
}

static void ipfilter_del(int idx) {
	IPFILTER_T *entry;
	int entryNum,enable=0;
	
	entry=(IPFILTER_T *)malloc(sizeof(*entry));
	*((char *)entry) = (char)idx;
	if (!apmib_get(MIB_IPFILTER, (void *)entry)) {
		printf("\tGet IPFILTER MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_IPFILTER_DEL, (void *)entry)) {
		printf("\tSet IPFILTER_DEL MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_IPFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet IPFILTER_NUM MIB Error\n");
		return(0);
	}

	if(!entryNum) {
		if (!apmib_set(MIB_IPFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet IPFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add filter ID %d\n",idx);
	free(entry);
}

static void ipfilter_clearall() {
	IPFILTER_T *entry;
	int intVal;
	entry=(IPFILTER_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_IPFILTER_DELALL, (void *)entry)) {
		printf("\tSet IPFILTER_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable ipfilter
	if (!apmib_set(MIB_IPFILTER_ENABLED, (void *)&intVal)) {
		printf("\tSet IPFILTER_ENABLED MIB Error\n");
		return(0);
	}
	printf("Clear All ipfilter list !!!\n");
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	free(entry);
}

static void ipfilter_disp() {
	IPFILTER_T *entry;
	int entryNum, i, enable;
	char *ip,*type;
	
	if (!apmib_get(MIB_IPFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet IPFILTER_NUM MIB Error\n");
		return(0);
	}
	entry=(IPFILTER_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_IPFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet IPFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: ipfilter on\n");
	else
		printf("Now: ipfilter off\n");
	
	printf("=========================================\n");
	printf("ID\tIP address\tProtocol\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_IPFILTER, (void *)entry)) {
			printf("\tGet IPFILTER MIB Error\n");
			return(0);
		}

		ip = inet_ntoa(*((struct in_addr *)entry->ipAddr));
		if ( !strcmp(ip, "0.0.0.0"))
			ip = "----";

		if ( entry->protoType == PROTO_BOTH )
			type = "TCP+UDP";
		else if ( entry->protoType == PROTO_TCP )
			type = "TCP";
		else
			type = "UDP";

		printf("%d\t%s\t%s\n",i,ip, type);
	}
	printf("=========================================\n");
	free(entry);
}

static void urlfilter_add(char *url) {
	URLFILTER_T *urlEntry;
	int enable;
	
	urlEntry=(URLFILTER_T *)malloc(sizeof(*urlEntry));
	strcpy(urlEntry->urlAddr, url);

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_URLFILTER_DEL, (void *)urlEntry);
	if (!apmib_set(MIB_URLFILTER_ADD,(void *)urlEntry)) {
		printf("\tSet URLFILTER_ADD MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_URLFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet URLFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_URLFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet URLFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add one filter %s\n",url);
	free(urlEntry);
}

static void urlfilter_del(int idx) {
	URLFILTER_T *urlEntry;
	int entryNum,enable=0;
	
	urlEntry=(URLFILTER_T *)malloc(sizeof(*urlEntry));
	*((char *)urlEntry) = (char)idx;
	if (!apmib_get(MIB_URLFILTER, (void *)urlEntry)) {
		printf("\tGet URLFILTER MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_URLFILTER_DEL, (void *)urlEntry)) {
		printf("\tSet URLFILTER_DEL MIB Error\n");
		return(0);
	}

	if (!apmib_get(MIB_URLFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet URLFILTER_NUM MIB Error\n");
		return(0);
	}

	if(!entryNum) {
		if (!apmib_set(MIB_URLFILTER_ENABLED, (void *)&enable)) {
			printf("\tSet URLFILTER_ENABLED MIB Error\n");
			return(0);
		}
	}
	
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add filter ID %d\n",idx);
	free(urlEntry);
}

static void urlfilter_clearall() {
	URLFILTER_T *entry;
	int intVal;
	
	entry=(URLFILTER_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_URLFILTER_DELALL, (void *)entry)) {
		printf("\tSet URLFILTER_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable PortFW
	if (!apmib_set(MIB_URLFILTER_ENABLED, (void *)&intVal)) {
		printf("\tSet URLFILTER_ENABLED MIB Error\n");
		return(0);
	}
	printf("Clear All urlfilter list !!!\n");
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	free(entry);
}

static void urlfilter_disp() {
	URLFILTER_T *entry;
	int entryNum, i, enable;
	if (!apmib_get(MIB_URLFILTER_NUM, (void *)&entryNum)) {
  		printf("\tGet URLFILTER_NUM MIB Error\n");
		return(0);
	}
	entry=(URLFILTER_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_URLFILTER_ENABLED, (void *)&enable)) {
		printf("\tGet URLFILTER_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: urlfilter on\n");
	else
		printf("Now: urlfilter off\n");
	
	printf("=========================================\n");
	printf("ID\tURL\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_URLFILTER, (void *)entry)) {
			printf("\tGet URLFILTER MIB Error\n");
			return(0);
		}
		printf("%d\t%s\n",i,entry->urlAddr);
	}
	printf("=========================================\n");
	free(entry);
}

static void ip_dhcp_add(char *mac,char *ip) {
	STATICLEASE_T *entry;
	unsigned long v1, v2, v3;
	struct in_addr curIpAddr, curSubnet;
	char tmpbuf[64];

	entry=(STATICLEASE_T *)malloc(sizeof(*entry));

	if (strlen(mac)!=12 || (mac[0] != '0' && mac[1] != '0') || !string_to_hex(mac, entry->macAddr, 12)) {
		printf("Invalid IP address or MAC address input !!!\n");
		return(0);
	}

	inet_aton(ip, (struct in_addr *)entry->ipAddr);
	getInAddr(ELAN_I, IP_ADDR, (void *)&curIpAddr);
	getInAddr(ELAN_I, SUBNET_MASK, (void *)&curSubnet);

	v1 = *((unsigned long *)entry->ipAddr);
	v2 = *((unsigned long *)&curIpAddr);
	v3 = *((unsigned long *)&curSubnet);

	if ( (v1 & v3) != (v2 & v3) ) {
		printf("Invalid IP address! It should be set within the current subnet\n");
		free(entry);
		return(0);
	}

	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_STATICLEASE_DEL, (void *)entry);
	if (!apmib_set(MIB_STATICLEASE_ADD,(void *)entry)) {
		printf("\tSet STATICLEASE_ADD MIB Error\n");
		return(0);
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
#ifdef HOME_GATEWAY
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"gw");
#else
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"ap");
#endif
	restart_daemon(tmpbuf);
	printf("Add one static lease client %s => %s\n",mac,ip);
	free(entry);
}

static void ip_dhcp_del(int idx) {
	STATICLEASE_T *entry;
	char tmpbuf[64];
	
	entry=(STATICLEASE_T *)malloc(sizeof(*entry));
	*((char *)entry) = (char)idx;
	if (!apmib_get(MIB_STATICLEASE, (void *)entry)) {
		printf("\tGet STATICLEASE MIB Error\n");
		return(0);
	}
	if (!apmib_set(MIB_STATICLEASE_DEL, (void *)entry)) {
		printf("\tSet STATICLEASE_DEL MIB Error\n");
		return(0);
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
#ifdef HOME_GATEWAY
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"gw");
#else
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"ap");
#endif
	restart_daemon(tmpbuf);
	printf("Add static lease ID %d\n",idx);
	free(entry);
}

static void ip_dhcp_clearall() {
	STATICLEASE_T *entry;
	char tmpbuf[64];
	
	entry=(STATICLEASE_T *)malloc(sizeof(*entry));
	
	if (!apmib_set(MIB_STATICLEASE_DELALL, (void *)entry)) {
		printf("\tSet STATICLEASE_DELALL MIB Error\n");
		return(0);
	}
	printf("Clear All static leased clients !!!\n");
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
#ifdef HOME_GATEWAY
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"gw");
#else
	sprintf(tmpbuf,"%s/%s %s %s", _CONFIG_SCRIPT_PATH, _DHCPD_SCRIPT_PROG,ELAN_I,"ap");
#endif
	restart_daemon(tmpbuf);
	free(entry);
}

static void ip_dhcp_disp() {
	STATICLEASE_T *entry;
	int entryNum, i, enable;
	char *ip;
	
	if (!apmib_get(MIB_STATICLEASE_NUM, (void *)&entryNum)) {
  		printf("\tGet STATICLEASE_NUM MIB Error\n");
		return(0);
	}
	entry=(STATICLEASE_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_DHCP, (void *)&enable)) {
		printf("\tGet DHCP MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: lan dhcp server on\n");
	else
		printf("Now: lan dhcp server off\n");
	
	printf("=========================================\n");
	printf("ID\tMAC Address\tIP Address\n");
	for (i=1; i<=entryNum; i++) {
		*((char *)entry) = (char)i;
		if (!apmib_get(MIB_STATICLEASE, (void *)entry)) {
			printf("\tGet STATICLEASE MIB Error\n");
			return(0);
		}
		ip = inet_ntoa(*((struct in_addr *)entry->ipAddr));
		if ( !strcmp(ip, "0.0.0.0"))
			ip = "----";
		
		printf("%d\t%02x:%02x:%02x:%02x:%02x:%02x\t%s\n",i,entry->macAddr[0],entry->macAddr[1],entry->macAddr[2],entry->macAddr[3],entry->macAddr[4],entry->macAddr[5],ip);
	}
	printf("=========================================\n");
	free(entry);
}

static void ip_dhcp_status() {
	int dhcpd,dhcpc;
	char *tgIp;
	struct in_addr tIp,esdhcp,eedhcp;
	
	if (!apmib_get(MIB_DHCP, (void *)&dhcpd)) { // dhcpd
		printf("\tGet DHCP MIB Error\n");
		return(0);
	}
	if (!apmib_get(MIB_WAN_DHCP, (void *)&dhcpc)) { // dhcpc
		printf("\tGet WAN DHCP MIB Error\n");
		return(0);
	}

	if(dhcpc !=1 && dhcpd !=2) {
		printf("LAN DHCP Status: off\n");
		printf("WAN DHCP Status: off\n");
		return(0);
	}
	
	if(dhcpd == 2) { // dhcpd
		if (!apmib_get(MIB_DHCP_CLIENT_START, (void *)&esdhcp)) {
			printf("\tGet DHCP Clinet Start MIB Error\n");
			return(0);
		}
		if (!apmib_get(MIB_DHCP_CLIENT_END, (void *)&eedhcp)) {
			printf("\tGet DHCP Clinet Start MIB Error\n");
			return(0);
		}
		printf("LAN DHCP Status: \n\n");
		printf("\tDHCP Client Start Address: %s\n", inet_ntoa(esdhcp));
		printf("\tDHCP Client End Address: %s\n", inet_ntoa(eedhcp));
		printf("\n");
	}

	
	if(dhcpc == 1) { // dhcpc
		printf("WAN DHCP Status: \n\n");
		if(!isDhcpClientExist(WAN_I))
			printf("\tGetting IP from DHCP server...\n");
		else {
			if(getInAddr(WAN_I, IP_ADDR, (void *)&tIp )) {
				tgIp=inet_ntoa(tIp);
				printf("\tWAN DHCP IP Address: %s\n", tgIp);
			}
		}
		printf("\n");
	}
}

static void nat_server_disp() {
	int l,intVal,enable;
	PORTFW_T *entry;
	char	*type, portRange[20], *ip;
	
	entry=(PORTFW_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_PORTFW_ENABLED, (void *)&enable)) {
		printf("\tGet PORTFW_ENABLED MIB Error\n");
		return(0);
	}
	if(enable)
		printf("Now: Server on\n");
	else
		printf("Now: Server off\n");
	
	if (!apmib_get(MIB_PORTFW_NUM, (void *)&intVal)) {
		printf("\tGet PORTFW_NUM MIB Error\n");
		return(0);
	}
	printf("==============================================\n");
	printf("ID\tIP Address\tProtocl\tPort Range\n");
	for(l=1;l<=intVal;l++) {
		*((char *)entry) = (char)l;
		if (!apmib_get(MIB_PORTFW, (void *)entry)) {
			printf("\tGet PORTFW MIB Error\n");
			return(0);
		}
		ip = inet_ntoa(*((struct in_addr *)entry->ipAddr));
		if ( !strcmp(ip, "0.0.0.0"))
			ip = "----";

		if ( entry->protoType == PROTO_BOTH )
			type = "TCP+UDP";
		else if ( entry->protoType == PROTO_TCP )
			type = "TCP";
		else
			type = "UDP";

		if ( entry->fromPort == 0)
			strcpy(portRange, "----");
		else if ( entry->fromPort == entry->toPort )
			snprintf(portRange, 20, "%d", entry->fromPort);
		else
			snprintf(portRange, 20, "%d-%d", entry->fromPort, entry->toPort);
		printf("%d\t%s\t%s\t%s\n",l,ip, type, portRange);
	}
	printf("==============================================\n");
	free(entry);
}

static void nat_server_add(struct op *t,int idx, int w) {
	char *strIp,*serv_info,*strFrom, *strTo;
	unsigned long v1, v2, v3;
	PORTFW_T *entry;
	struct in_addr curIpAddr, curSubnet;
	int enable,t_port;

	entry=(PORTFW_T *)malloc(sizeof(*entry));
	strIp=t->words[3]; // ip 0:nat 1:server 2:add 3:serverip
	inet_aton(strIp, (struct in_addr *)entry->ipAddr);
	getInAddr(ELAN_I, IP_ADDR, (void *)&curIpAddr);
	getInAddr(ELAN_I, SUBNET_MASK, (void *)&curSubnet);

	v1 = *((unsigned long *)entry->ipAddr);
	v2 = *((unsigned long *)&curIpAddr);
	v3 = *((unsigned long *)&curSubnet);

	if ( (v1 & v3) != (v2 & v3) ) {
		printf("Invalid IP address! It should be set within the current subnet\n");
		free(entry);
		return(0);
	}
	strcpy(entry->comment, "");
	if(idx) {
		switch(idx) {
			case 1: // web, 80
				entry->protoType = PROTO_TCP;
				entry->fromPort=80;
				entry->toPort=80;
				serv_info="WEB";
				break;
			case 2: // ftp, 20-21
				entry->protoType = PROTO_TCP;
				entry->fromPort=20;
				entry->toPort=21;
				serv_info="FTP";
				break;
			case 3: // pop3, 110
				entry->protoType = PROTO_TCP;
				entry->fromPort=110;
				entry->toPort=110;
				serv_info="POP3";
				break;
			case 4: // smpt, 25
				entry->protoType = PROTO_TCP;
				entry->fromPort=25;
				entry->toPort=25;
				serv_info="SMPT";
				break;
			case 5: // dns, 53
				entry->protoType = PROTO_UDP;
				entry->fromPort=53;
				entry->toPort=53;
				serv_info="DNS";
				break;
			case 6: // telnet, 23
				entry->protoType = PROTO_TCP;
				entry->fromPort=23;
				entry->toPort=23;
				serv_info="telnet";
				break;
			default:
				printf("Error! Invalid server type, It should be 1 ~ 6!!!\n");
				return(0);
				break;
		}
	}
	else {
		serv_info="Customize";
		switch(atoi(t->words[5])) {
			case 1:
				entry->protoType=PROTO_BOTH;
				break;
			case 2:
				entry->protoType=PROTO_TCP;
				break;
			case 3:
				entry->protoType=PROTO_UDP;
				break;
			default:
				printf("Invalid Protocol number, It should be set 1:TCP+UDP, 2:TCP or 3:UDP !!!\n");
				return(0);
				break;
		}
		strFrom=t->words[6];
		if(!string_to_dec(strFrom, &t_port) || t_port<1 || t_port>65535) {
			printf("Invalid value of from-port\n");
			return(0);
		}
		entry->fromPort=(unsigned short)t_port;
		if(w) {
			strTo=t->words[7];
			if(!string_to_dec(strTo, &t_port) || t_port<entry->fromPort || t_port>65535) {
				printf("Invalid value of to-port\n");
				return(0);
			}
			entry->toPort=(unsigned short)t_port;
		}
		else
			entry->toPort=entry->fromPort;
	}
	// set to MIB. try to delete it first to avoid duplicate case
	apmib_set(MIB_PORTFW_DEL, (void *)entry);
	if (!apmib_set(MIB_PORTFW_ADD, (void *)entry)){
		printf("\tSet PORTFW MIB Error\n");
		return(0);
	}
	
	if (!apmib_get(MIB_PORTFW_ENABLED, (void *)&enable)) {
		printf("\tGet PORTFW_ENABLED MIB Error\n");
		return(0);
	}
	if(!enable) {
		enable=1;
		if (!apmib_set(MIB_PORTFW_ENABLED, (void *)&enable)) {
			printf("\tSet PORTFW_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Add one %s Server to %s\n",serv_info,strIp);
	free(entry);
}

static void nat_server_delete(int idx) {
	int intVal,enable;
	PORTFW_T *entry;

	entry=(PORTFW_T *)malloc(sizeof(*entry));
	if (!apmib_get(MIB_PORTFW_NUM, (void *)&intVal)) {
		printf("\tGet PORTFW_NUM MIB Error\n");
		return(0);
	}
	if(idx > intVal) {
		printf("format: ip nat server clear <id>\n");
		return(0);
	}
	else {
		*((char *)entry) = (char)idx;
		if (!apmib_get(MIB_PORTFW, (void *)entry)) {
			printf("\tGet PORTFW MIB Error\n");
			return(0);
		}
		if (!apmib_set(MIB_PORTFW_DEL, (void *)entry)) {
			printf("\tSet PORTFW_DEL MIB Error\n");
			return(0);
		}
		intVal--;
	}
	if(!intVal) {
		enable=0;
		if (!apmib_set(MIB_PORTFW_ENABLED, (void *)&enable)) {
			printf("\tSet PORTFW_ENABLED MIB Error\n");
			return(0);
		}
	}
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	printf("Clear Server Port ID %d\n",idx);
	free(entry);
}

static void nat_server_clearall() {
	int intVal;
	PORTFW_T *entry;

	entry=(PORTFW_T *)malloc(sizeof(*entry));
	if (!apmib_set(MIB_PORTFW_DELALL, (void *)entry)) {
		printf("\tSet PORTFW_DELALL MIB Error\n");
		return(0);
	}
	intVal=0; // disable PortFW
	if (!apmib_set(MIB_PORTFW_ENABLED, (void *)&intVal)) {
		printf("\tSet PORTFW_ENABLED MIB Error\n");
		return(0);
	}
	printf("Reset All NAT Server Ports !!!\n");
	apmib_update(CURRENT_SETTING);
	restart_daemon(_WEBS_PROG_NAME);
	restart_daemon(_FIREWALL_SCRIPT_PROG);
	free(entry);
}

static void wlan_survey_connect(int idx) {
	unsigned char res;
	NETWORK_TYPE_T net;
	int chan;

	int status, encrypt;
	int wait_time;
	char tmpBuf[64];
	char cmdBuf[64];

	AUTH_TYPE_T authType;

	if (pStatus == NULL) {
		printf("Please refresh again !!!\n");
		return(0);
	}
	//sscanf(strSel, "sel%d", &idx);

	if ( idx >= pStatus->number ) { // invalid index
		printf("SConnect failed !!!\n");
		return(0);
	}

	// check encryption type match or not
	if ( !apmib_get( MIB_WLAN_ENCRYPT, (void *)&encrypt) ) {
		printf("\tGet WLAN ENCRYPT MIB Error\n");
		return(0);
	}
	else {
		// no encryption
		if (encrypt == ENCRYPT_DISABLED) {
			if (pStatus->bssdb[idx].bdCap & 0x00000010) {
				printf("\tEncryption type mismatch !!!\n");
				return(0);
			}
			else
				; // success
		}
		// legacy encryption
		else if (encrypt == ENCRYPT_WEP) {
			if ((pStatus->bssdb[idx].bdCap & 0x00000010) == 0) {
				printf("\tEncryption type mismatch !!!\n");
				return(0);
			}
			else if (pStatus->bssdb[idx].bdTstamp[0] != 0) {
				printf("\tEncryption type mismatch !!!\n");
				return(0);
			}
			else
				; // success
		}
		// WPA/WPA2
		else {
			int isPSK, auth;
			apmib_get(MIB_WLAN_WPA_AUTH, (void *)&auth);
			if (auth == WPA_AUTH_PSK)
				isPSK = 1;
			else
				isPSK = 0;
				if ((pStatus->bssdb[idx].bdCap & 0x00000010) == 0) {
					printf("\tEncryption type mismatch !!!\n");
				return(0);
				}
				else if (pStatus->bssdb[idx].bdTstamp[0] == 0) {
					printf("\tEncryption type mismatch !!!\n");
				return(0);
				}
				else if (encrypt == ENCRYPT_WPA) {
					if (((pStatus->bssdb[idx].bdTstamp[0] & 0x0000ffff) == 0) ||
						(isPSK && !(pStatus->bssdb[idx].bdTstamp[0] & 0x4000)) ||
						(!isPSK && (pStatus->bssdb[idx].bdTstamp[0] & 0x4000)) ) {
						printf("\tEncryption type mismatch !!!\n");
						return(0);
				}
			}
			else if (encrypt == ENCRYPT_WPA2) {
				if (((pStatus->bssdb[idx].bdTstamp[0] & 0xffff0000) == 0) ||
						(isPSK && !(pStatus->bssdb[idx].bdTstamp[0] & 0x40000000)) ||
						(!isPSK && (pStatus->bssdb[idx].bdTstamp[0] & 0x40000000)) ) {
					printf("\tEncryption type mismatch !!!\n");
					return(0);
				}
			}
			else
				; // success
		}
	}

	// Set SSID, network type to MIB
	memcpy(tmpBuf, pStatus->bssdb[idx].bdSsIdBuf, pStatus->bssdb[idx].bdSsId.Length);
	tmpBuf[pStatus->bssdb[idx].bdSsId.Length] = '\0';
	if (!apmib_set(MIB_WLAN_SSID, (void *)tmpBuf)) {
		printf("\tSet WLAN SSID MIB Error\n");
		return(0);
	}

	if ( pStatus->bssdb[idx].bdCap & cESS )
		net = INFRASTRUCTURE;
	else
		net = ADHOC;

	if (!apmib_set(MIB_WLAN_NETWORK_TYPE, (void *)&net)) {
		printf("\tSet WLAN NETWORK_TYPE MIB Error\n");
		return(0);
	}

	if (net == ADHOC) {
		chan = pStatus->bssdb[idx].ChannelNumber;
		if (!apmib_set( MIB_WLAN_CHAN_NUM, (void *)&chan)) {
			printf("\tSet WLAN CHAN_NUM MIB Error\n");
			return(0);
		}
	}

	apmib_update(CURRENT_SETTING);

	res = idx;
	wait_time = 0;
	while (1) {
		if ( getWlJoinRequest(WLAN_I, &pStatus->bssdb[idx], &res) < 0 ) {
			printf("Join request failed !!!\n");
			return(0);
		}
		if ( res == 1 ) { // wait
			if (wait_time++ > 5) {
				printf("connect-request timeout !!!\n");
				return(0);
			}
			sleep(1);
			continue;
		}
		break;
	}

	if ( res != 2 ) {
		wait_time = 0;
		while (1) {
			if ( getWlJoinResult(WLAN_I, &res) < 0 ) {
				printf("Get Join result failed !!!\n");
				return(0);
			}
			if ( res != 0xff ) { // completed
				if (wait_time++ > 10) {
					printf("connect timeout !!!\n");
					return(0);
				}
				break;
			}
			sleep(1);
		}

		if (!(res!=STATE_Bss && res!=STATE_Ibss_Idle && res!=STATE_Ibss_Active )) {
			status = 0;
			if (encrypt == ENCRYPT_WPA || encrypt == ENCRYPT_WPA2) {
				bss_info bss;
				wait_time = 0;
				while (wait_time++ < 5) {
					getWlBssInfo(WLAN_I, &bss);
					if (bss.state == STATE_CONNECTED)
						break;
					sleep(1);
				}
				if (wait_time >= 5)
					status = 1;
			}
		}
	}
	restart_daemon(_WEBS_PROG_NAME);
	sprintf(cmdBuf,"%s","wlanapp.sh start wlan0 br0");
	system(cmdBuf);
}

static void wlan_survey_scan() {
	int wait_time,status;
	wait_time = 0;
	unsigned char res;
	
	while (1) {
		if ( getWlSiteSurveyRequest(WLAN_I,  &status) < 0 ) {
			printf("Site-survey request failed !!!\n");
			return(0);
		}
		if (status != 0) {	// not ready
			if (wait_time++ > 5) {
				printf("scan request timeout !!!\n");
				return(0);
			}
			sleep(1);
		}
		else
			break;
	}

	// wait until scan completely
	wait_time = 0;
	while (1) {
		res = 1;	// only request request status
		if ( getWlSiteSurveyResult(WLAN_I, (SS_STATUS_Tp)&res) < 0 ) {
			free(pStatus);
			pStatus = NULL;
			printf("Read site-survey status failed !!!\n");
			return(0);
		}
		if (res == 0xff) {   // in progress
			if (wait_time++ > 10)
			{
				free(pStatus);
				pStatus = NULL;
				printf("scan timeout !!!\n");
				return(0);
			}
			sleep(1);
		}
		else
			break;
	}

}

static void wlan_survey() {
	BssDscr *pBss;
	bss_info bss;
	WLAN_MODE_T mode;
	char tmpBuf[100], ssidbuf[40], tmp1Buf[10], tmp2Buf[20];
	int i,wlan_disabled;

	if (!apmib_get( MIB_WLAN_DISABLED, (void *)&wlan_disabled)){
		printf("\tGet WLAN DISABLED MIB Error\n");
		return(0);
	}

	if(wlan_disabled) {
		printf("Now: wlan off\n");
		printf("please turn on wlan first !!!\n");
		return(0);
	}

	wlan_survey_scan();
	
	if (pStatus==NULL) {
		pStatus = calloc(1, sizeof(SS_STATUS_T));
		if ( pStatus == NULL ) {
			printf("Allocate buffer failed!\n");
			return(0);
		}
	}

	pStatus->number = 0; // request BSS DB

	if ( getWlSiteSurveyResult(WLAN_I, pStatus) < 0 ) {
		free(pStatus); //sc_yang
		pStatus = NULL;
		return(0);
	}

	if (!apmib_get( MIB_WLAN_MODE, (void *)&mode) ) {
		printf("Get MIB_WLAN_MODE MIB failed!\n");
		return(0);
	}

	if ( getWlBssInfo(WLAN_I, &bss) < 0) {
		printf("Get bssinfo failed!\n");
		return(0);
	}
	printf("==============================================\n");
	printf("ID\tSSID\tBSSID\tChannel Type\tEncrypt\tSignal\n");

	for (i=0; i<pStatus->number && pStatus->number!=0xff; i++) {
		pBss = &pStatus->bssdb[i];
		snprintf(tmpBuf, 200, "%02x:%02x:%02x:%02x:%02x:%02x",
			pBss->bdBssId[0], pBss->bdBssId[1], pBss->bdBssId[2],
			pBss->bdBssId[3], pBss->bdBssId[4], pBss->bdBssId[5]);
		memcpy(ssidbuf, pBss->bdSsIdBuf, pBss->bdSsId.Length);
		ssidbuf[pBss->bdSsId.Length] = '\0';
		if (pBss->network==BAND_11B)
			strcpy(tmp1Buf, " (B)");
		else if (pBss->network==BAND_11G)
			strcpy(tmp1Buf, " (G)");
		else if (pBss->network==(BAND_11G|BAND_11B))
			strcpy(tmp1Buf, " (B+G)");
		else
			strcpy(tmp1Buf, " (A)");

		if ((pBss->bdCap & cPrivacy) == 0)
			sprintf(tmp2Buf, "no");
		else {
			if (pBss->bdTstamp[0] == 0)
				sprintf(tmp2Buf, "WEP");
			else {
				int wpa_exist = 0, idx = 0;
				if (pBss->bdTstamp[0] & 0x0000ffff) {
					idx = sprintf(tmp2Buf, "WPA");
					if (((pBss->bdTstamp[0] & 0x0000f000) >> 12) == 0x4)
						idx += sprintf(tmp2Buf+idx, "-PSK");
					wpa_exist = 1;
				}
				if (pBss->bdTstamp[0] & 0xffff0000) {
					if (wpa_exist)
						idx += sprintf(tmp2Buf+idx, "/");
					idx += sprintf(tmp2Buf+idx, "WPA2");
					if (((pBss->bdTstamp[0] & 0xf0000000) >> 28) == 0x4)
						idx += sprintf(tmp2Buf+idx, "-PSK");
				}
			}
		}

		printf("%d\t%s\t%s\t%d%s\t%s\t%s\t%d\n",i,ssidbuf, tmpBuf, pBss->ChannelNumber, tmp1Buf,((pBss->bdCap & cIBSS) ? "Ad hoc" : "AP"), tmp2Buf, pBss->rssi);
	}



	printf("==============================================\n");
}

/* -------- do.c -------- */

/*
 * built-in commands: doX
 */

static int
doreload(t)
struct op *t;
{
	register int i;
	char *mode=NULL;

	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;
	
	if(i>1) {
		if(!strcmp(t->words[0],"?") || !strcmp(t->words[0],"help")) {
			printf("Too many inputs !!!\n");
			return(1);
		}
	}

	switch(i) {
		case 0:
			mode="all";
			break;
		case 1:
			if(!strcmp(t->words[i-1],"br") || !strcmp(t->words[i-1],"brid") || !strcmp(t->words[i-1],"bridge"))
				mode="bridge";
			else if(!strcmp(t->words[i-1],"wan"))
				mode="wan";
			else {
				printf("format : reload [bridge|wan], default is reload all\n");
				return(0);
			}
			break;
		default:
			printf("format : reload [bridge|wan], default is reload all\n");
			return(0);
			break;
	}

	apmib_update(CURRENT_SETTING);
	printf("\n\tplease wait some minutes...\n\n");
	run_script(mode);
}

static int
dosys(t)
struct op *t;
{
	int i, getPort, getWeb, getIcmp, getTelent, getIPSec, getPPTP, getL2TP, getUPnP;
	char *pPort = NULL, *arg = NULL;
	unsigned int portnum;

	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;

	if(i>1) {
		if(!strcmp(t->words[0],"?") || !strcmp(t->words[0],"help")) {
			printf("Too many inputs !!!\n");
			return(1);
		}
	}

	switch(i) {
		case 0: //sys
			//printf("\nValid format: sys %s\n\n", SYS_CMDS);
			printf("Insufficient arguments !!!\n");
			return(1);
			break;
		case 1: //server, passthruvpn, upnp
			if(!strcmp(t->words[0],"reload")) { //update flash and script
				apmib_update(CURRENT_SETTING);
				printf("\n\tplease wait a moment...\n\n");
				arg = "wan";
				run_script(arg);
				return(1);
			}
			else if(!strcmp(t->words[i-1],"serv") || !strcmp(t->words[i-1],"server"))
				printf("format: sys server <access|port>\n");
			else if(!strcmp(t->words[i-1],"pass") || !strcmp(t->words[i-1],"passthruvpn"))
				printf("format: sys passthruvpn <ipsec|pptp|l2tp> <0:disabled|1:enabled>\n");
			else if(!strcmp(t->words[i-1],"upnp"))
				printf("format: sys upnp <0:disabled|1:enabled>\n");
			else
				//printf("format: sys %s\n\n", SYS_CMDS);
				invalid_subcmds(SYS_CMDS);
			break;
		case 2: //[server]access|port; [passthruvpn]ipsec|pptp|l2tp; [upnp]
			if(!strcmp(t->words[i-2],"serv") || !strcmp(t->words[i-2],"server")) {
    			if(!strcmp(t->words[i-1],"access"))
    				printf("format: sys server access <web|icmp|telnet> <0:disabled|1:enabled>\n");
    			else if(!strcmp(t->words[i-1],"port"))
    				printf("format: sys server port <web> <portnum>\n");
    			else
    				printf("format: sys server <access|port>\n");
			}
			else if(!strcmp(t->words[i-2],"pass") || !strcmp(t->words[i-2],"passthruvpn")) {
				if(!strcmp(t->words[i-1], "ipsec"))
					printf("format: sys passthruvpn ipsec <0:disabled|1:enabled>\n");
				else if(!strcmp(t->words[i-1], "pptp"))
					printf("format: sys passthruvpn pptp <0:disabled|1:enabled>\n");
				else if(!strcmp(t->words[i-1], "l2tp"))
					printf("format: sys passthruvpn l2tp <0:disabled|1:enabled>\n");
				else
					printf("format: sys passthruvpn <ipsec|pptp|l2tp> <0:disabled|1:enabled>\n");
			}
			else if(!strcmp(t->words[i-2],"upnp")) {
				if (!apmib_get(MIB_UPNP_ENABLED,(void *)&getUPnP)){
					printf("\n\tGet MIB_UPNP_ENABLED error\n\n");
					return 0;
				}
				if(!strcmp(t->words[i-1],"0")) {
					if(!getUPnP) {
						printf("\n\tthe same as system, no save\n\n");
						return 0;
					}
					else
						getUPnP=0;	//Disable UPnP service
				}
				else if(!strcmp(t->words[i-1],"1")) {
					if(getUPnP) {
						printf("\n\tthe same as system, no save\n\n");
						return 0;
					}
					else
						getUPnP=1;	//Enable UPnP service
				}
				else {
					printf("format: sys upnp <0:disabled|1:enabled>\n");
					if(getUPnP)
						printf("Now: UPnP service is enabled\n");
					else
						printf("Now: UPnP service is disabled\n");
					return 0;
				}

				if (!apmib_set(MIB_UPNP_ENABLED,(void *)&getUPnP)) {
					printf("\n\tSet MIB_UPNP_ENABLED error\n\n");
					return 0;
				}
			}//end of [upnp]
			else
				invalid_subcmds(SYS_CMDS);
			break;
		case 3: //[access]web|icmp|telnet, [port]web; [passthruvpn]ipsec|pptp|l2tp
			if(!strcmp(t->words[i-3],"serv") || !strcmp(t->words[i-3],"server")) {
    			if(!strcmp(t->words[i-2],"access")) {
    				if(!strcmp(t->words[i-1],"web"))
    					printf("format: sys server access web <0:disabled|1:enabled>\n");
    				else if(!strcmp(t->words[i-1],"icmp"))
    					printf("format: sys server access icmp <0:disabled|1:enabled>\n");
    				else if(!strcmp(t->words[i-1],"telnet"))
    					printf("format: sys server access telnet <0:disabled|1:enabled>\n");
    				else
    					printf("format: sys server access <web|icmp|telnet> <0:disabled|1:enabled>\n");
    			}
    			else if(!strcmp(t->words[i-2],"port")) {
    				if(!strcmp(t->words[i-1],"web"))
    					printf("format: sys server port web <portnum>\n");
    				else
    					printf("format: sys server port web <portnum>\n");
    			}
    			else
    				printf("format: sys server <access|port>\n");
			}
			else if(!strcmp(t->words[i-3],"pass") || !strcmp(t->words[i-3],"passthruvpn")) {
				if(!strcmp(t->words[i-2], "ipsec")) {
    				if (!apmib_get(VPN_PASSTHRU_IPSEC_ENABLED,(void *)&getIPSec)){
    					printf("\n\tGet VPN_PASSTHRU_IPSEC_ENABLED error\n\n");
    					return 0;
    				}
    				if(!strcmp(t->words[i-1],"0")) {
    					if(!getIPSec) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getIPSec=0;	//Disable IPsec pass through on VPN connection
    				}
    				else if(!strcmp(t->words[i-1],"1")) {
    					if(getIPSec) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getIPSec=1;	//Enable IPsec pass through on VPN connection
    				}
    				else {
    					printf("format: sys passthruvpn ipsec <0:disabled|1:enabled>\n");
    					if(getIPSec)
    						printf("Now: IPsec pass through on VPN connection is enabled\n");
    					else
    						printf("Now: IPsec pass through on VPN connection is disabled\n");
					return 0;
    				}
    				if (!apmib_set(VPN_PASSTHRU_IPSEC_ENABLED,(void *)&getIPSec)) {
    					printf("\n\tSet VPN_PASSTHRU_IPSEC_ENABLED error\n\n");
    					return 0;
    				}
				}//end of [passthruvpn]ipsec
				else if(!strcmp(t->words[i-2], "pptp")) {
    				if (!apmib_get(VPN_PASSTHRU_PPTP_ENABLED,(void *)&getPPTP)){
    					printf("\n\tGet VPN_PASSTHRU_PPTP_ENABLED error\n\n");
    					return 0;
    				}
    				if(!strcmp(t->words[i-1],"0")) {
    					if(!getPPTP) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getPPTP=0;	//Disable PPTP pass through on VPN connection
    				}
    				else if(!strcmp(t->words[i-1],"1")) {
    					if(getPPTP) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getPPTP=1;	//Enable PPTP pass through on VPN connection
    				}
    				else {
						printf("format: sys passthruvpn pptp <0:disabled|1:enabled>\n");
    					if(getPPTP)
    						printf("Now: PPTP pass through on VPN connection is enabled\n");
    					else
    						printf("Now: PPTP pass through on VPN connection is disabled\n");
					return 0;
    				}
    				if (!apmib_set(VPN_PASSTHRU_PPTP_ENABLED,(void *)&getPPTP)) {
    					printf("\n\tSet VPN_PASSTHRU_PPTP_ENABLED error\n\n");
    					return 0;
    				}
				}//end of [passthruvpn]pptp
				else if(!strcmp(t->words[i-2], "l2tp")) {
					if (!apmib_get(VPN_PASSTHRU_L2TP_ENABLED,(void *)&getL2TP)){
    					printf("\n\tGet VPN_PASSTHRU_L2TP_ENABLED error\n\n");
    					return 0;
    				}
    				if(!strcmp(t->words[i-1],"0")) {
    					if(!getL2TP) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getL2TP=0;	//Disable L2TP pass through on VPN connection
    				}
    				else if(!strcmp(t->words[i-1],"1")) {
    					if(getL2TP) {
    						printf("\n\tthe same as system, no save\n\n");
						return 0;
    					}
    					else
    						getL2TP=1;	//Enable L2TP pass through on VPN connection
    				}
    				else {
						printf("format: sys passthruvpn l2tp <0:disabled|1:enabled>\n");
    					if(getL2TP)
    						printf("Now: L2TP pass through on VPN connection is enabled\n");
    					else
    						printf("Now: L2TP pass through on VPN connection is disabled\n");
					return 0;
    				}
    				if (!apmib_set(VPN_PASSTHRU_L2TP_ENABLED,(void *)&getL2TP)) {
    					printf("\n\tSet VPN_PASSTHRU_L2TP_ENABLED error\n\n");
    					return 0;
    				}
				}//end of [passthruvpn]l2tp
				else
					printf("format: sys passthruvpn <ipsec|pptp|l2tp> <0:disabled|1:enabled>\n");
			}
			break;
		case 4: //[web|icmp|telnet / port] value
			if(!strcmp(t->words[i-3],"access")) {
				if(!strcmp(t->words[i-2],"web")) {
					if (!apmib_get(WEB_WAN_ACCESS_ENABLED,(void *)&getWeb)){
						printf("\n\tGet WEB_WAN_ACCESS_ENABLED error\n\n");
						return 0;
					}

					if(!strcmp(t->words[i-1],"0")) {
						if(!getWeb) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getWeb=0;	//Disable Web Server Access via WAN
					}
					else if(!strcmp(t->words[i-1],"1")) {
						if(getWeb) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getWeb=1;	//Enable Web Server Access via WAN
					}
					else {
						printf("format: sys server access web <0:disabled|1:enabled>\n");
						if(getWeb)
							printf("Now: server web (Enable Web Server Access via WAN) is enabled\n");
						else
							printf("Now: server web (Enable Web Server Access via WAN) is disabled\n");
						return 0;
					}

					if (!apmib_set(WEB_WAN_ACCESS_ENABLED,(void *)&getWeb)) {
						printf("\n\tSet WEB_WAN_ACCESS_ENABLED error\n\n");
						return 0;
					}
				}//end of [access]web
				else if(!strcmp(t->words[i-2],"icmp")) {
					if (!apmib_get(MIB_PING_WAN_ENABLED,(void *)&getIcmp)){
						printf("\n\tGet MIB_PING_WAN_ENABLED error\n\n");
						return 0;
					}

					if(!strcmp(t->words[i-1],"0")) {
						if(!getIcmp) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getIcmp=0;	//Disable Respond to WAN ping
					}
					else if(!strcmp(t->words[i-1],"1")) {
						if(getIcmp) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getIcmp=1;	//Enable Respond to WAN ping
					}
					else {
						printf("format: sys server access icmp <0:disabled|1:enabled>\n");
						if(getIcmp)
							printf("Now: server icmp(Respond to WAN ping) is enabled\n");
						else
							printf("Now: server icmp(Respond to WAN ping) is disabled\n");
						return 0;
					}

					if (!apmib_set(MIB_PING_WAN_ENABLED,(void *)&getIcmp)) {
						printf("\n\tSet MIB_PING_WAN_ENABLED error\n\n");
						return 0;
					}
				}//end of [access]icmp
				else if(!strcmp(t->words[i-2],"telnet")) {
					if (!apmib_get(MIB_REMOTE_TELNET,(void *)&getTelent)){
						printf("\n\tGet MIB_REMOTE_TELNET error\n\n");
						return 0;
					}

					if(!strcmp(t->words[i-1],"0")) {
						if(!getTelent) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getTelent=0;	//Disable Remote Telnet
					}
					else if(!strcmp(t->words[i-1],"1")) {
						if(getTelent) {
							printf("\n\tthe same as system, no save\n\n");
							return 0;
						}
						else
							getTelent=1;	//Enable Remote Telnet
					}
					else {
						printf("format: sys server access telnet <0:disabled|1:enabled>\n");
						if(getTelent)
							printf("Now: server telnet is enabled\n");
						else
							printf("Now: server telnet is disabled\n");
						return 0;
					}

					if (!apmib_set(MIB_REMOTE_TELNET,(void *)&getTelent)) {
						printf("\n\tSet MIB_REMOTE_TELNET error\n\n");
						return 0;
					}
				}//end of [access]telnet
				else
					printf("\nValid format: sys server access <web|icmp|telnet> <0:disabled|1:enabled>\n\n");
			}//end of [server]access
			else if(!strcmp(t->words[i-3],"port")) {
				if(!strcmp(t->words[i-2],"web")) {
					if (!apmib_get(MIB_WEB_WAN_ACCESS_PORT,(void *)&getPort)){
						printf("\n\tGet MIB_WEB_WAN_ACCESS_PORT error\n\n");
						return 0;
					}

					pPort = t->words[i-1];
					portnum=strtol(pPort, (char**)NULL, 10);

					if(portnum == getPort) {
						printf("\n\tthe same as system, no save\n\n");
						return 0;
					}
					else if(checkRemotePort(portnum)){
						if (!apmib_set(MIB_WEB_WAN_ACCESS_PORT,(void *)&portnum)){
							printf("\n\tSet MIB_WEB_WAN_ACCESS_PORT error\n\n");
							return 0;
						}
					}
					else {
						printf("format: sys server port <web> <portnum>\n");
						printf("Now: server web (Enable Web Server Access via WAN) port is %d\n\n", getPort);
						return 0;
					}
				}//end of [port]web
				else
					printf("format: sys server port <web> <portnum>\n");
			}//end of [server]port
			else
				printf("format: sys server <access|port>\n");
			break;
		default:
			invalid_subcmds(SYS_CMDS);
			break;
	}
	return(1);
}

static int
dowan(t)
struct op *t;
{
	register int i;
	int l,intVal;
	char *buff,macaddr[12];
	char *tgIp,*gIp;
	struct in_addr tIp,eIp,eMask,eGateway;
	unsigned long v1, v2, v3;
	
	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;
	
	if(i>1) {
		if(!strcmp(t->words[0],"?") || !strcmp(t->words[0],"help")) {
			printf("Too many inputs !!!\n");
			return(1);
		}
	}

	switch(i) {
		case 0:
			printf("Insufficient arguments !!!\n");
			return(1);
			break;
		case 1:
			/*if(!strcmp(t->words[i-1],"re") || !strcmp(t->words[i-1],"reload")) {
				printf("\n\tplease wait some minutes...\n\n");
				run_script();
			}
			else*/ if(!strcmp(t->words[i-1],"clo") || !strcmp(t->words[i-1],"clone"))
				printf("format: wan clone [mac_addr|ip_addr]\n");
			else if(!strcmp(t->words[i-1],"dns"))
				printf("format: wan dns [auto|manual]\n");
			else if(!strcmp(t->words[i-1],"stat") || !strcmp(t->words[i-1],"static"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"dh") || !strcmp(t->words[i-1],"dhcp")) {
				printf("format: wan dhcp [on|release|renew]\n");
				wan_status();
			}
			else if(!strcmp(t->words[i-1],"ppp") || !strcmp(t->words[i-1],"pppoe"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"pptp"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"l2") || !strcmp(t->words[i-1],"l2tp"))
				printf("Insufficient arguments !!!\n");
			else {
				invalid_subcmds(WAN_CMDS);
				wan_status();
			}
			break;
		case 2:
			if(!strcmp(t->words[i-2],"clo") || !strcmp(t->words[i-2],"clone")) {
				if (!apmib_get(MIB_WANCLONE_SELECT, (void *)&l)) {
					printf("\tGet WANCLONE_SELECT MIB Error\n");
					return(0);
				}
				buff=(char *)malloc(BUFFSIZE);
				if(l) {
					if (!apmib_get(MIB_PCMAC_WANCLONE_IP, (void *)&eIp)) {
						printf("\tGet PCMAC_WANCLONE_IP MIB Error\n");
						return(0);
					}
					gIp=inet_ntoa(eIp);
				}
				else {
					if (!apmib_get(MIB_WAN_MAC_ADDR, (void *)buff)) {
						printf("\tGet WAN_MAC_ADDR MIB Error\n");
						return(0);
					}
				}
				
				if(!strcmp(t->words[i-1],"?")) {
					printf("format: wan clone [mac_addr|ip_addr]\n");
					if(l)
						printf("Now: Clone PC's MAC %s\n",gIp);						
					else
						printf("Now: Clone MAC Address %02x%02x%02x%02x%02x%02x\n",buff[0], buff[1],buff[2], buff[3], buff[4], buff[5]);
				}
				else {
					if (!inet_aton(t->words[i-1], &eIp))  {
						if (strlen(t->words[i-1])!=12 || !string_to_hex(t->words[i-1], macaddr, 12)) {
							printf("Invalid IP address or MAC address input !!!\n");
							return(0);
						}
						else {
							if(!strcmp(buff,t->words[i-1]))
								printf("Now: Clone MAC Address %02x%02x%02x%02x%02x%02x\n",buff[0], buff[1],buff[2], buff[3], buff[4], buff[5]);
							else {
								intVal=0;
								if (!apmib_set(MIB_WANCLONE_SELECT, (void *)&intVal)) {
									printf("\tSet WANCLONE_SELECT MIB Error\n");
									return(0);
								}
								if (!apmib_set(MIB_WAN_MAC_ADDR, (void *)macaddr)) {
									printf("\tSet WAN_MAC_ADDR MIB Error\n");
									return(0);
								}
								//apmib_update(CURRENT_SETTING);
								//printf("\n\tplease wait some minutes...\n\n");
								//run_script();
							}
						}
					}
					else {
						if(!strcmp(gIp,t->words[i-1]))
							printf("Now: Clone PC's MAC %s\n",gIp);
						else {
							intVal=1;
							if (!apmib_set(MIB_WANCLONE_SELECT, (void *)&intVal)) {
								printf("\tSet WANCLONE_SELECT MIB Error\n");
								return(0);
							}
							if (!apmib_set(MIB_PCMAC_WANCLONE_IP, (void *)&eIp)) {
								printf("\tSet PCMAC_WANCLONE_IP MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
					}
					free(buff);
				}
			}
			else if(!strcmp(t->words[i-2],"dns")) {
				if (!apmib_get(MIB_WAN_DNS_MODE, (void *)&intVal)) {
					printf("\tGet WAN DNS Mode MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"au") || !strcmp(t->words[i-1],"auto")) {
					if(!intVal) {
						printf("Now: DNS Automatically\n");
						return(0);
					}
					intVal=0; // 0 = auto
					if (!apmib_set(MIB_WAN_DNS_MODE, (void *)&intVal)) {
						printf("\tSet WAN DNS Mode MIB Error\n");
						return(0);
					}
				}
				else if(!strcmp(t->words[i-1],"man") || !strcmp(t->words[i-1],"manual")) {
					if(intVal) {
						printf("Now: DNS Manually\n");
						return(0);
					}
					intVal=1; // 0 = auto
					if (!apmib_set(MIB_WAN_DNS_MODE, (void *)&intVal)) {
						printf("\tSet WAN DNS Mode MIB Error\n");
						return(0);
					}
				}
				else {
					printf("format: wan dns [auto|manual]\n");
					if(intVal)
						printf("Now: DNS Manually\n");
					else
						printf("Now: DNS Automatically\n");
					return(0);
				}

				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"stat") || !strcmp(t->words[i-2],"static")) {
				printf("format: wan static [ip_addr <netmask> <gateway>]\n");
				wan_status();
			}
			else if(!strcmp(t->words[i-2],"dh") || !strcmp(t->words[i-2],"dhcp")) {
				if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
					printf("\tGet WAN_DHCP MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"on")) {
					if(intVal != 1) {
						intVal=1;
						if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
							printf("\tGet WAN_DHCP MIB Error\n");
							return(0);
						}
						//apmib_update(CURRENT_SETTING);
						//printf("\n\tplease wait some minutes...\n\n");
						//run_script();
					}
					else
						wan_status();
				}
				if(!strcmp(t->words[i-1],"rel") || !strcmp(t->words[i-1],"release")) {
					if(intVal == 1) {
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"ifconfig %s 0.0.0.0",WAN_I);
						system(buff);
						free(buff);
						printf("WAN DHCP Released !!!\n");
					}
					else
						wan_status();
				}
				else if(!strcmp(t->words[i-1],"ren") || !strcmp(t->words[i-1],"renew")) {
					if(intVal == 1) {
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"ifconfig %s 0.0.0.0; %s/%s %s wait %s &",WAN_I,_CONFIG_SCRIPT_PATH,_DHCPC_SCRIPT_PROG,WAN_I,_NO_SCRIPT_MSG);
						system(buff);
						free(buff);
						printf("WAN DHCP Renew !!!\n");
					}
					else
						wan_status();
				}
				else {
					printf("format: wan dhcp [on|release|renew]\n");
					wan_status();
				}
			}
			else if(!strcmp(t->words[i-2],"ppp") || !strcmp(t->words[i-2],"pppoe"))
				pppoe_control(t,i);
			else if(!strcmp(t->words[i-2],"pptp"))
				tunnel_control(t,i);
			else if(!strcmp(t->words[i-2],"l2") || !strcmp(t->words[i-2],"l2tp"))
				tunnel_control(t,i);
			else
				invalid_subcmds(WAN_CMDS);	
			break;
		case 3:
			if(!strcmp(t->words[i-3],"stat") || !strcmp(t->words[i-3],"static")) {
				printf("format: wan static [ip_addr <netmask> <gateway>]\n");
				wan_status();
			}
			else if(!strcmp(t->words[i-3],"ppp") || !strcmp(t->words[i-3],"pppoe"))
				pppoe_control(t,i);
			else if(!strcmp(t->words[i-3],"pptp"))
				tunnel_control(t,i);
			else if(!strcmp(t->words[i-3],"l2") || !strcmp(t->words[i-3],"l2tp"))
				tunnel_control(t,i);
			else
				invalid_subcmds(WAN_CMDS);	
			break;
		case 4:
			if(!strcmp(t->words[i-4],"stat") || !strcmp(t->words[i-4],"static")) {
				if (!inet_aton(t->words[i-3], &eIp)) {
					printf("Invalid IP address input !!!\n");
					return(0);
				}
				if (!inet_aton(t->words[i-2], &eMask)) {
					printf("Invalid NetMask address input !!!\n");
					return(0);
				}
				if (!inet_aton(t->words[i-1], &eGateway)) {
					printf("Invalid Gateway address input !!!\n");
					return(0);
				}
				v1 = *((unsigned long *)&eGateway);
				v2 = *((unsigned long *)&eIp);
				v3 = *((unsigned long *)&eMask);
				if ( (v1 & v3) != (v2 & v3) )	{
					printf("Invalid Gateway or IP address! It should be set within the same subnet !!!\n");
					return(0);
				}
				if (!apmib_set(MIB_WAN_IP_ADDR, (void *)&eIp)) {
					printf("\tSet WAN_IP_ADDR MIB Error\n");
					return(0);
				}
				if (!apmib_set(MIB_WAN_SUBNET_MASK, (void *)&eMask)) {
					printf("\tSet WAN_SUBNET_MASK MIB Error\n");
					return(0);
				}
				if (!apmib_set(MIB_WAN_DEFAULT_GATEWAY, (void *)&eGateway)) {
					printf("\tSet WAN_DEFAULT_GATEWAY MIB Error\n");
					return(0);
				}
				intVal=0; // wan static ip
				if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
					printf("\tSet WAN_DHCP MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-4],"ppp") || !strcmp(t->words[i-4],"pppoe"))
				pppoe_control(t,i);
			else if(!strcmp(t->words[i-4],"pptp"))
				tunnel_control(t,i);
			else if(!strcmp(t->words[i-4],"l2") || !strcmp(t->words[i-4],"l2tp"))
				tunnel_control(t,i);
			else
				invalid_subcmds(WAN_CMDS);	
			break;
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
		case 10:
		case 11:
		case 12:
		case 13:
		case 14:
			if(!strcmp(t->words[0],"ppp") || !strcmp(t->words[0],"pppoe"))
				pppoe_control(t,i);
			else if(!strcmp(t->words[0],"pptp"))
				tunnel_control(t,i);
			else if(!strcmp(t->words[0],"l2") || !strcmp(t->words[0],"l2tp"))
				tunnel_control(t,i);
			else
				invalid_subcmds(WAN_CMDS);	
			break;
		
		default:
			invalid_subcmds(WAN_CMDS);
			wan_status();
			break;
	}
}

static int
doip(t)
struct op *t;
{
	int l;
	char *buff;
	int intVal;
	register int i;
	char *tgIp,*gIp,*gdns1,*gdns2,*gdns3;
	struct in_addr tIp,eIp,eMask,esdhcp,eedhcp,edns1,edns2,edns3;
	struct user_net_device_stats *elan_stats,*wlan_stats,*wan_stats;

	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;
	
	if(i>1) {
		if(!strcmp(t->words[0],"?") || !strcmp(t->words[0],"help")) {
			printf("Too many inputs !!!\n");
			return(1);
		}
	}
	
	switch(i) {
		case 0:
			printf("Insufficient arguments !!!\n");
			return(1);
			break;
		case 1: //address, dhcp, dns, ifconfig, ping, route, status, urlfilter, ipfilter, macfilter, nat
			if(!strcmp(t->words[i-1],"?"))
				invalid_subcmds(IP_CMDS);
			/*else if(!strcmp(t->words[i-1],"re") || !strcmp(t->words[i-1],"reload")) {
				printf("\n\tplease wait some minutes...\n\n");
				run_script();
			}*/
			else if(!strcmp(t->words[i-1],"addr") || !strcmp(t->words[i-1],"address"))
				printf("format: ip addr <IP address>\n");
			else if(!strcmp(t->words[i-1],"dh") || !strcmp(t->words[i-1],"dhcp"))
				invalid_subcmds(IP_DHCP_CMDS);
			else if(!strcmp(t->words[i-1],"dns"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"pi") || !strcmp(t->words[i-1],"ping"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"rou") || !strcmp(t->words[i-1],"route"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"stat") || !strcmp(t->words[i-1],"status")) {
				elan_stats=(struct user_net_device_stats *)malloc(sizeof(*elan_stats));
				if (getStats(ELAN_I, elan_stats) < 0) {
					elan_stats->tx_packets = 0;
					elan_stats->rx_packets = 0;
				}
				wan_stats=(struct user_net_device_stats *)malloc(sizeof(*wan_stats));
				if (getStats(WAN_I, wan_stats) < 0) {
					wan_stats->tx_packets = 0;
					wan_stats->rx_packets = 0;
				}
				wlan_stats=(struct user_net_device_stats *)malloc(sizeof(*wlan_stats));
				if (getStats(WLAN_I,wlan_stats) < 0) {
					wlan_stats->tx_packets = 0;
					wlan_stats->rx_packets = 0;
				}
				printf("==========================================\n");
				printf("Wireless LAN: \n");
				printf("\tSent Packets: %d\n",(int)wlan_stats->tx_packets);
				printf("\tReceived Packets: %d\n",(int)wlan_stats->rx_packets);
				printf("Ethernet LAN: \n");
				printf("\tSent Packets: %d\n",(int)elan_stats->tx_packets);
				printf("\tReceived Packets: %d\n",(int)elan_stats->rx_packets);
				printf("Ethernet WAN: \n");
				printf("\tSent Packets: %d\n",(int)wan_stats->tx_packets);
				printf("\tReceived Packets: %d\n",(int)wan_stats->rx_packets);
				printf("==========================================\n");
				free(elan_stats);
				free(wan_stats);
				free(wlan_stats);
			}
			else if(!strcmp(t->words[i-1],"nat"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"url") || !strcmp(t->words[i-1],"urlf") || !strcmp(t->words[i-1],"urlfilter"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"ipf") || !strcmp(t->words[i-1],"ipfilter"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"macf") || !strcmp(t->words[i-1],"macfilter"))
				printf("Insufficient arguments !!!\n");
			else if(!strcmp(t->words[i-1],"if") || !strcmp(t->words[i-1],"ifconf") || !strcmp(t->words[i-1],"ifconfig"))
				system("ifconfig");
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 2: // address, dhcp, dns, route, nat
			if(!strcmp(t->words[i-2],"addr") || !strcmp(t->words[i-2],"address")) {
				if (!apmib_get(MIB_IP_ADDR, (void *)&eIp)) {
					printf("\tGet IP Addr MIB Error\n");
					return(0);
				}
				gIp=inet_ntoa(eIp);
				
				if(!strcmp(t->words[i-1],"?")) {
					printf("format: ip addr <IP address>\n");
					printf("Now: %s\n",gIp);
				}
				else {
					if (!inet_aton(t->words[i-1], &eIp))  {
						printf("Invalid IP address input !!!\n");
						return(0);
					}
					if(!strcmp(gIp,t->words[i-1])) {
						printf("Now: %s\n",gIp);
						return(0);
					}
					if (!apmib_set(MIB_IP_ADDR, (void *)&eIp)) {
						printf("\tSet IP_ADDR MIB Error\n");
						return(0);
					}
					//apmib_update(CURRENT_SETTING);
					//printf("\n\tplease wait some minutes...\n\n");
					//run_script();
				}
			}
			else if(!strcmp(t->words[i-2],"dh") || !strcmp(t->words[i-2],"dhcp")) {
				if (!apmib_get(MIB_DHCP, (void *)&l)) {
					printf("\tGet DHCP MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"on")) {
					intVal=2;
					if (!apmib_set(MIB_DHCP, (void *)&intVal)) {
						printf("\tSet DHCP MIB Error\n");
						return(0);
					}
					//apmib_update(CURRENT_SETTING);
					//printf("\n\tplease wait some minutes...\n\n");
					//run_script();
				}
				else if(!strcmp(t->words[i-1],"off")) {
					intVal=0;
					if (!apmib_set(MIB_DHCP, (void *)&intVal)) {
						printf("\tSet DHCP MIB Error\n");
						return(0);
					}
					//apmib_update(CURRENT_SETTING);
					//printf("\n\tplease wait some minutes...\n\n");
					//run_script();
				}
				else if(!strcmp(t->words[i-1],"cl") || !strcmp(t->words[i-1],"client"))
					printf("format: ip dhcp client [start_ip <end_ip>]\n");
				else if(!strcmp(t->words[i-1],"sl") || !strcmp(t->words[i-1],"slease"))
					printf("Insufficient arguments !!!\n");
				else if(!strcmp(t->words[i-1],"stat") || !strcmp(t->words[i-1],"status")) {
					ip_dhcp_status();
				}
				else {
					invalid_subcmds(IP_DHCP_CMDS);
					if(l == 2)
						printf("Now: LAN dhcp on\n");
					else if(l == 0)
						printf("Now: LAN dhcp off\n");
				}
			}
			else if(!strcmp(t->words[i-2],"dns")) {
				if(!strcmp(t->words[i-1],"ser") || !strcmp(t->words[i-1],"serv") || !strcmp(t->words[i-1],"server"))
					printf("Insufficient arguments !!!\n");
				else
					invalid_subcmds(IP_DNS_CMDS);
			}
			else if(!strcmp(t->words[i-2],"pi") || !strcmp(t->words[i-2],"ping")) {
				if(!strcmp(t->words[i-1],"?"))
					printf("format: ip ping <IP address>\n");
				else {
					if (!inet_aton(t->words[i-1], &eIp))  {
						printf("Invalid IP address input !!!\n");
						return(0);
					}
					buff=(char *)malloc(BUFFSIZE);
					sprintf(buff,"ping -c %d %s",PING_COUNT,t->words[i-1]);
					system(buff);
					free(buff);
				}
			}
			else if(!strcmp(t->words[i-2],"rou") || !strcmp(t->words[i-2],"route")) {
				if(!strcmp(t->words[i-1],"?"))
					invalid_subcmds(IP_ROUTE_CMDS);
				else if(!strcmp(t->words[i-1],"stat") || !strcmp(t->words[i-1],"status")) {
					buff=(char *)malloc(BUFFSIZE);
					sprintf(buff,"route -n -e");
					system(buff);
					free(buff);
				}
				else if(!strcmp(t->words[i-1],"add"))
					printf("format: ip route add <dest_addr|default> <gateway> [<metric>]\n");
				else if(!strcmp(t->words[i-1],"dr") || !strcmp(t->words[i-1],"drop"))
					printf("format: ip route drop <host addr>\n");
				else
					invalid_subcmds(IP_ROUTE_CMDS);
			}
			else if(!strcmp(t->words[i-2],"nat")) {
				if(!strcmp(t->words[i-1],"server")) {
					invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-1],"service")) {
					invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				//else if(!strcmp(t->words[i-1],"reset") || !strcmp(t->words[i-1],"resetport"))
				//	nat_server_clearall(); // nat server resetport
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else if(!strcmp(t->words[i-2],"url") || !strcmp(t->words[i-2],"urlf") || !strcmp(t->words[i-2],"urlfilter")) {
				if(!strcmp(t->words[i-1],"cust") || !strcmp(t->words[i-1],"custom") || !strcmp(t->words[i-1],"customize"))
					invalid_subcmds(IP_F_CUST_CMDS);
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-2],"ipf") || !strcmp(t->words[i-2],"ipfilter")) {
				if(!strcmp(t->words[i-1],"cust") || !strcmp(t->words[i-1],"custom") || !strcmp(t->words[i-1],"customize"))
					invalid_subcmds(IP_F_CUST_CMDS);
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-2],"macf") || !strcmp(t->words[i-2],"macfilter")) {
				if(!strcmp(t->words[i-1],"cust") || !strcmp(t->words[i-1],"custom") || !strcmp(t->words[i-1],"customize"))
					invalid_subcmds(IP_F_CUST_CMDS);
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-2],"if") || !strcmp(t->words[i-2],"ifconf") || !strcmp(t->words[i-2],"ifconfig")) {
				l=atoi(t->words[i-1]);
				buff=(char *)malloc(BUFFSIZE);
				if(l == 1) { // WAN interface
					sprintf(buff,"ifconfig %s",WAN_I);
					system(buff);
				}
				else if(l == 2) { // LAN interface
					sprintf(buff,"ifconfig %s",ELAN_I);
					system(buff);
				}
				else
					ifconfig_format();
				free(buff);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 3: //dhcp, dns, route, nat
			if(!strcmp(t->words[i-3],"dh") || !strcmp(t->words[i-3],"dhcp")) {
				if(!strcmp(t->words[i-2],"cl") || !strcmp(t->words[i-2],"client")) {
					printf("format: ip dhcp client [start_ip <end_ip>]\n");
				}
				else if(!strcmp(t->words[i-2],"sl") || !strcmp(t->words[i-2],"slease")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp")) {
						ip_dhcp_disp();
					}
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall")) {
						ip_dhcp_clearall();
					}
					else if(!strcmp(t->words[i-1],"add")) {
						printf("Insufficient arguments !!!\n");
						printf("format: ip dhcp slease add [mac_addr <ip_addr>]\n");
					}
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete")) {
						printf("Insufficient arguments !!!\n");
						printf("format: ip dhcp slease delete [id]\n");
					}
					else
						invalid_subcmds(IP_DHCP_SLEASE_CMDS);
				}
				else if(!strcmp(t->words[i-2],"stat") || !strcmp(t->words[i-2],"status"))
					ip_dhcp_status();
				else
					invalid_subcmds(IP_DHCP_CMDS);
			}
			else if(!strcmp(t->words[i-3],"dns")) {
				if(!strcmp(t->words[i-2],"ser") || !strcmp(t->words[i-2],"serv") || !strcmp(t->words[i-2],"server")) {
					if (!apmib_get(MIB_WAN_DNS_MODE, (void *)&intVal)) {
						printf("\tGet WAN DNS Mode MIB Error\n");
						return(0);
					}
					if(!intVal) 
						printf("DNS Mode: Automatically\n");
					else {
						if ( !inet_aton(t->words[i-1], &edns1) ) {
							printf("Invalid Primary DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS1, (void *)&edns1)) {
							printf("\tSet WAN DNS1 MIB Error\n");
							return(0);
						}
						printf("Set Primary DNS Server: %s\n",t->words[i-1]);
						apmib_update(CURRENT_SETTING);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"echo 'nameserver %s' > %s",t->words[i-1],_DNS_M_CONFIG);
						system(buff);
						free(buff);
						restart_daemon(_WEBS_PROG_NAME);
						dns_dnrd(1,t);
					}
				}
				else 
					invalid_subcmds(IP_DNS_CMDS);
			}
			else if(!strcmp(t->words[i-3],"rou") || !strcmp(t->words[i-3],"route")) {
				if(!strcmp(t->words[i-2],"ad") || !strcmp(t->words[i-2],"add"))
					printf("format: ip route add <dest_addr|default> <gateway> [<metric>]\n");
				else if(!strcmp(t->words[i-2],"dr") || !strcmp(t->words[i-2],"drop")) {
					if(!strcmp(t->words[i-1],"?"))
						printf("format: ip route drop <host_addr|default>\n");
					else {
						if(!strcmp(t->words[i-1],"default")) {
							buff=(char *)malloc(BUFFSIZE);
							sprintf(buff,"route del %s",t->words[i-1]);
							system(buff);
							free(buff);
							printf("Route Drop %s\n",t->words[i-1]);
						}
						else {
							if (!inet_aton(t->words[i-1], &eIp))  {
								printf("Invalid Host IP address input !!!\n");
								return(0);
							}
							buff=(char *)malloc(BUFFSIZE);
							sprintf(buff,"route del %s",t->words[i-1]);
							system(buff);
							free(buff);
							printf("Route Drop %s\n",t->words[i-1]);
						}
					}
				}
			}
			else if(!strcmp(t->words[i-3],"nat")) {
				if(!strcmp(t->words[i-2],"server")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp"))
						nat_server_disp(); //nat server disp
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete"))
						printf("format: ip nat server delete <id>\n");
					else if(!strcmp(t->words[i-1],"add"))
						nat_server_add_format();
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall"))
						nat_server_clearall();
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-2],"service")) {
					if(!strcmp(t->words[i-1],"sap1") || !strcmp(t->words[i-1],"sap2") || !strcmp(t->words[i-1],"sap3") || !strcmp(t->words[i-1],"sap4") || !strcmp(t->words[i-1],"sap5") || !strcmp(t->words[i-1],"sap6") || !strcmp(t->words[i-1],"sap7") || !strcmp(t->words[i-1],"sap8"))
						printf("format: ip nat service %s [on|off|clear|edit <name> <in_proto> <in_from_port> <in_to_port> <trig_proto> <trig_from_port> <trig_to_port>]\n",t->words[i-1]);
					else if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp"))
						sap_disp();
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else if(!strcmp(t->words[i-3],"url") || !strcmp(t->words[i-3],"urlf") || !strcmp(t->words[i-3],"urlfilter")) {
				if(!strcmp(t->words[i-2],"cust") || !strcmp(t->words[i-2],"custom") || !strcmp(t->words[i-2],"customize")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp"))
						urlfilter_disp();
					else if(!strcmp(t->words[i-1],"ad") || !strcmp(t->words[i-1],"add"))
						printf("format: ip urlfilter customize add [url]\n");
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete"))
						printf("format: ip urlfilter customize delete [id]\n");
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall"))
						urlfilter_clearall();
					else
						invalid_subcmds(IP_F_CUST_CMDS);
				}
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-3],"ipf") || !strcmp(t->words[i-3],"ipfilter")) {
				if(!strcmp(t->words[i-2],"cust") || !strcmp(t->words[i-2],"custom") || !strcmp(t->words[i-2],"customize")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp"))
						ipfilter_disp();
					else if(!strcmp(t->words[i-1],"ad") || !strcmp(t->words[i-1],"add"))
						printf("format: ip ipfilter customize add [url]\n");
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete"))
						printf("format: ip ipfilter customize delete [id]\n");
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall"))
						ipfilter_clearall();
					else
						invalid_subcmds(IP_F_CUST_CMDS);
				}
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-3],"macf") || !strcmp(t->words[i-3],"macfilter")) {
				if(!strcmp(t->words[i-2],"cust") || !strcmp(t->words[i-2],"custom") || !strcmp(t->words[i-2],"customize")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp"))
						macfilter_disp();
					else if(!strcmp(t->words[i-1],"ad") || !strcmp(t->words[i-1],"add"))
						printf("format: ip macfilter customize add [url]\n");
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete"))
						printf("format: ip macfilter customize delete [id]\n");
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall"))
						macfilter_clearall();
					else
						invalid_subcmds(IP_F_CUST_CMDS);
				}
				else
					invalid_subcmds(IP_F_CMDS);
			}
			else if(!strcmp(t->words[i-3],"if") || !strcmp(t->words[i-3],"ifconf") || !strcmp(t->words[i-3],"ifconfig")) {
				if (!inet_aton(t->words[i-1], &tIp))  {
					printf("Invalid IP address input !!!\n");
					return(0);
				}
				
				l=atoi(t->words[i-2]);
				buff=(char *)malloc(BUFFSIZE);
				if(l == 1) { // WAN interface
					if (!apmib_get(MIB_WAN_IP_ADDR, (void *)&eIp)) {
						printf("\tGet WAN IP Addr MIB Error\n");
						return(0);
					}
					gIp=inet_ntoa(eIp);
					if(!strcmp(gIp,t->words[i-1]))
						printf("Now: WAN IP Address %s\n",t->words[i-1]);
					else {
						if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
							printf("\tGet MIB_WAN_DHCP MIB Error\n");
							return(0);
						}
						//confirm WAN is fixed ip or not
						if(intVal != 0) { // WAN fixed ip mode
							intVal=0;
							if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
								printf("\tSet MIB_WAN_DHCP MIB Error\n");
								return(0);
							}
						}
						if (!apmib_set(MIB_WAN_IP_ADDR, (void *)&tIp)) {
							printf("\tGet WAN IP Addr MIB Error\n");
							return(0);
						}
						apmib_update(CURRENT_SETTING);
						sprintf(buff,"ifconfig %s %s",WAN_I,t->words[i-1]);
						system(buff);
						restart_daemon(_WEBS_PROG_NAME);
						printf("WAN IP Address Change to %s !!!\n",t->words[i-1]);
					}
				}
				else if(l == 2) { // LAN interface
					if (!apmib_get(MIB_IP_ADDR, (void *)&eIp)) {
						printf("\tGet IP Addr MIB Error\n");
						return(0);
					}
					gIp=inet_ntoa(eIp);
					if(!strcmp(gIp,t->words[i-1]))
						printf("Now: LAN IP Address %s\n",t->words[i-1]);
					else {
						if (!apmib_set(MIB_IP_ADDR, (void *)&tIp)) {
							printf("\tSet LAN IP Addr MIB Error\n");
							return(0);
						}
						apmib_update(CURRENT_SETTING);
						restart_daemon(_WEBS_PROG_NAME);
						printf("LAN IP Address Change to %s !!!\n", t->words[i-1]);
						printf("Please Re-login again !!!\n");
						sprintf(buff,"ifconfig %s %s",ELAN_I,t->words[i-1]);
						system(buff);
					}
				}
				else
					ifconfig_format();
				free(buff);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 4: // dhcp, dns, route, nat, urlfilter, ipfilter, macfilter
			if(!strcmp(t->words[i-4],"dh") || !strcmp(t->words[i-4],"dhcp")) {
				if(!strcmp(t->words[i-3],"cl") || !strcmp(t->words[i-3],"client")) {
					if (!inet_aton(t->words[i-2], &esdhcp)) {
						printf("Invalid DHCP start address input !!!\n");
						return(0);
					}
					if (!inet_aton(t->words[i-1], &eedhcp)) {
						printf("Invalid DHCP end address input !!!\n");
						return(0);
					}
					if (!apmib_set(MIB_DHCP_CLIENT_START, (void *)&esdhcp)) {
						printf("\tSet DHCP start address MIB Error\n");
						return(0);
					}
					if (!apmib_set(MIB_DHCP_CLIENT_END, (void *)&eedhcp)) {
						printf("\tSet DHCP end address MIB Error\n");
						return(0);
					}
					apmib_update(CURRENT_SETTING);
					restart_daemon(_WEBS_PROG_NAME);
				}
				else if(!strcmp(t->words[i-3],"sl") || !strcmp(t->words[i-3],"slease")) {
					if(!strcmp(t->words[i-2],"dis") || !strcmp(t->words[i-2],"disp")) {
						ip_dhcp_disp();
					}
					else if(!strcmp(t->words[i-2],"cla") || !strcmp(t->words[i-2],"clearall")) {
						ip_dhcp_clearall();
					}
					else if(!strcmp(t->words[i-2],"add")) {
						printf("Insufficient arguments !!!\n");
						printf("format: ip dhcp slease add [mac_addr <ip_addr>]\n");
					}
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No Such %d static leased client !!!\n",l);
							return(0);
						}
						if(!apmib_get(MIB_STATICLEASE_NUM,(void *)&intVal)) {
							printf("\tGet STATICLEASE_NUM MIB Error\n");
							return(0);
						}
						if(l > intVal)
							printf("No Such %d static leased client !!!\n",l);
						else
							ip_dhcp_del(l);
					}
					else
						invalid_subcmds(IP_DHCP_SLEASE_CMDS);
				}
				else if(!strcmp(t->words[i-2],"stat") || !strcmp(t->words[i-2],"status"))
					ip_dhcp_status();
				else
					invalid_subcmds(IP_DHCP_CMDS);
			}
			else if(!strcmp(t->words[i-4],"dns")) {
				if(!strcmp(t->words[i-3],"ser") || !strcmp(t->words[i-3],"serv") || !strcmp(t->words[i-3],"server")) {
					if (!apmib_get(MIB_WAN_DNS_MODE, (void *)&intVal)) {
						printf("\tGet WAN DNS Mode MIB Error\n");
						return(0);
					}
					if(!intVal) 
						printf("DNS Mode: Automatically\n");
					else {
						if (!inet_aton(t->words[i-2], &edns1)) {
							printf("Invalid Primary DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS1, (void *)&edns1)) {
							printf("\tSet WAN DNS1 MIB Error\n");
							return(0);
						}
						if (!inet_aton(t->words[i-1], &edns2)) {
							printf("Invalid Secondary DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS2, (void *)&edns2)) {
							printf("\tSet WAN DNS2 MIB Error\n");
							return(0);
						}
						printf("Set Primary DNS Server: %s\n",t->words[i-2]);
						printf("Set Secondary DNS Server: %s\n",t->words[i-1]);
						apmib_update(CURRENT_SETTING);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"echo 'nameserver %s' > %s",t->words[i-2],_DNS_M_CONFIG);
						system(buff);
						sprintf(buff,"echo 'nameserver %s' >> %s",t->words[i-1],_DNS_M_CONFIG);
						system(buff);
						free(buff);
						restart_daemon(_WEBS_PROG_NAME);
						dns_dnrd(2,t);
					}
				}
				else
					invalid_subcmds(IP_DNS_CMDS);
			}
			else if(!strcmp(t->words[i-4],"rou") || !strcmp(t->words[i-4],"route")) {
				if(!strcmp(t->words[i-3],"add")) {
					if(!strcmp(t->words[i-2],"def") || !strcmp(t->words[i-2],"default")) {
						if (!inet_aton(t->words[i-1], &eIp))  {
							printf("Invalid Gateway IP address input !!!\n");
							return(0);
						}
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"route add %s gw %s",t->words[i-2],t->words[i-1]);
						system(buff);
						free(buff);
						printf("Add one Route %s to %s\n",t->words[i-2],t->words[i-1]);
					}
					else {
						if (!inet_aton(t->words[i-2], &eIp))  {
							printf("Invalid Dest IP address input !!!\n");
							return(0);
						}
						if (!inet_aton(t->words[i-1], &eIp))  {
							printf("Invalid Gateway IP address input !!!\n");
							return(0);
						}
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"route add %s gw %s",t->words[i-2],t->words[i-1]);
						system(buff);
						free(buff);
						printf("Add one Route %s to %s !!!\n",t->words[i-2],t->words[i-1]);
					}
				}
				else
					invalid_subcmds(IP_ROUTE_CMDS);
			}
			else if(!strcmp(t->words[i-4],"nat")) {
				if(!strcmp(t->words[i-3],"server")) {
					if(!strcmp(t->words[i-2],"dis") || !strcmp(t->words[i-2],"disp"))
						nat_server_disp();
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No such %d item !!!\n",l);
							return(0);
						}
						nat_server_delete(l);
					}
					else if(!strcmp(t->words[i-2],"add"))
						nat_server_add_format();
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-3],"service")) {
					if(!strcmp(t->words[i-2],"sap1") || !strcmp(t->words[i-2],"sap2") || !strcmp(t->words[i-2],"sap3") || !strcmp(t->words[i-2],"sap4") || !strcmp(t->words[i-2],"sap5") || !strcmp(t->words[i-2],"sap6") || !strcmp(t->words[i-2],"sap7") || !strcmp(t->words[i-2],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else if(!strcmp(t->words[i-4],"url") || !strcmp(t->words[i-4],"urlf") || !strcmp(t->words[i-4],"urlfilter")) {
				if(!strcmp(t->words[i-3],"cust") || !strcmp(t->words[i-3],"custom") || !strcmp(t->words[i-3],"customize")) {
					if(!strcmp(t->words[i-2],"add"))
						urlfilter_add(t->words[i-1]);
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No Such %d urlfilter !!!\n",l);
							return(0);
						}
						if(!apmib_get(MIB_URLFILTER_NUM,(void *)&intVal)) {
							printf("\tGet URLFILTER_NUM MIB Error\n");
							return(0);
						}
						if(l > intVal)
							printf("No Such %d urlfilter !!!\n",l);
						else
							urlfilter_del(l);
					}
				}
				else
					invalid_subcmds(IP_F_CUST_CMDS);
			}
			else if(!strcmp(t->words[i-4],"ipf") || !strcmp(t->words[i-4],"ipfilter")) {
				if(!strcmp(t->words[i-3],"cust") || !strcmp(t->words[i-3],"custom") || !strcmp(t->words[i-3],"customize")) {
					if(!strcmp(t->words[i-2],"add")) {
						//ipfilter_add(t->words[i-1]);
						printf("Insufficient arguments !!!\n");
						printf("format: ip ipfilter customize add [ip_addr] <1:TCP+UDP|2:TCP|3:UDP>\n");
					}
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No Such %d ipfilter !!!\n",l);
							return(0);
						}
						if(!apmib_get(MIB_IPFILTER_NUM,(void *)&intVal)) {
							printf("\tGet URLFILTER_NUM MIB Error\n");
							return(0);
						}
						if(l > intVal)
							printf("No Such %d ipfilter !!!\n",l);
						else
							ipfilter_del(l);
					}
				}
				else
					invalid_subcmds(IP_F_CUST_CMDS);
			}
			else if(!strcmp(t->words[i-4],"macf") || !strcmp(t->words[i-4],"macfilter")) {
				if(!strcmp(t->words[i-3],"cust") || !strcmp(t->words[i-3],"custom") || !strcmp(t->words[i-3],"customize")) {
					if(!strcmp(t->words[i-2],"add"))
						macfilter_add(t->words[i-1]);
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No Such %d macfilter !!!\n",l);
							return(0);
						}
						if(!apmib_get(MIB_MACFILTER_NUM,(void *)&intVal)) {
							printf("\tGet MACFILTER_NUM MIB Error\n");
							return(0);
						}
						if(l > intVal)
							printf("No Such %d macfilter !!!\n",l);
						else
							macfilter_del(l);
					}
				}
				else
					invalid_subcmds(IP_F_CUST_CMDS);
			}
			else if(!strcmp(t->words[i-4],"if") || !strcmp(t->words[i-4],"ifconf") || !strcmp(t->words[i-4],"ifconfig")) {
				if (!inet_aton(t->words[i-2], &eIp))  {
					printf("Invalid IP address input !!!\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"dyn") || !strcmp(t->words[i-1],"dynam") || !strcmp(t->words[i-1],"dynamic")) {
					l=atoi(t->words[i-3]);
					if(l == 1) { // WAN interface
						if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
							printf("\tGet MIB_WAN_DHCP MIB Error\n");
							return(0);
						}
						//confirm WAN is dynamic or not
						if(intVal != 1) { // WAN DHCP mode
							intVal=1;
							if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
								printf("\tSet MIB_WAN_DHCP MIB Error\n");
								return(0);
							}
							printf("Dynamic WAN IP Address Successfully !!!\n");
						}
						else
							printf("Rebew Dynamic WAN IP Address !!!\n");
						apmib_update(CURRENT_SETTING);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"ifconfig %s 0.0.0.0; %s/%s %s wait %s &",WAN_I,_CONFIG_SCRIPT_PATH,_DHCPC_SCRIPT_PROG,WAN_I,_NO_SCRIPT_MSG);
						system(buff);
						restart_daemon(_WEBS_PROG_NAME);
						printf("WAN IP Address Change to Dynamic !!!\n");
						free(buff);
					}
					else if(l == 2) // LAN interface
						printf("Dynamic not support at LAN IP Address !!!\n");
					else
						ifconfig_format();
				}
				else
					ifconfig_format();
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 5: // dns, route, nat, ipfilter
			if(!strcmp(t->words[i-5],"dh") || !strcmp(t->words[i-5],"dhcp")) {
				if(!strcmp(t->words[i-4],"cl") || !strcmp(t->words[i-4],"client")) {
					printf("Too many inputs !!!\n");
					printf("format : ip dhcp client [start_ip <end_ip>]\n");
				}
				else if(!strcmp(t->words[i-4],"sl") || !strcmp(t->words[i-4],"slease")) {
					if(!strcmp(t->words[i-3],"dis") || !strcmp(t->words[i-3],"disp")) {
						ip_dhcp_disp();
					}
					else if(!strcmp(t->words[i-3],"cla") || !strcmp(t->words[i-3],"clearall")) {
						ip_dhcp_clearall();
					}
					else if(!strcmp(t->words[i-3],"add")) {
						ip_dhcp_add(t->words[i-2],t->words[i-1]);
						/*printf("Insufficient arguments !!!\n");
						printf("format: ip dhcp slease add [mac_addr <ip_addr>]\n");*/
					}
					else if(!strcmp(t->words[i-3],"del") || !strcmp(t->words[i-3],"delete")) {
						printf("Too many inputs !!!\n");
						printf("format : ip dhcp client delete [id]\n");
					}
					else
						invalid_subcmds(IP_DHCP_SLEASE_CMDS);
				}
				else if(!strcmp(t->words[i-2],"stat") || !strcmp(t->words[i-2],"status"))
					ip_dhcp_status();
				else
					invalid_subcmds(IP_DHCP_CMDS);
			}
			else if(!strcmp(t->words[i-5],"dns")) {
				if(!strcmp(t->words[i-4],"ser") || !strcmp(t->words[i-4],"serv") || !strcmp(t->words[i-4],"server")) {
					if (!apmib_get(MIB_WAN_DNS_MODE, (void *)&intVal)) {
						printf("\tGet WAN DNS Mode MIB Error\n");
						return(0);
					}
					if(!intVal) 
						printf("DNS Mode: Automatically\n");
					else {
						if (!inet_aton(t->words[i-3], &edns1)) {
							printf("Invalid Primary DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS1, (void *)&edns1)) {
							printf("\tSet WAN DNS1 MIB Error\n");
							return(0);
						}
						if (!inet_aton(t->words[i-2], &edns2)) {
							printf("Invalid Secondary DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS2, (void *)&edns2)) {
							printf("\tSet WAN DNS2 MIB Error\n");
							return(0);
						}
						if (!inet_aton(t->words[i-1], &edns3)) {
							printf("Invalid Third DNS IP address input !!!\n");
							return(0);
						}
						if (!apmib_set(MIB_WAN_DNS3, (void *)&edns3)) {
							printf("\tSet WAN DNS3 MIB Error\n");
							return(0);
						}
						printf("Set Primary DNS Server: %s\n",t->words[i-3]);
						printf("Set Secondary DNS Server: %s\n",t->words[i-2]);
						printf("Set Third DNS Server: %s\n",t->words[i-1]);
						apmib_update(CURRENT_SETTING);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"echo 'nameserver %s' > %s",t->words[i-3],_DNS_M_CONFIG);
						system(buff);
						sprintf(buff,"echo 'nameserver %s' >> %s",t->words[i-2],_DNS_M_CONFIG);
						system(buff);
						sprintf(buff,"echo 'nameserver %s' >> %s",t->words[i-1],_DNS_M_CONFIG);
						system(buff);
						free(buff);
						restart_daemon(_WEBS_PROG_NAME);
						dns_dnrd(3,t);
					}
				}
				else
					invalid_subcmds(IP_DNS_CMDS);
			}
			else if(!strcmp(t->words[i-5],"rou") || !strcmp(t->words[i-5],"route")) {
				if(!strcmp(t->words[i-4],"add")) {
					if(!strcmp(t->words[i-3],"def") || !strcmp(t->words[i-3],"default")) {
						if (!inet_aton(t->words[i-2], &eIp))  {
							printf("Invalid Gateway IP address input !!!\n");
							return(0);
						}
						intVal=atoi(t->words[i-1]);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"route add %s gw %s metric %d",t->words[i-3],t->words[i-2],intVal);
						system(buff);
						free(buff);
						printf("Add one Route %s to %s\n with metric %d !!!",t->words[i-3],t->words[i-2],intVal);
					}
					else {
						if (!inet_aton(t->words[i-3], &eIp))  {
							printf("Invalid Dest IP address input !!!\n");
							return(0);
						}
						if (!inet_aton(t->words[i-2], &eIp))  {
							printf("Invalid Gateway IP address input !!!\n");
							return(0);
						}
						intVal=atoi(t->words[i-1]);
						buff=(char *)malloc(BUFFSIZE);
						sprintf(buff,"route add %s gw %s metric %d",t->words[i-3],t->words[i-2],intVal);
						system(buff);
						free(buff);
						printf("Add one Route %s to %s with metric %d !!!\n",t->words[i-3],t->words[i-2],intVal);
					}
				}
				else
					invalid_subcmds(IP_ROUTE_CMDS);
			}
			else if(!strcmp(t->words[i-5],"nat")) {
				if(!strcmp(t->words[i-4],"server")) {
					if(!strcmp(t->words[i-3],"dis") || !strcmp(t->words[i-3],"disp"))
						nat_server_disp();
					else if(!strcmp(t->words[i-3],"add")) {
						if (!inet_aton(t->words[i-2], &eIp))  {
							printf("Invalid IP address input !!!\n");
							return(0);
						}
						l=atoi(t->words[i-1]);
						if(l)
							nat_server_add(t,l,0); // setup pre-define server such as web, ftp and so on...
						else
							nat_server_add_format();
					}
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-4],"service")) {
					if(!strcmp(t->words[i-3],"sap1") || !strcmp(t->words[i-3],"sap2") || !strcmp(t->words[i-3],"sap3") || !strcmp(t->words[i-3],"sap4") || !strcmp(t->words[i-3],"sap5") || !strcmp(t->words[i-3],"sap6") || !strcmp(t->words[i-3],"sap7") || !strcmp(t->words[i-3],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else if(!strcmp(t->words[i-5],"ipf") || !strcmp(t->words[i-5],"ipfilter")) {
				if(!strcmp(t->words[i-4],"cust") || !strcmp(t->words[i-4],"custom") || !strcmp(t->words[i-4],"customize")) {
					if(!strcmp(t->words[i-3],"add")) {
						l=atoi(t->words[i-1]);
						ipfilter_add(t->words[i-2],l);
					}
					else if(!strcmp(t->words[i-3],"del") || !strcmp(t->words[i-3],"delete")) {
						printf("Too many inputs !!!\n");
						printf("format: ip ipfilter customize delete [id]\n");
					}
				}
				else
					invalid_subcmds(IP_F_CUST_CMDS);
			}
			else if(!strcmp(t->words[i-5],"if") || !strcmp(t->words[i-5],"ifconf") || !strcmp(t->words[i-5],"ifconfig")) {
				if (!inet_aton(t->words[i-3], &eIp))  {
					printf("Invalid IP address input !!!\n");
					return(0);
				}
				if(!strcmp(t->words[i-2],"broad") || !strcmp(t->words[i-2],"broadcast")) {
					if (!inet_aton(t->words[i-1], &tIp))  {
						printf("Invalid Broadcast input !!!\n");
						return(0);
					}
					l=atoi(t->words[i-4]);
					buff=(char *)malloc(BUFFSIZE);
					if(l == 1) { // WAN interface
						if (!apmib_get(MIB_WAN_IP_ADDR, (void *)&tIp)) {
							printf("\tGet WAN IP Addr MIB Error\n");
							return(0);
						}
						gIp=inet_ntoa(tIp);
						if(!strcmp(gIp,t->words[i-3]))
							printf("Now: WAN IP Address %s\n",t->words[i-1]);
						else {
							if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
								printf("\tGet WAN_DHCP MIB Error\n");
								return(0);
							}
							if(intVal != 0) { // WAN fixed ip mode
								intVal=0;
								if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
									printf("\tSet MIB_WAN_DHCP MIB Error\n");
									return(0);
								}
							}
							if (!apmib_set(MIB_WAN_IP_ADDR, (void *)&eIp)) {
								printf("\tSet WAN IP Addr MIB Error\n");
								return(0);
							}
							sprintf(buff,"ifconfig %s %s broadcast %s",WAN_I,t->words[i-3],t->words[i-1]);
							system(buff);
							apmib_update(CURRENT_SETTING);
							restart_daemon(_WEBS_PROG_NAME);
							printf("WAN IP Address Change to %s with broadcast %s !!!\n", t->words[i-3], t->words[i-1]);
						}
					}
					else if(l == 2) { // LAN interface
						if (!apmib_get(MIB_IP_ADDR, (void *)&tIp)) {
							printf("\tGet LAN IP Addr MIB Error\n");
							return(0);
						}
						gIp=inet_ntoa(tIp);
						if(!strcmp(gIp,t->words[i-3]))
							printf("Now: LAN IP Address %s\n",t->words[i-1]);
						else {
							if (!apmib_set(MIB_IP_ADDR, (void *)&eIp)) {
								printf("\tSet LAN IP Addr MIB Error\n");
								return(0);
							}
							apmib_update(CURRENT_SETTING);
							restart_daemon(_WEBS_PROG_NAME);
							printf("LAN IP Address Change to %s with broadcast %s !!!\n", t->words[i-3],t->words[i-1]);
							printf("Please Re-login again !!!\n");
							sprintf(buff,"ifconfig %s %s broadcast %s",ELAN_I,t->words[i-3],t->words[i-1]);
							system(buff);
						}
					}
					free(buff);
				}
				else if(!strcmp(t->words[i-2],"mtu")) {
					if(atoi(t->words[i-1]) < 1 || atoi(t->words[i-1]) > 1500) {
						printf("Invalid Mtu value. It should be set less than 1500 !!!");
						return(0);
					}
						
					l=atoi(t->words[i-4]);
					if(l == 1) { // WAN interface
						if (!apmib_get(MIB_WAN_IP_ADDR, (void *)&tIp)) {
							printf("\tGet WAN IP Addr MIB Error\n");
							return(0);
						}
						gIp=inet_ntoa(tIp);
						if(!strcmp(gIp,t->words[i-3]))
							printf("Now: WAN IP Address %s\n",t->words[i-1]);
						else {
							if (!apmib_get(MIB_WAN_DHCP, (void *)&intVal)) {
								printf("\tGet WAN_DHCP MIB Error\n");
								return(0);
							}
							if(intVal != 0) { // WAN fixed ip mode
								intVal=0;
								if (!apmib_set(MIB_WAN_DHCP, (void *)&intVal)) {
									printf("\tSet MIB_WAN_DHCP MIB Error\n");
									return(0);
								}
							}
							if (!apmib_set(MIB_WAN_IP_ADDR, (void *)&eIp)) {
								printf("\tSet WAN IP Addr MIB Error\n");
								return(0);
							}
							sprintf(buff,"ifconfig %s %s mtu %d",WAN_I,t->words[i-3],atoi(t->words[i-1]));
							system(buff);
							apmib_update(CURRENT_SETTING);
							restart_daemon(_WEBS_PROG_NAME);
							printf("WAN IP Address Change to %s with mtu %d !!!\n", t->words[i-3],atoi(t->words[i-1]));
						}
					}
					else if(l == 2){ // LAN interface
						if (!apmib_get(MIB_IP_ADDR, (void *)&tIp)) {
							printf("\tGet LAN IP Addr MIB Error\n");
							return(0);
						}
						gIp=inet_ntoa(tIp);
						if(!strcmp(gIp,t->words[i-3]))
							printf("Now: LAN IP Address %s\n",t->words[i-1]);
						else {
							if (!apmib_set(MIB_IP_ADDR, (void *)&eIp)) {
								printf("\tSet LAN IP Addr MIB Error\n");
								return(0);
							}
							apmib_update(CURRENT_SETTING);
							restart_daemon(_WEBS_PROG_NAME);
							printf("LAN IP Address Change to %s with mtu %d !!!\n", t->words[i-3],atoi(t->words[i-1]));
							printf("Please Re-login again !!!\n");
							sprintf(buff,"ifconfig %s %s mtu %d",ELAN_I,t->words[i-3],atoi(t->words[i-1]));
							system(buff);
						}
					}
				}
				else
					ifconfig_format();
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 6: // nat
			if(!strcmp(t->words[i-6],"nat")) {
				if(!strcmp(t->words[i-5],"server")) {
					if(!strcmp(t->words[i-4],"dis") || !strcmp(t->words[i-4],"disp"))
						nat_server_disp();
					else if(!strcmp(t->words[i-4],"add")) {
						l=atoi(t->words[i-2]);
						if(l)
							printf("Too many inputs !!!\n");
						nat_server_add_format();
					}
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-5],"service")) {
					if(!strcmp(t->words[i-4],"sap1") || !strcmp(t->words[i-4],"sap2") || !strcmp(t->words[i-4],"sap3") || !strcmp(t->words[i-4],"sap4") || !strcmp(t->words[i-4],"sap5") || !strcmp(t->words[i-4],"sap6") || !strcmp(t->words[i-4],"sap7") || !strcmp(t->words[i-4],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 7: // nat
			if(!strcmp(t->words[i-7],"nat")) {
				if(!strcmp(t->words[i-6],"server")) {
					if(!strcmp(t->words[i-5],"dis") || !strcmp(t->words[i-5],"disp"))
						nat_server_disp();
					else if(!strcmp(t->words[i-5],"add")) {
						if (!inet_aton(t->words[i-4], &eIp))  {
							printf("Invalid IP address input !!!\n");
							return(0);
						}
						l=atoi(t->words[i-3]);
						if(l) {
							printf("Too many inputs !!!\n");
							nat_server_add_format();
						}
						else
							nat_server_add(t,l,0); //customize server, but not input the last argument [to port]
					}
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-6],"service")) {
					if(!strcmp(t->words[i-5],"sap1") || !strcmp(t->words[i-5],"sap2") || !strcmp(t->words[i-5],"sap3") || !strcmp(t->words[i-5],"sap4") || !strcmp(t->words[i-5],"sap5") || !strcmp(t->words[i-5],"sap6") || !strcmp(t->words[i-5],"sap7") || !strcmp(t->words[i-5],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 8: // nat
			if(!strcmp(t->words[i-8],"nat")) {
				if(!strcmp(t->words[i-7],"server")) {
					if(!strcmp(t->words[i-6],"dis") || !strcmp(t->words[i-6],"disp"))
						nat_server_disp();
					else if(!strcmp(t->words[i-6],"add")) {
						if (!inet_aton(t->words[i-5], &eIp))  {
							printf("Invalid IP address input !!!\n");
							return(0);
						}
						l=atoi(t->words[i-4]);
						if(l) {
							printf("Too many inputs !!!\n");
							nat_server_add_format();
						}
						else
							nat_server_add(t,l,1); //customize server
					}
					else
						invalid_subcmds(IP_NAT_SERVER_CMDS);
				}
				else if(!strcmp(t->words[i-7],"service")) {
					if(!strcmp(t->words[i-6],"sap1") || !strcmp(t->words[i-6],"sap2") || !strcmp(t->words[i-6],"sap3") || !strcmp(t->words[i-6],"sap4") || !strcmp(t->words[i-6],"sap5") || !strcmp(t->words[i-6],"sap6") || !strcmp(t->words[i-6],"sap7") || !strcmp(t->words[i-6],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 9: // nat
			if(!strcmp(t->words[i-9],"nat")) {
				if(!strcmp(t->words[i-8],"service")) {
					if(!strcmp(t->words[i-7],"sap1") || !strcmp(t->words[i-7],"sap2") || !strcmp(t->words[i-7],"sap3") || !strcmp(t->words[i-7],"sap4") || !strcmp(t->words[i-7],"sap5") || !strcmp(t->words[i-7],"sap6") || !strcmp(t->words[i-7],"sap7") || !strcmp(t->words[i-7],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 10: // nat
			if(!strcmp(t->words[i-10],"nat")) {
				if(!strcmp(t->words[i-9],"service")) {
					if(!strcmp(t->words[i-8],"sap1") || !strcmp(t->words[i-8],"sap2") || !strcmp(t->words[i-8],"sap3") || !strcmp(t->words[i-8],"sap4") || !strcmp(t->words[i-8],"sap5") || !strcmp(t->words[i-8],"sap6") || !strcmp(t->words[i-8],"sap7") || !strcmp(t->words[i-8],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		case 11: // nat
			if(!strcmp(t->words[i-11],"nat")) {
				if(!strcmp(t->words[i-10],"service")) {
					if(!strcmp(t->words[i-9],"sap1") || !strcmp(t->words[i-9],"sap2") || !strcmp(t->words[i-9],"sap3") || !strcmp(t->words[i-9],"sap4") || !strcmp(t->words[i-9],"sap5") || !strcmp(t->words[i-9],"sap6") || !strcmp(t->words[i-9],"sap7") || !strcmp(t->words[i-9],"sap8"))
						sap_control(t,i);
					else
						invalid_subcmds(IP_NAT_SERVICE_CMDS);
				}
				else
					invalid_subcmds(IP_NAT_CMDS);
			}
			else
				invalid_subcmds(IP_CMDS);
			break;
		default:
			invalid_subcmds(IP_CMDS);
			break;
	}
		
	return(1);
}

static int
dowlan(t)
struct op *t;
{
	char *buff;
	register int i;
	int k,kk,intVal,ich,creg,l;
	register char *cp;
	WLAN_STA_INFO_Tp pInfo;

	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;
	
	if(i>1) {
		if(!strcmp(t->words[0],"?")) {
			printf("Too many inputs !!!\n");
			return(1);
		}
	}
	
	switch(i) {
		case 0:
			printf("Insufficient arguments !!!\n");
			return(1);
			break;
		case 1: //active, association, chid, essid, version, band, mode
			if(!strcmp(t->words[i-1],"?"))
				invalid_subcmds(WLAN_CMDS);
			/*else if(!strcmp(t->words[i-1],"re") || !strcmp(t->words[i-1],"reload")) {
				printf("\n\tplease wait some minutes...\n\n");
				run_script();
			}*/
			else if(!strcmp(t->words[i-1],"act") || !strcmp(t->words[i-1],"active"))
				printf("format: wlan active [on|off]\n");
			else if(!strcmp(t->words[i-1],"ass") || !strcmp(t->words[i-1],"assoc") || !strcmp(t->words[i-1],"association")) {

				cp = calloc(1, sizeof(WLAN_STA_INFO_T) * (MAX_STA_NUM+1));

				if ( cp == 0 ) {
					printf("Allocate buffer failed!\n");
					return 0;
				}
				if ( getWlStaInfo(WLAN_I,  (WLAN_STA_INFO_Tp)cp ) < 0 ) {
					printf("Read wlan sta info failed!\n");
					return(0);
				}
				printf("===================================================================\n");
				printf("MAC Address\tTx Packet\tRx Packet\tTx Rate (Mbps)\tPower Saving\n");
				for (k=1; k<=MAX_STA_NUM; k++) {
					pInfo = (WLAN_STA_INFO_Tp)&cp[k*sizeof(WLAN_STA_INFO_T)];
					if (pInfo->aid && (pInfo->flag & STA_INFO_FLAG_ASOC)) {
						printf("%02x:%02x:%02x:%02x:%02x:%02x\t%d\t%d\t%d%s\t%s\t%d\n",
							pInfo->addr[0],pInfo->addr[1],pInfo->addr[2],pInfo->addr[3],pInfo->addr[4],pInfo->addr[5],
							pInfo->tx_packets, pInfo->rx_packets, pInfo->txOperaRates/2,
							((pInfo->txOperaRates%2) ? ".5" : "" ), ((pInfo->flag & STA_INFO_FLAG_ASLEEP) ? "yes" : "no"));
					}
				}
				printf("===================================================================\n");
				free(cp);
			}

			else if(!strcmp(t->words[i-1],"ch") || !strcmp(t->words[i-1],"chid"))
				printf("format: wlan chid [channel id|auto]\n");
			else if(!strcmp(t->words[i-1],"ess") || !strcmp(t->words[i-1],"essid"))
				printf("format: wlan essid [ess id]\n");
			else if(!strcmp(t->words[i-1],"ver")  || !strcmp(t->words[i-1],"version")) {
				if (!apmib_get(MIB_WLAN_BAND, (void *)&intVal)) {
					printf("\tGet WLAN Band MIB Error\n");
					return(0);
				}
				switch(intVal) {
					case 1: // 11b
						printf("Now: 802.11b\n");
						break;
					case 2: // 11g
						printf("Now: 802.11g\n");
						break;
					case 3: // 11b+g
						printf("Now: 802.11b+g\n");
						break;
					case 4: // 11a
						printf("Now: 802.11a\n");
						break;
				}
			}
			else if(!strcmp(t->words[i-1],"surv") || !strcmp(t->words[i-1],"survey"))
				wlan_survey();
			else if(!strcmp(t->words[i-1],"ban") || !strcmp(t->words[i-1],"band"))
				printf("format: wlan band [1:b|g|bg]\n");
			else if(!strcmp(t->words[i-1],"mod") || !strcmp(t->words[i-1],"mode"))
				printf("format: wlan mode [ap|client <infrastructure|adhoc>|bridge|repeater]\n");
			else if(!strcmp(t->words[i-1],"aut") || !strcmp(t->words[i-1],"auth"))
				printf("format: wlan auth [open|share|auto]\n");
			else if(!strcmp(t->words[i-1],"pream") || !strcmp(t->words[i-1],"preamble"))
				printf("format: wlan preamble [long|short]\n");
			else if(!strcmp(t->words[i-1],"bss") || !strcmp(t->words[i-1],"bssid"))
				printf("format: wlan bssid [on|off]\n");
			else if(!strcmp(t->words[i-1],"iap") || !strcmp(t->words[i-1],"iapp"))
				printf("format: wlan iapp [on|off]\n");
			else if(!strcmp(t->words[i-1],"acl"))
				invalid_subcmds(WLAN_ACL_CMDS);
			else if(!strcmp(t->words[i-1],"sec") || !strcmp(t->words[i-1],"security"))
				invalid_subcmds(WLAN_SEC_CMDS);
			else if(!strcmp(t->words[i-1],"wds"))
				invalid_subcmds(WLAN_WDS_CMDS);
			else
				invalid_subcmds(WLAN_CMDS);
			break;
		case 2: // active, chid, essid, band, mode, survey, security, acl
			if(!strcmp(t->words[i-2],"act") || !strcmp(t->words[i-2],"active")) {
				if (!apmib_get(MIB_WLAN_DISABLED, (void *)&intVal)) {
					printf("\tGet WLAN Active MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"?")) {
					printf("format: wlan active [on|off]\n");
					if(!intVal)
						printf("Now: active on\n");
					else
						printf("Now: active off\n");
					return(0);
				}
				else if(!strcmp(t->words[i-1],"on")) {
					if(!intVal) {
						printf("Now: active on\n");
						return(0);
					}
					else
						intVal=0;	//enable wlan
				}
				else if(!strcmp(t->words[i-1],"off")) {
					if(intVal) {
						printf("Now: active off\n");
						return(0);
					}
					else
						intVal=1;	//disable wlan
				}
				else {
					printf("format: wlan active [on|off]\n");
					if(!intVal)
						printf("Now: active on\n");
					else
						printf("Now: active off\n");
					return(0);
				}
				
				if (!apmib_set(MIB_WLAN_DISABLED, (void *)&intVal)) {
					printf("\tSet WLAN Active MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"ch") || !strcmp(t->words[i-2],"chid")) {
				if (!apmib_get(MIB_WLAN_CHAN_NUM, (void *)&intVal)) {
					printf("\tGet WLAN Channel MIB Error\n");
					return(0);
				}
				if (!apmib_get(MIB_WLAN_REG_DOMAIN, (void *)&creg)) {
					printf("\tGet WLAN RegDomain MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"?")) {
					printf("format: wlan chid [channel_id|auto]\n");
					if(!intVal)
						printf("Now: channel auto\n");
					else
						printf("Now: channel %d\n",intVal);
					return(0);
				}
				else if(!strcmp(t->words[i-1],"auto")) {
					if(!intVal) {
						printf("Now: channel auto\n");
						return(0);
					}
					else
						intVal=0;	// 0 means channel auto
				}
				else  {
					ich=atoi(t->words[i-1]);
					switch(creg) {
						case 1:
						case 2:
							if(!(ich>=1 && ich<=11)) {
								printf("format: wlan chid [1-11|auto]\n");
								printf("Now: channel %d\n",intVal);
								return;
							}
							break;
						case 3:
						case 5:
							if(!(ich>=1 && ich<=13)) {
								printf("format: wlan chid [1-13|auto]\n");
								printf("Now: channel %d\n",intVal);
								return;
							}
							break;
						case 4:
							if(!(ich>=5 && ich<=7)) {
								printf("format: wlan chid [5-7|auto]\n");
								printf("Now: channel %d\n",intVal);
								return;
							}
							break;
						case 6:
							if(!(ich>=1 && ich<=14)) {
								printf("format: wlan chid [1-14|auto]\n");
								printf("Now: channel %d\n",intVal);
								return;
							}
							break;
						default:
							break;
					}
					if(intVal == ich) {
						printf("Now: channel %d\n",intVal);
						return(0);
					}
					intVal=ich;
				}
				if(!apmib_set(MIB_WLAN_CHAN_NUM, (void *)&intVal)) {
					printf("\tSet WLAN Channel MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"ess") || !strcmp(t->words[i-2],"essid")) {
				buff=(char *)malloc(BUFFSIZE);
				if (!apmib_get(MIB_WLAN_SSID, (void *)buff)) {
					printf("\tGet WLAN SSID MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"?")) {
					printf("format: wlan essid [ess id]\n");
					printf("Now: %s\n",buff);
					free(buff);
				}
				else {
					if(!apmib_set(MIB_WLAN_SSID, (void *)t->words[i-1])) {
						printf("\tSet WLAN SSID MIB Error\n");
						return(0);
					}
					//apmib_update(CURRENT_SETTING);
					//printf("\n\tplease wait some minutes...\n\n");
					//run_script();
				}
			}
			else if(!strcmp(t->words[i-2],"ban") || !strcmp(t->words[i-2],"band")) {
				if (!apmib_get(MIB_WLAN_BAND, (void *)&k)) {
					printf("\tGet WLAN BAND MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"b"))
					intVal=1;
				else if(!strcmp(t->words[i-1],"g"))
					intVal=2;
				else if(!strcmp(t->words[i-1],"bg"))
					intVal=3;
				else {
					printf("format: wlan band [b|g|bg]\n");
					switch(k) {
						case 1:
							printf("Now: 802.11b\n");
							break;
						case 2:
							printf("Now: 802.11g\n");
							break;
						case 3:
							printf("Now: 802.11b+g\n");
							break;
						default:
							printf("Now: unknow band type\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_BAND, (void *)&intVal)) {
					printf("\tSet WLAN BAND MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"mod") || !strcmp(t->words[i-2],"mode")) {
				if (!apmib_get( MIB_WLAN_MODE, (void *)&k)) {
					printf("\tGet WLAN MODE MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"ap"))
					intVal=0;
				else if(!strcmp(t->words[i-1],"cli") || !strcmp(t->words[i-1],"client")) {
					printf("Insufficient arguments !!!\n");
					printf("format: wlan mode client <infrastructure|adhoc>\n");
					goto now_wlan_mode;
				}
				else if(!strcmp(t->words[i-1],"brid") || !strcmp(t->words[i-1],"bridge"))
					intVal=2;
				else if(!strcmp(t->words[i-1],"rep") || !strcmp(t->words[i-1],"repeater"))
					intVal=3;
				else {
					printf("format: wlan mode [ap|client <infrastructure|adhoc>|bridge|repeater]\n");
now_wlan_mode:
					switch(k) {
						case 0:
							printf("Now: AP\n");
							break;
						case 1:
							printf("Now: Client\n");
							break;
						case 2:
							printf("Now: Bridge\n");
							break;
						case 3:
							printf("Now: repeater\n");
							break;
						default:
							printf("Now: unknow network mode\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_MODE, (void *)&intVal)) {
					printf("\tSet WLAN MODE MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"aut") || !strcmp(t->words[i-2],"auth")) {
				if (!apmib_get( MIB_WLAN_AUTH_TYPE, (void *)&k)) {
					printf("\tGet WLAN AUTH_TYPE MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"op") || !strcmp(t->words[i-1],"open"))
					intVal=0;
				else if(!strcmp(t->words[i-1],"sh") || !strcmp(t->words[i-1],"share"))
					intVal=1;
				else if(!strcmp(t->words[i-1],"au") || !strcmp(t->words[i-1],"auto"))
					intVal=2;
				else {
					printf("format: wlan auth [open|share|auto]\n");
					switch(k) {
						case 0:
							printf("Now: Open System\n");
							break;
						case 1:
							printf("Now: Shared Key\n");
							break;
						case 2:
							printf("Now: Auto\n");
							break;
						default:
							printf("Now: unknow auth type\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_AUTH_TYPE, (void *)&intVal)) {
					printf("\tSet WLAN AUTH_TYPE MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"pream") || !strcmp(t->words[i-2],"preamble")) {
				if (!apmib_get( MIB_WLAN_PREAMBLE_TYPE, (void *)&k)) {
					printf("\tGet WLAN PREAMBLE_TYPE MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"lo") || !strcmp(t->words[i-1],"long"))
					intVal=0;
				else if(!strcmp(t->words[i-1],"sho") || !strcmp(t->words[i-1],"short"))
					intVal=1;
				else {
					printf("format: wlan preamble [long|short]\n");
					switch(k) {
						case 0:
							printf("Now: Long\n");
							break;
						case 1:
							printf("Now: Short\n");
							break;
						default:
							printf("Now: unknow preamble type\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_PREAMBLE_TYPE, (void *)&intVal)) {
					printf("\tSet WLAN PREAMBLE_TYPE MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"bss") || !strcmp(t->words[i-2],"bssid")) {
				if (!apmib_get( MIB_WLAN_HIDDEN_SSID, (void *)&k)) {
					printf("\tGet WLAN HIDDEN_SSID MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"on"))
					intVal=0;
				else if(!strcmp(t->words[i-1],"off"))
					intVal=1;
				else {
					printf("format: wlan bssid [on|off]\n");
					switch(k) {
						case 0:
							printf("Now: Broadcast SSID enabled\n");
							break;
						case 1:
							printf("Now: Broadcast SSID disabled\n");
							break;
						default:
							printf("Now: unknow value\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_HIDDEN_SSID, (void *)&intVal)) {
					printf("\tSet WLAN HIDDEN_SSID MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"iap") || !strcmp(t->words[i-2],"iapp")) {
				if (!apmib_get( MIB_WLAN_IAPP_DISABLED, (void *)&k)) {
					printf("\tGet WLAN IAPP_DISABLED MIB Error\n");
					return(0);
				}
				if(!strcmp(t->words[i-1],"on"))
					intVal=0;
				else if(!strcmp(t->words[i-1],"off"))
					intVal=1;
				else {
					printf("format: wlan iapp [on|off]\n");
					switch(k) {
						case 0:
							printf("Now: IAPP enabled\n");
							break;
						case 1:
							printf("Now: IAPP disabled\n");
							break;
						default:
							printf("Now: unknow value\n");
							break;
					}
					return(0);
				}
				if(!apmib_set(MIB_WLAN_IAPP_DISABLED, (void *)&intVal)) {
					printf("\tSet WLAN IAPP_DISABLED MIB Error\n");
					return(0);
				}
				//apmib_update(CURRENT_SETTING);
				//printf("\n\tplease wait some minutes...\n\n");
				//run_script();
			}
			else if(!strcmp(t->words[i-2],"surv") || !strcmp(t->words[i-2],"survey")) {
				if(!strcmp(t->words[i-1],"con") || !strcmp(t->words[i-1],"connect")) {
					if (!apmib_get( MIB_WLAN_MODE, (void *)&k)) {
						printf("\tGet WLAN MODE MIB Error\n");
						return(0);
					}
					if(k != 1) {
						printf("Error, this operation only invoked under client mode. Please change wlan mode first !!!\n");	
						switch(k) {
							case 0:
								printf("Now: AP\n");
								break;
							case 2:
								printf("Now: Bridge\n");
								break;
							case 3:
								printf("Now: repeater\n");
								break;
							default:
								printf("Now: unknow network mode\n");
								break;
						}
						return(0);
					}
					else {
						printf("Insufficient arguments !!!\n");
						printf("format: wlan survey connect [id]\n");
					}
				}
				else
					invalid_subcmds(WLAN_CMDS);
			}
			else if(!strcmp(t->words[i-2],"acl")) {
				if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp")) {
					acl_disp();
				}
				else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall")) {
					acl_clearall();
				}
				else if(!strcmp(t->words[i-1],"add")) {
					printf("Insufficient arguments !!!\n");
					printf("format: wlan acl add [mac_addr]\n");
				}
				else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete")) {
					printf("Insufficient arguments !!!\n");
					printf("format: wlan acl delete [id]\n");
				}
				else
					invalid_subcmds(WLAN_ACL_CMDS);
			}
			else if(!strcmp(t->words[i-2],"sec") || !strcmp(t->words[i-2],"security")) {
				if(!strcmp(t->words[i-1],"enc") || !strcmp(t->words[i-1],"encrypt"))
					invalid_subcmds(WLAN_SEC_ENC_CMDS);
				else if(!strcmp(t->words[i-1],"1x"))
					printf("format: wlan 8021x [off|on <port> <ip_addr> <password>] <tkip|aes|mixed>\n");
				else if(!strcmp(t->words[i-1],"pre") || !strcmp(t->words[i-1],"preauth"))
					printf("format: wlan preauth [off|on]\n");
				else
					invalid_subcmds(WLAN_SEC_CMDS);
			}
			else if(!strcmp(t->words[i-2],"wds")) {
				if(!strcmp(t->words[i-1],"enc") || !strcmp(t->words[i-1],"encrypt"))
					invalid_subcmds(WLAN_WDS_ENC_CMDS);
				else if(!strcmp(t->words[i-1],"pee") || !strcmp(t->words[i-1],"peer"))
					invalid_subcmds(WLAN_WDS_PEER_CMDS);
				else
					invalid_subcmds(WLAN_WDS_CMDS);
			}
			else
				invalid_subcmds(WLAN_CMDS);
			break;
		case 3: // mode, survey, acl, security
			if(!strcmp(t->words[i-3],"mod") || !strcmp(t->words[i-3],"mode")) {
				if(!strcmp(t->words[i-2],"cl") || !strcmp(t->words[i-2],"client")) {
					int c=1;
					if (!apmib_get( MIB_WLAN_MODE, (void *)&k)) {
						printf("\tGet WLAN MODE MIB Error\n");
						return(0);
					}
					if (!apmib_get( MIB_WLAN_NETWORK_TYPE, (void *)&kk)) {
						printf("\tGet WLAN NETWORK_TYPE MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-1],"infra") || !strcmp(t->words[i-1],"infrastructure"))
						intVal=0;
					else if(!strcmp(t->words[i-1],"adh") || !strcmp(t->words[i-1],"adhoc"))
						intVal=1;
					else {
						printf("format: wlan mode client <infrastructure|adhoc>\n");
						switch(k) {
							case 0:
								printf("Now: AP\n");
								break;
							case 1:
								printf("Now: Client");
								switch(kk) {
									case 0:
										printf("\tInfrastructure\n");
										break;
									case 1:
										printf("\tAd hoc\n");
										break;
									default:
										printf("\tunknow client type\n");
										break;
								}
								break;
							case 2:
								printf("Now: Bridge\n");
								break;
							case 3:
								printf("Now: repeater\n");
								break;
							default:
								printf("Now: unknow network type\n");
								break;
						}
						return(0);
					}
					if(!apmib_set(MIB_WLAN_MODE, (void *)&c)) {
						printf("\tSet WLAN MODE MIB Error\n");
						return(0);
					}
					if(!apmib_set(MIB_WLAN_NETWORK_TYPE, (void *)&intVal)) {
						printf("\tSet WLAN MODE MIB Error\n");
						return(0);
					}
					//apmib_update(CURRENT_SETTING);
					//printf("\n\tplease wait some minutes...\n\n");
					//run_script();
				}
				else
					invalid_subcmds(WLAN_CMDS);
			}
			else if(!strcmp(t->words[i-3],"surv") || !strcmp(t->words[i-3],"survey")) {
				if(!strcmp(t->words[i-2],"con") || !strcmp(t->words[i-2],"connect")) {
					if (!apmib_get( MIB_WLAN_MODE, (void *)&k)) {
						printf("\tGet WLAN MODE MIB Error\n");
						return(0);
					}
					if(k != 1) {
						printf("Error, this operation only invoked under client mode. Please change wlan mode first !!!\n");	
						switch(k) {
							case 0:
								printf("Now: AP\n");
								break;
							case 2:
								printf("Now: Bridge\n");
								break;
							case 3:
								printf("Now: repeater\n");
								break;
							default:
								printf("Now: unknow network mode\n");
								break;
						}
						return(0);
					}
					else {
						intVal=atoi(t->words[i-1]);
						wlan_survey_connect(intVal);
					}
				}
				else
					invalid_subcmds(WLAN_CMDS);
			}
			else if(!strcmp(t->words[i-3],"acl")) {
				if(!strcmp(t->words[i-2],"dis") || !strcmp(t->words[i-2],"disp")) {
					acl_disp();
				}
				else if(!strcmp(t->words[i-2],"cla") || !strcmp(t->words[i-2],"clearall")) {
					acl_clearall();
				}
				else if(!strcmp(t->words[i-2],"ad") || !strcmp(t->words[i-2],"add"))
					acl_add(t->words[i-1]);
				else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
					l=atoi(t->words[i-1]);
					if(!l) {
						printf("No Such %d acl !!!\n",l);
						return(0);
					}
					if(!apmib_get(MIB_WLAN_AC_NUM,(void *)&intVal)) {
						printf("\tGet WLAN_AC_NUM MIB Error\n");
						return(0);
					}
					if(l > intVal)
						printf("No Such %d acl !!!\n",l);
					else
						acl_del(l);
				}
				else
					invalid_subcmds(WLAN_ACL_CMDS);
			}
			else if(!strcmp(t->words[i-3],"sec") || !strcmp(t->words[i-3],"security")) {
				if(!strcmp(t->words[i-2],"enc") || !strcmp(t->words[i-2],"encrypt")) {
					if (!apmib_get( MIB_WLAN_ENCRYPT, (void *)&k)) {
						printf("\tGet WLAN ENCRYPT MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-1],"off")) {
						if(k) {
							intVal=0;
							if (!apmib_set( MIB_WLAN_ENCRYPT, (void *)&intVal)) {
								printf("\tSet WLAN ENCRYPT MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: encrypt off\n");
					}
					else
						invalid_subcmds(WLAN_SEC_ENC_CMDS);
				}
				else if(!strcmp(t->words[i-2],"1x")) {
					if (!apmib_get( MIB_WLAN_ENABLE_1X, (void *)&k)) {
						printf("\tGet WLAN ENABLE_1X MIB Error\n");
						return(0);
					}
					if (!apmib_get( MIB_WLAN_WPA_AUTH, (void *)&kk)) {
						printf("\tGet WLAN WPA_AUTH MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-1],"off")) {
						if(k) {
							intVal=0; // disable 802.1x
							if (!apmib_set( MIB_WLAN_ENABLE_1X, (void *)&intVal)) {
								printf("\tSet WLAN ENABLE_1X MIB Error\n");
								return(0);
							}
							if(kk == 1) {
								intVal=2; // change wpa-auth to radius
								if (!apmib_set(MIB_WLAN_WPA_AUTH, (void *)&intVal)) { // wpa auth type
									printf("\tSet WLAN_WPA_AUTH MIB Error\n");
									return(0);
								}
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: 802.1x off\n");
					}
					else
						printf("format: wlan security 1x [off|on <port> <ip_addr> <password> [wep <64|128>|wpa <tkip|aes>|wpa2 <mixed|aes>]]\n");
				}
				else if(!strcmp(t->words[i-2],"pre") || !strcmp(t->words[i-2],"preauth")) {
					/*if (!apmib_get( MIB_WLAN_ENCRYPT, (void *)&intVal)) {
						printf("\tGet WLAN ENCRYPT MIB Error\n");
						return(0);
					}
					if(intVal != 4 || intVal != 6) {
						printf("Error, pre-auth can't be used in non-WPA2 mode !!!\n");
						return(0);
					}*/
					if (!apmib_get( MIB_WLAN_WPA2_PRE_AUTH, (void *)&kk)) {
						printf("\tGet WLAN WPA2_PRE_AUTH MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-1],"off")) {
						if(kk) {
							intVal=0;
							if (!apmib_set( MIB_WLAN_WPA2_PRE_AUTH, (void *)&intVal)) {
								printf("\tSet WLAN WPA2_PRE_AUTH MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: pre-auth off\n");
					}
					else if(!strcmp(t->words[i-1],"on")) {
						if(!kk) {
							intVal=1;
							if (!apmib_set( MIB_WLAN_WPA2_PRE_AUTH, (void *)&intVal)) {
								printf("\tSet WLAN WPA2_PRE_AUTH MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: pre-auth on\n");
					}
					else {
						printf("format: wlan preauth [off|on]\n");
						if(kk)
							printf("Now: pre-auth on\n");
						else
							printf("Now: pre-auth off\n");
					}
				}
				else
					invalid_subcmds(WLAN_SEC_CMDS);
			}
			else if(!strcmp(t->words[i-3],"wds")) {
				if(!strcmp(t->words[i-2],"enc") || !strcmp(t->words[i-2],"encrypt")) {
					if (!apmib_get( MIB_WLAN_WDS_ENCRYPT, (void *)&k)) {
						printf("\tGet WLAN WDS ENCRYPT MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-1],"off")) {
						if(k) {
							intVal=0;
							if (!apmib_set( MIB_WLAN_WDS_ENCRYPT, (void *)&intVal)) {
								printf("\tSet WLAN WDS ENCRYPT MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: wds encrypt off\n");
					}
					else
						invalid_subcmds(WLAN_WDS_ENC_CMDS);
				}
				else if(!strcmp(t->words[i-2],"pee") || !strcmp(t->words[i-2],"peer")) {
					if(!strcmp(t->words[i-1],"dis") || !strcmp(t->words[i-1],"disp")) {
						wds_peer_disp();
					}
					else if(!strcmp(t->words[i-1],"cla") || !strcmp(t->words[i-1],"clearall")) {
						wds_peer_clearall();
					}
					else if(!strcmp(t->words[i-1],"add")) {
						printf("Insufficient arguments !!!\n");
						printf("format: wlan wds peer add [mac_addr]\n");
					}
					else if(!strcmp(t->words[i-1],"del") || !strcmp(t->words[i-1],"delete")) {
						printf("Insufficient arguments !!!\n");
						printf("format: wlan wds peer delete [id]\n");
					}
					else
						invalid_subcmds(WLAN_WDS_PEER_CMDS);
				}
				else
					invalid_subcmds(WLAN_WDS_CMDS);
			}
			else
				invalid_subcmds(WLAN_CMDS);
			break;
		case 4:
			if(!strcmp(t->words[i-4],"sec") || !strcmp(t->words[i-4],"security")) {
				if(!strcmp(t->words[i-3],"enc") || !strcmp(t->words[i-3],"encrypt")) {
					if (!apmib_get( MIB_WLAN_ENCRYPT, (void *)&k)) {
						printf("\tGet WLAN ENCRYPT MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-2],"off")) {
						printf("Too many inputs !!!\n");
						invalid_subcmds(WLAN_SEC_ENC_CMDS);
					}
					else if(!strcmp(t->words[i-2],"wep")) {
						printf("format: wlan security encrypt wep [64|128] <ascii|hex> <1:key1|2:key2|3:key3|4:key4> <key>\n");
					}
					else if(!strcmp(t->words[i-2],"wpa")) {
						printf("format: wlan security encrypt wpa [tkip|aes] <pass|hex> <key> [gklt]\n");
					}
					else if(!strcmp(t->words[i-2],"wpa2")) {
						printf("format: wlan security encrypt wpa2 [mixed|aes] <pass|hex> <key> [gklt]\n");
					}
					else
						invalid_subcmds(WLAN_SEC_ENC_CMDS);
				}
				else if(!strcmp(t->words[i-3],"1x")) {
					if (!apmib_get( MIB_WLAN_ENABLE_1X, (void *)&k)) {
						printf("\tGet WLAN ENABLE_1X MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-2],"on"))
						printf("format: wlan security 1x on <port> <ip_addr> <password> [wep <64|128>|wpa <tkip|aes>|wpa2 <mixed|aes>]\n");
					else
						printf("format: wlan security 1x [off|on <port> <ip_addr> <password> [wep <64|128>|wpa <tkip|aes>|wpa2 <mixed|aes>]]\n");
				}
			}
			else if(!strcmp(t->words[i-4],"wds")) {
				if(!strcmp(t->words[i-3],"enc") || !strcmp(t->words[i-3],"encrypt")) {
					if (!apmib_get( MIB_WLAN_WDS_ENCRYPT, (void *)&k)) {
						printf("\tGet WLAN WDS ENCRYPT MIB Error\n");
						return(0);
					}
					if(!strcmp(t->words[i-2],"off")) {
						if(k) {
							intVal=0;
							if (!apmib_set( MIB_WLAN_WDS_ENCRYPT, (void *)&intVal)) {
								printf("\tSet WLAN WDS ENCRYPT MIB Error\n");
								return(0);
							}
							//apmib_update(CURRENT_SETTING);
							//printf("\n\tplease wait some minutes...\n\n");
							//run_script();
						}
						else
							printf("Now: wds encrypt off\n");
					}
					else if(!strcmp(t->words[i-2],"wep")) {
						printf("format: wlan wds encrypty wep [64|128] <ascii|hex> <key>\n");
					}
					else if(!strcmp(t->words[i-2],"wpa")) {
						printf("format: wlan wds encrypty wpa [tkip|aes] <pass|hex> <key>\n");
					}
					else if(!strcmp(t->words[i-2],"wpa2")) {
						printf("format: wlan wds encrypty wpa2 [mixed|aes] <pass|hex> <key>\n");
					}
					else
						invalid_subcmds(WLAN_WDS_ENC_CMDS);
				}
				else if(!strcmp(t->words[i-3],"pee") || !strcmp(t->words[i-3],"peer")) {
					if(!strcmp(t->words[i-2],"dis") || !strcmp(t->words[i-2],"disp")) {
						wds_peer_disp();
					}
					else if(!strcmp(t->words[i-2],"cla") || !strcmp(t->words[i-2],"clearall")) {
						wds_peer_clearall();
					}
					else if(!strcmp(t->words[i-2],"ad") || !strcmp(t->words[i-2],"add"))
						wds_peer_add(t->words[i-1]);
					else if(!strcmp(t->words[i-2],"del") || !strcmp(t->words[i-2],"delete")) {
						l=atoi(t->words[i-1]);
						if(!l) {
							printf("No Such %d acl !!!\n",l);
							return(0);
						}
						if(!apmib_get(MIB_WLAN_WDS_NUM,(void *)&intVal)) {
							printf("\tGet WLAN_WDS_NUM MIB Error\n");
							return(0);
						}
						if(l > intVal)
							printf("No Such %d wds peer !!!\n",l);
						else
							wds_peer_del(l);
					}
					else
						invalid_subcmds(WLAN_WDS_PEER_CMDS);
				}
				else
					invalid_subcmds(WLAN_WDS_CMDS);
			}
			else
				invalid_subcmds(WLAN_CMDS);
			break;
		case 5:
		case 6:
			if(!strcmp(t->words[0],"wds")) {
				if(!strcmp(t->words[1],"enc") || !strcmp(t->words[1],"encrypt"))
					wds_encrypt_control(t,i);
				else
					invalid_subcmds(WLAN_WDS_CMDS);
				break;
			}
		case 7:
		case 8:
			if(!strcmp(t->words[0],"sec") || !strcmp(t->words[0],"security")) {
				if(!strcmp(t->words[1],"enc") || !strcmp(t->words[1],"encrypt"))
					wlan_encrypt_control(t,i);
				else if(!strcmp(t->words[1],"1x")) {
					if(!strcmp(t->words[2],"on")) 
						wlan_1x_control(t,i);
					else
						printf("format: wlan security 1x [off|on <port> <ip_addr> <password> [wep <64|128>|wpa <tkip|aes>|wpa2 <mixed|aes>]]\n");
				}
			}
			else
				invalid_subcmds(WLAN_CMDS);
			break;
		default:
			invalid_subcmds(WLAN_CMDS);
			break;
	}
		
	return 1;
}

static int dohelp()
{
	int col;
	const struct builtincmd *x;

	printf("\nValid commands are: \n");
	printf("-------------------\n");

	for (col=0, x = builtincmds; x->builtinfunc != NULL; x++) {
		if (!x->name)
			continue;
		col += printf("%s%s", ((col == 0) ? "\t" : " "), x->name);
		if (col > 60) {
			printf("\n");
			col = 0;
		}
	}
#ifdef CONFIG_FEATURE_SH_STANDALONE_SHELL
	{
		int i;
		const struct BB_applet *applet;
		extern const struct BB_applet applets[];
		extern const size_t NUM_APPLETS;

		for (i=0, applet = applets; i < NUM_APPLETS; applet++, i++) {
			if (!applet->name)
				continue;
		
			col += printf("%s%s", ((col == 0) ? "\t" : " "), 
					applet->name);
			if (col > 60) {
				printf("\n");
				col = 0;
			}
		}
	}
#endif
	printf("\n\n");
	return EXIT_SUCCESS;
}

static int
doexit(t)
struct op *t;
{
	register char *cp;

	execflg = 0;
	if ((cp = t->words[1]) != NULL)
		setstatus(getn(cp));
	leave();
	/* NOTREACHED */
	return(0);
}

static int
doexec(t)
register struct op *t;
{
	register int i;
	jmp_buf ex;
	xint *ofail;

	t->ioact = NULL;
	for(i = 0; (t->words[i]=t->words[i+1]) != NULL; i++)
		;
	if (i == 0)
		return(1);
	execflg = 1;
	ofail = failpt;
	if (setjmp(failpt = ex) == 0)
		execute(t, NOPIPE, NOPIPE, FEXEC);
	failpt = ofail;
	execflg = 0;
	return(1);
}


static int
forkexec( register struct op *t, int *pin, int *pout, int act, char **wp, int *pforked)
{
	int i, rv;
	int (*shcom)() = NULL;
	register int f;
	char *cp = NULL;
	struct ioword **iopp;
	int resetsig;
	char **owp;

	int *hpin = pin;
	int *hpout = pout;
	int hforked;
	char *hwp;
	int hinteractive;
	int hintr;
	struct brkcon * hbrklist;
	int hexecflg;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &pin;
	(void) &pout;
	(void) &wp;
	(void) &shcom;
	(void) &cp;
	(void) &resetsig;
	(void) &owp;
#endif	

	owp = wp;
	resetsig = 0;
	*pforked = 0;
	rv = -1;	/* system-detected error */
	if (t->type == TCOM) {
		while ((cp = *wp++) != NULL)
			;
		cp = *wp;

		/* strip all initial assignments */
		/* not correct wrt PATH=yyy command  etc */
#ifndef MARK_BY_EASON
		if (flag['x'])
			echo (cp ? wp: owp);
#endif
		if (cp == NULL && t->ioact == NULL) {
			while ((cp = *owp++) != NULL && assign(cp, COPYV))
				;
			return(setstatus(0));
		}
		else if (cp != NULL)
			shcom = inbuilt(cp);
	}
	t->words = wp;
	f = act;
	if (shcom == NULL && (f & FEXEC) == 0) {

		hpin = pin;
		hpout = pout;
		hforked = *pforked;
		hwp = *wp;
		hinteractive = interactive;
		hintr = intr;
		hbrklist = brklist;
		hexecflg = execflg;
	
		i = vfork();
		if (i != 0) {
                	/* who wrote this crappy non vfork safe shit? */
			pin = hpin;
			pout = hpout;
			*pforked = hforked;
			*wp = hwp;
			interactive = hinteractive;
			intr = hintr;
			brklist = hbrklist;
			execflg = hexecflg;

			*pforked = 0;
			if (i == -1)
				return(rv);
			if (pin != NULL)
				closepipe(pin);
			return(pout==NULL? setstatus(waitfor(i,0)): 0);
		}

		if (interactive) {
			signal(SIGINT, SIG_IGN);
			signal(SIGQUIT, SIG_IGN);
			resetsig = 1;
		}
		interactive = 0;
		intr = 0;
		(*pforked)++;
		brklist = 0;
		execflg = 0;
	}	
#ifndef MARK_BY_EASON
	if (owp != NULL)
		while ((cp = *owp++) != NULL && assign(cp, COPYV))
			if (shcom == NULL)
				export(lookup(cp));
#endif
#ifdef COMPIPE
	if ((pin != NULL || pout != NULL) && shcom != NULL && shcom != doexec) {
		err("piping to/from shell builtins not yet done");
		return(-1);
	}
#endif
	if (pin != NULL) {
		dup2(pin[0], 0);
		closepipe(pin);
	}
	if (pout != NULL) {
		dup2(pout[1], 1);
		closepipe(pout);
	}
	if ((iopp = t->ioact) != NULL) {
		if (shcom != NULL && shcom != doexec) {
			prs(cp);
			err(": cannot redirect shell command");
			return(-1);
		}
		while (*iopp)
			if (iosetup(*iopp++, pin!=NULL, pout!=NULL))
				return(rv);
	}
	if (shcom)
		return(setstatus((*shcom)(t)));
	/* should use FIOCEXCL */
	for (i=FDBASE; i<NOFILE; i++)
		close(i);
	if (resetsig) {
		signal(SIGINT, SIG_DFL);
		signal(SIGQUIT, SIG_DFL);
	}
	if (t->type == TPAREN)
		exit(execute(t->left, NOPIPE, NOPIPE, FEXEC));
	if (wp[0] == NULL)
		exit(0);

#ifndef MARK_BY_EASON
	cp = rexecve(wp[0], wp, makenv());
	prs(wp[0]); prs(": "); warn(cp);
#else
	dohelp();
#endif

	if (!execflg)
		trap[0] = NULL;
	leave();
	/* NOTREACHED */
	exit(1);
}

#ifndef MARK_BY_EASON
static char **
makenv()

{
	register struct wdblock *wb;
	register struct var *vp;

	wb = NULL;
	for (vp = vlist; vp; vp = vp->next)
		if (vp->status & EXPORT)
			wb = addword(vp->name, wb);
	wb = addword((char *)0, wb);
	return(getwords(wb));
}

static void
export(vp)
struct var *vp;
{
	vp->status |= EXPORT;
}

static void
echo(wp)
register char **wp;
{
	register int i;

	prs("+");
	for (i=0; wp[i]; i++) {
		if (i)
			prs(" ");
		prs(wp[i]);
	}
	prs("\n");
}

/*
 * PATH-searching interface to execve.
 * If getenv("PATH") were kept up-to-date,
 * execvp might be used.
 */
static char *
rexecve(c, v, envp)
char *c, **v, **envp;
{
	register int i;
	register char *sp, *tp;
	int eacces = 0, asis = 0;

#ifdef CONFIG_FEATURE_SH_STANDALONE_SHELL
	char *name = c;
#ifdef CONFIG_FEATURE_SH_APPLETS_ALWAYS_WIN
	name = get_last_path_component(name);
#endif
	optind = 1;
	if (find_applet_by_name(name)) {
		/* We have to exec here since we vforked.  Running 
		 * run_applet_by_name() won't work and bad things
		 * will happen. */
		execve("/proc/self/exe", v, envp);
		execve("busybox", v, envp);
	}
#endif

	sp = any('/', c)? "": path->value;
	asis = *sp == '\0';
	while (asis || *sp != '\0') {
		asis = 0;
		tp = e.linep;
		for (; *sp != '\0'; tp++)
			if ((*tp = *sp++) == ':') {
				asis = *sp == '\0';
				break;
			}
		if (tp != e.linep)
			*tp++ = '/';
		for (i = 0; (*tp++ = c[i++]) != '\0';)
			;

		execve(e.linep, v, envp);
		switch (errno) {
		case ENOEXEC:
			*v = e.linep;
			tp = *--v;
			*v = e.linep;
			execve("/bin/sh", v, envp);
			*v = tp;
			return("no Shell");

		case ENOMEM:
			return("program too big");

		case E2BIG:
			return("argument list too long");

		case EACCES:
			eacces++;
			break;
		}
	}
	return(errno==ENOENT ? "not found" : "cannot execute");
}


static int
dolabel()
{
	return(0);
}

static int
dochdir(t)
register struct op *t;
{
	register char *cp, *er;

	if ((cp = t->words[1]) == NULL && (cp = homedir->value) == NULL)
		er = ": no home directory";
	else if(chdir(cp) < 0)
		er = ": bad directory";
	else
		return(0);
	prs(cp != NULL? cp: "cd");
	err(er);
	return(1);
}

static int
doshift(t)
register struct op *t;
{
	register int n;

	n = t->words[1]? getn(t->words[1]): 1;
	if(dolc < n) {
		err("nothing to shift");
		return(1);
	}
	dolv[n] = dolv[0];
	dolv += n;
	dolc -= n;
	setval(lookup("#"), putn(dolc));
	return(0);
}

/*
 * execute login and newgrp directly
 */
static int
dologin(t)
struct op *t;
{
	register char *cp;

	if (interactive) {
		signal(SIGINT, SIG_DFL);
		signal(SIGQUIT, SIG_DFL);
	}
	cp = rexecve(t->words[0], t->words, makenv());
	prs(t->words[0]); prs(": "); err(cp);
	return(1);
}

static int
doumask(t)
register struct op *t;
{
	register int i, n;
	register char *cp;

	if ((cp = t->words[1]) == NULL) {
		i = umask(0);
		umask(i);
		for (n=3*4; (n-=3) >= 0;)
			putc('0'+((i>>n)&07), stderr);
		putc('\n', stderr);
	} else {
		for (n=0; *cp>='0' && *cp<='9'; cp++)
			n = n*8 + (*cp-'0');
		umask(n);
	}
	return(0);
}

static int
dodot(t)
struct op *t;
{
	register int i;
	register char *sp, *tp;
	char *cp;

	if ((cp = t->words[1]) == NULL)
		return(0);
	sp = any('/', cp)? ":": path->value;
	while (*sp) {
		tp = e.linep;
		while (*sp && (*tp = *sp++) != ':')
			tp++;
		if (tp != e.linep)
			*tp++ = '/';
		for (i = 0; (*tp++ = cp[i++]) != '\0';)
			;
		if ((i = open(e.linep, 0)) >= 0) {
			exstat = 0;
			next(remap(i));
			return(exstat);
		}
	}
	prs(cp);
	err(": not found");
	return(-1);
}

static int
dowait(t)
struct op *t;
{
	register int i;
	register char *cp;

	if ((cp = t->words[1]) != NULL) {
		i = getn(cp);
		if (i == 0)
			return(0);
	} else
		i = -1;
	setstatus(waitfor(i, 1));
	return(0);
}


static int
doread(t)
struct op *t;
{
	register char *cp, **wp;
	register int nb = 0;
	register int  nl = 0;

	if (t->words[1] == NULL) {
		err("Usage: read name ...");
		return(1);
	}
	for (wp = t->words+1; *wp; wp++) {
		for (cp = e.linep; !nl && cp < elinep-1; cp++)
			if ((nb = read(0, cp, sizeof(*cp))) != sizeof(*cp) ||
			    (nl = (*cp == '\n')) ||
			    (wp[1] && any(*cp, ifs->value)))
				break;
		*cp = 0;
		if (nb <= 0)
			break;
		setval(lookup(*wp), e.linep);
	}
	return(nb <= 0);
}

static int
doeval(t)
register struct op *t;
{
	return(RUN(awordlist, t->words+1, wdchar));
}

static int
dotrap(t)
register struct op *t;
{
	register int  n, i;
	register int  resetsig;

	if (t->words[1] == NULL) {
		for (i=0; i<=_NSIG; i++)
			if (trap[i]) {
				prn(i);
				prs(": ");
				prs(trap[i]);
				prs("\n");
			}
		return(0);
	}
	resetsig = isdigit(*t->words[1]);
	for (i = resetsig ? 1 : 2; t->words[i] != NULL; ++i) {
		n = getsig(t->words[i]);
		freecell(trap[n]);
		trap[n] = 0;
		if (!resetsig) {
			if (*t->words[1] != '\0') {
				trap[n] = strsave(t->words[1], 0);
				setsig(n, sig);
			} else
				setsig(n, SIG_IGN);
		} else {
			if (interactive)
				if (n == SIGINT)
					setsig(n, onintr);
				else
					setsig(n, n == SIGQUIT ? SIG_IGN 
							       : SIG_DFL);
			else
				setsig(n, SIG_DFL);
		}
	}
	return(0);
}

static int
dobreak(t)
struct op *t;
{
	return(brkcontin(t->words[1], 1));
}

static int
docontinue(t)
struct op *t;
{
	return(brkcontin(t->words[1], 0));
}

static int
doexport(t)
struct op *t;
{
	rdexp(t->words+1, export, EXPORT);
	return(0);
}

static int
doreadonly(t)
struct op *t;
{
	rdexp(t->words+1, ronly, RONLY);
	return(0);
}
static int
doset(t)
register struct op *t;
{
	register struct var *vp;
	register char *cp;
	register int n;

	if ((cp = t->words[1]) == NULL) {
		for (vp = vlist; vp; vp = vp->next)
			varput(vp->name, 1);
		return(0);
	}
	if (*cp == '-') {
		/* bad: t->words++; */
		for(n = 0; (t->words[n]=t->words[n+1]) != NULL; n++)
			;
		if (*++cp == 0)
			flag['x'] = flag['v'] = 0;
		else
			for (; *cp; cp++)
				switch (*cp) {
				case 'e':
					if (!interactive)
						flag['e']++;
					break;

				default:
					if (*cp>='a' && *cp<='z')
						flag[(int)*cp]++;
					break;
				}
		setdash();
	}
	if (t->words[1]) {
		t->words[0] = dolv[0];
		for (n=1; t->words[n]; n++)
			setarea((char *)t->words[n], 0);
		dolc = n-1;
		dolv = t->words;
		setval(lookup("#"), putn(dolc));
		setarea((char *)(dolv-1), 0);
	}
	return(0);
}

/*
 * Copyright (c) 1999 Herbert Xu <herbert@debian.org>
 * This file contains code for the times builtin.
 */
static int dotimes ()
{
	struct tms buf;
	long int clk_tck = sysconf(_SC_CLK_TCK);

	times(&buf);
	printf("%dm%fs %dm%fs\n%dm%fs %dm%fs\n",
	       (int) (buf.tms_utime / clk_tck / 60),
	       ((double) buf.tms_utime) / clk_tck,
	       (int) (buf.tms_stime / clk_tck / 60),
	       ((double) buf.tms_stime) / clk_tck,
	       (int) (buf.tms_cutime / clk_tck / 60),
	       ((double) buf.tms_cutime) / clk_tck,
	       (int) (buf.tms_cstime / clk_tck / 60),
	       ((double) buf.tms_cstime) / clk_tck);
	return 0;
}


#endif

static int (*inbuilt(char *s))()
{
	const struct builtincmd *bp;

	for (bp = builtincmds; bp->name != NULL; bp++)
		if (strcmp(bp->name, s) == 0)
			return(bp->builtinfunc);

	return((int(*)())NULL);
}

static int
getsig(s)
char *s;
{
	register int n;

	if ((n = getn(s)) < 0 || n > _NSIG) {
		err("trap: bad signal number");
		n = 0;
	}
	return(n);
}

static void
setsig( register int n, void (*f)(int))
{
	if (n == 0)
		return;
	if (signal(n, SIG_IGN) != SIG_IGN || ourtrap[n]) {
		ourtrap[n] = 1;
		signal(n, f);
	}
}

static int
getn(as)
char *as;
{
	register char *s;
	register int n, m;

	s = as;
	m = 1;
	if (*s == '-') {
		m = -1;
		s++;
	}
	for (n = 0; isdigit(*s); s++)
		n = (n*10) + (*s-'0');
	if (*s) {
		prs(as);
		err(": bad number");
	}
	return(n*m);
}

/*static int
brkcontin(cp, val)
register char *cp;
int val;
{
	register struct brkcon *bc;
	register int nl;

	nl = cp == NULL? 1: getn(cp);
	if (nl <= 0)
		nl = 999;
	do {
		if ((bc = brklist) == NULL)
			break;
		brklist = bc->nextlev;
	} while (--nl);
	if (nl) {
		err("bad break/continue level");
		return(1);
	}
	isbreak = val;
	longjmp(bc->brkpt, 1);
	
}

static void
rdexp(wp, f, key)
register char **wp;
void (*f)();
int key;
{
	if (*wp != NULL) {
		for (; *wp != NULL; wp++) {
			if (isassign(*wp)) {
				char *cp;
				assign(*wp, COPYV);
				for (cp = *wp; *cp != '='; cp++)
					;
				*cp = '\0';
			}
			if (checkname(*wp))
				(*f)(lookup(*wp));
			else
				badid(*wp);
		}
	} else
		putvlist(key, 1);
}

static void
badid(s)
register char *s;
{
	prs(s);
	err(": bad identifier");
}

static void
varput(s, out)
register char *s;
int out;
{
	if (isalnum(*s) || *s == '_') {
		write(out, s, strlen(s));
		write(out, "\n", 1);
	}
}*/


/* -------- eval.c -------- */

/*
 * ${}
 * `command`
 * blank interpretation
 * quoting
 * glob
 */

static char ** eval( char **ap, int f)
{
	struct wdblock *wb;
	char **wp;
	char **wf;
	jmp_buf ev;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &wp;
	(void) &ap;
#endif
	wp = NULL;
	wb = NULL;
	wf = NULL;
	if (newenv(setjmp(errpt = ev)) == 0) {
		while (*ap && isassign(*ap))
			expand(*ap++, &wb, f & ~DOGLOB);
		if (flag['k']) {
			for (wf = ap; *wf; wf++) {
				if (isassign(*wf))
					expand(*wf, &wb, f & ~DOGLOB);
			}
		}
		for (wb = addword((char *)0, wb); *ap; ap++) {
			if (!flag['k'] || !isassign(*ap))
				expand(*ap, &wb, f & ~DOKEY);
		}
		wb = addword((char *)0, wb);
		wp = getwords(wb);
		quitenv();
	} else
		gflg = 1;
	return(gflg? (char **)NULL: wp);
}

/*
 * Make the exported environment from the exported
 * names in the dictionary. Keyword assignments
 * will already have been done.
 */


static char *
evalstr(cp, f)
register char *cp;
int f;
{
	struct wdblock *wb;

	wb = NULL;
	if (expand(cp, &wb, f)) {
		if (wb == NULL || wb->w_nword == 0 || (cp = wb->w_words[0]) == NULL)
			cp = "";
		DELETE(wb);
	} else
		cp = NULL;
	return(cp);
}

static int
expand( char *cp, register struct wdblock **wbp, int f)
{
	jmp_buf ev;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &cp;
#endif
	gflg = 0;
	if (cp == NULL)
		return(0);
	if (!anys("$`'\"", cp) &&
	    !anys(ifs->value, cp) &&
	    ((f&DOGLOB)==0 || !anys("[*?", cp))) {
		cp = strsave(cp, areanum);
		if (f & DOTRIM)
			unquote(cp);
		*wbp = addword(cp, *wbp);
		return(1);
	}
	if (newenv(setjmp(errpt = ev)) == 0) {
		PUSHIO(aword, cp, strchar);
		e.iobase = e.iop;
		while ((cp = blank(f)) && gflg == 0) {
			e.linep = cp;
			cp = strsave(cp, areanum);
			if ((f&DOGLOB) == 0) {
				if (f & DOTRIM)
					unquote(cp);
				*wbp = addword(cp, *wbp);
			} else
				*wbp = glob(cp, *wbp);
		}
		quitenv();
	} else
		gflg = 1;
	return(gflg == 0);
}

/*
 * Blank interpretation and quoting
 */
static char *
blank(f)
int f;
{
	register int c, c1;
	register char *sp;
	int scanequals, foundequals;

	sp = e.linep;
	scanequals = f & DOKEY;
	foundequals = 0;

loop:
	switch (c = subgetc('"', foundequals)) {
	case 0:
		if (sp == e.linep)
			return(0);
		*e.linep++ = 0;
		return(sp);

	default:
		if (f & DOBLANK && any(c, ifs->value))
			goto loop;
		break;

	case '"':
	case '\'':
		scanequals = 0;
		if (INSUB())
			break;
		for (c1 = c; (c = subgetc(c1, 1)) != c1;) {
			if (c == 0)
				break;
			if (c == '\'' || !any(c, "$`\""))
				c |= QUOTE;
			*e.linep++ = c;
		}
		c = 0;
	}
	unget(c);
	if (!isalpha(c) && c != '_')
		scanequals = 0;
	for (;;) {
		c = subgetc('"', foundequals);
		if (c == 0 ||
		    f & (DOBLANK && any(c, ifs->value)) ||
		    (!INSUB() && any(c, "\"'"))) {
		        scanequals = 0;
			unget(c);
			if (any(c, "\"'"))
				goto loop;
			break;
		}
		if (scanequals) {
			if (c == '=') {
				foundequals = 1;
				scanequals  = 0;
			}
			else if (!isalnum(c) && c != '_')
				scanequals = 0;
		}
		*e.linep++ = c;
	}
	*e.linep++ = 0;
	return(sp);
}

/*
 * Get characters, substituting for ` and $
 */
static int
subgetc(ec, quoted)
register int ec;
int quoted;
{
	register char c;

again:
	c = my_getc(ec);
	if (!INSUB() && ec != '\'') {
		if (c == '`') {
			if (grave(quoted) == 0)
				return(0);
			e.iop->task = XGRAVE;
			goto again;
		}
		if (c == '$' && (c = dollar(quoted)) == 0) {
			e.iop->task = XDOLL;
			goto again;
		}
	}
	return(c);
}

/*
 * Prepare to generate the string returned by ${} substitution.
 */
static int
dollar(quoted)
int quoted;
{
	int otask;
	struct io *oiop;
	char *dolp;
	register char *s, c, *cp=NULL;
	struct var *vp;

	c = readc();
	s = e.linep;
	if (c != '{') {
		*e.linep++ = c;
		if (isalpha(c) || c == '_') {
			while ((c = readc())!=0 && (isalnum(c) || c == '_'))
				if (e.linep < elinep)
					*e.linep++ = c;
			unget(c);
		}
		c = 0;
	} else {
		oiop = e.iop;
		otask = e.iop->task;
		e.iop->task = XOTHER;
		while ((c = subgetc('"', 0))!=0 && c!='}' && c!='\n')
			if (e.linep < elinep)
				*e.linep++ = c;
		if (oiop == e.iop)
			e.iop->task = otask;
		if (c != '}') {
			err("unclosed ${");
			gflg++;
			return(c);
		}
	}
	if (e.linep >= elinep) {
		err("string in ${} too long");
		gflg++;
		e.linep -= 10;
	}
	*e.linep = 0;
	if (*s)
		for (cp = s+1; *cp; cp++)
			if (any(*cp, "=-+?")) {
				c = *cp;
				*cp++ = 0;
				break;
			}
	if (s[1] == 0 && (*s == '*' || *s == '@')) {
		if (dolc > 1) {
			/* currently this does not distinguish $* and $@ */
			/* should check dollar */
			e.linep = s;
			PUSHIO(awordlist, dolv+1, dolchar);
			return(0);
		} else {	/* trap the nasty ${=} */
			s[0] = '1';
			s[1] = 0;
		}
	}
	vp = lookup(s);
	if ((dolp = vp->value) == null) {
		switch (c) {
		case '=':
			if (isdigit(*s)) {
				err("cannot use ${...=...} with $n");
				gflg++;
				break;
			}
			setval(vp, cp);
			dolp = vp->value;
			break;

		case '-':
			dolp = strsave(cp, areanum);
			break;

		case '?':
			if (*cp == 0) {
				prs("missing value for ");
				err(s);
			} else
				err(cp);
			gflg++;
			break;
		}
	} else if (c == '+')
		dolp = strsave(cp, areanum);
	if (flag['u'] && dolp == null) {
		prs("unset variable: ");
		err(s);
		gflg++;
	}
	e.linep = s;
	PUSHIO(aword, dolp, quoted ? qstrchar : strchar);
	return(0);
}

/*
 * Run the command in `...` and read its output.
 */
static int
grave(quoted)
int quoted;
{
	register int i;
	char *cp;
	int pf[2];

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &cp;
#endif
	for (cp = e.iop->argp->aword; *cp != '`'; cp++)
		if (*cp == 0) {
			err("no closing `");
			return(0);
		}
	if (openpipe(pf) < 0)
		return(0);
	if ((i = vfork()) == -1) {
		closepipe(pf);
		err("try again");
		return(0);
	}
	if (i != 0) {
		e.iop->argp->aword = ++cp;
		close(pf[1]);
		PUSHIO(afile, remap(pf[0]), quoted? qgravechar: gravechar);
		return(1);
	}
	*cp = 0;
	/* allow trapped signals */
	for (i=0; i<=_NSIG; i++)
		if (ourtrap[i] && signal(i, SIG_IGN) != SIG_IGN)
			signal(i, SIG_DFL);
	dup2(pf[1], 1);
	closepipe(pf);
	flag['e'] = 0;
	flag['v'] = 0;
	flag['n'] = 0;
	cp = strsave(e.iop->argp->aword, 0);
	areanum = 1;
	freehere(areanum);
	freearea(areanum);	/* free old space */
	e.oenv = NULL;
	e.iop = (e.iobase = iostack) - 1;
	unquote(cp);
	interactive = 0;
	PUSHIO(aword, cp, nlchar);
	onecommand();
	exit(1);
}

static char *
unquote(as)
register char *as;
{
	register char *s;

	if ((s = as) != NULL)
		while (*s)
			*s++ &= ~QUOTE;
	return(as);
}

/* -------- glob.c -------- */

/*
 * glob
 */

#define	scopy(x) strsave((x), areanum)
#define	BLKSIZ	512
#define	NDENT	((BLKSIZ+sizeof(struct dirent)-1)/sizeof(struct dirent))

static	struct wdblock	*cl, *nl;
static	char	spcl[] = "[?*";

static struct wdblock *
glob(cp, wb)
char *cp;
struct wdblock *wb;
{
	register int i;
	register char *pp;

	if (cp == 0)
		return(wb);
	i = 0;
	for (pp = cp; *pp; pp++)
		if (any(*pp, spcl))
			i++;
		else if (!any(*pp & ~QUOTE, spcl))
			*pp &= ~QUOTE;
	if (i != 0) {
		for (cl = addword(scopy(cp), (struct wdblock *)0); anyspcl(cl); cl = nl) {
			nl = newword(cl->w_nword*2);
			for(i=0; i<cl->w_nword; i++) { /* for each argument */
				for (pp = cl->w_words[i]; *pp; pp++)
					if (any(*pp, spcl)) {
						globname(cl->w_words[i], pp);
						break;
					}
				if (*pp == '\0')
					nl = addword(scopy(cl->w_words[i]), nl);
			}
			for(i=0; i<cl->w_nword; i++)
				DELETE(cl->w_words[i]);
			DELETE(cl);
		}
		for(i=0; i<cl->w_nword; i++)
			unquote(cl->w_words[i]);
		glob0((char *)cl->w_words, cl->w_nword, sizeof(char *), xstrcmp);
		if (cl->w_nword) {
			for (i=0; i<cl->w_nword; i++)
				wb = addword(cl->w_words[i], wb);
			DELETE(cl);
			return(wb);
		}
	}
	wb = addword(unquote(cp), wb);
	return(wb);
}

static void
globname(we, pp)
char *we;
register char *pp;
{
	register char *np, *cp;
	char *name, *gp, *dp;
	int k;
	DIR *dirp;
	struct dirent *de;
	char dname[NAME_MAX+1];
	struct stat dbuf;

	for (np = we; np != pp; pp--)
		if (pp[-1] == '/')
			break;
	for (dp = cp = space((int)(pp-np)+3); np < pp;)
		*cp++ = *np++;
	*cp++ = '.';
	*cp = '\0';
	for (gp = cp = space(strlen(pp)+1); *np && *np != '/';)
		*cp++ = *np++;
	*cp = '\0';
	dirp = opendir(dp);
	if (dirp == 0) {
		DELETE(dp);
		DELETE(gp);
		return;
	}
	dname[NAME_MAX] = '\0';
	while ((de=readdir(dirp))!=NULL) {
		/* XXX Hmmm... What this could be? (abial) */
		/*
		if (ent[j].d_ino == 0)
			continue;
		*/
		strncpy(dname, de->d_name, NAME_MAX);
			if (dname[0] == '.')
				if (*gp != '.')
					continue;
			for(k=0; k<NAME_MAX; k++)
				if (any(dname[k], spcl))
					dname[k] |= QUOTE;
			if (gmatch(dname, gp)) {
				name = generate(we, pp, dname, np);
				if (*np && !anys(np, spcl)) {
					if (stat(name,&dbuf)) {
						DELETE(name);
						continue;
					}
				}
				nl = addword(name, nl);
			}
	}
	closedir(dirp);
	DELETE(dp);
	DELETE(gp);
}

/*
 * generate a pathname as below.
 * start..end1 / middle end
 * the slashes come for free
 */
static char *
generate(start1, end1, middle, end)
char *start1;
register char *end1;
char *middle, *end;
{
	char *p;
	register char *op, *xp;

	p = op = space((int)(end1-start1)+strlen(middle)+strlen(end)+2);
	for (xp = start1; xp != end1;)
		*op++ = *xp++;
	for (xp = middle; (*op++ = *xp++) != '\0';)
		;
	op--;
	for (xp = end; (*op++ = *xp++) != '\0';)
		;
	return(p);
}

static int
anyspcl(wb)
register struct wdblock *wb;
{
	register int i;
	register char **wd;

	wd = wb->w_words;
	for (i=0; i<wb->w_nword; i++)
		if (anys(spcl, *wd++))
			return(1);
	return(0);
}

static int
xstrcmp(p1, p2)
char *p1, *p2;
{
	return(strcmp(*(char **)p1, *(char **)p2));
}

/* -------- word.c -------- */

static struct wdblock *
newword(nw)
register int nw;
{
	register struct wdblock *wb;

	wb = (struct wdblock *) space(sizeof(*wb) + nw*sizeof(char *));
	wb->w_bsize = nw;
	wb->w_nword = 0;
	return(wb);
}

static struct wdblock *
addword(wd, wb)
char *wd;
register struct wdblock *wb;
{
	register struct wdblock *wb2;
	register int nw;

	if (wb == NULL)
		wb = newword(NSTART);
	if ((nw = wb->w_nword) >= wb->w_bsize) {
		wb2 = newword(nw * 2);
		memcpy((char *)wb2->w_words, (char *)wb->w_words, nw*sizeof(char *));
		wb2->w_nword = nw;
		DELETE(wb);
		wb = wb2;
	}
	wb->w_words[wb->w_nword++] = wd;
	return(wb);
}
static 
char **
getwords(wb)
register struct wdblock *wb;
{
	register char **wd;
	register int nb;

	if (wb == NULL)
		return((char **)NULL);
	if (wb->w_nword == 0) {
		DELETE(wb);
		return((char **)NULL);
	}
	wd = (char **) space(nb = sizeof(*wd) * wb->w_nword);
	memcpy((char *)wd, (char *)wb->w_words, nb);
	DELETE(wb);	/* perhaps should done by caller */
	return(wd);
}

int (*func)(char *, char *);
int	globv;

static void
glob0(a0, a1, a2, a3)
char *a0;
unsigned a1;
int a2;
int (*a3) (char *, char *);
{
	func = a3;
	globv = a2;
	glob1(a0, a0 + a1 * a2);
}

static void
glob1(base, lim)
char *base, *lim;
{
	register char *i, *j;
	int v2;
	char *lptr, *hptr;
	int c;
	unsigned n;


	v2 = globv;

top:
	if ((n=(int)(lim-base)) <= v2)
		return;
	n = v2 * (n / (2*v2));
	hptr = lptr = base+n;
	i = base;
	j = lim-v2;
	for(;;) {
		if (i < lptr) {
			if ((c = (*func)(i, lptr)) == 0) {
				glob2(i, lptr -= v2);
				continue;
			}
			if (c < 0) {
				i += v2;
				continue;
			}
		}

begin:
		if (j > hptr) {
			if ((c = (*func)(hptr, j)) == 0) {
				glob2(hptr += v2, j);
				goto begin;
			}
			if (c > 0) {
				if (i == lptr) {
					glob3(i, hptr += v2, j);
					i = lptr += v2;
					goto begin;
				}
				glob2(i, j);
				j -= v2;
				i += v2;
				continue;
			}
			j -= v2;
			goto begin;
		}


		if (i == lptr) {
			if (lptr-base >= lim-hptr) {
				glob1(hptr+v2, lim);
				lim = lptr;
			} else {
				glob1(base, lptr);
				base = hptr+v2;
			}
			goto top;
		}


		glob3(j, lptr -= v2, i);
		j = hptr -= v2;
	}
}

static void
glob2(i, j)
char *i, *j;
{
	register char *index1, *index2, c;
	int m;

	m = globv;
	index1 = i;
	index2 = j;
	do {
		c = *index1;
		*index1++ = *index2;
		*index2++ = c;
	} while(--m);
}

static void
glob3(i, j, k)
char *i, *j, *k;
{
	register char *index1, *index2, *index3;
	int c;
	int m;

	m = globv;
	index1 = i;
	index2 = j;
	index3 = k;
	do {
		c = *index1;
		*index1++ = *index3;
		*index3++ = *index2;
		*index2++ = c;
	} while(--m);
}

/* -------- io.c -------- */

/*
 * shell IO
 */

static int my_getc( int ec)
{
	register int c;

	if(e.linep > elinep) {
		while((c=readc()) != '\n' && c)
			;
		err("input line too long");
		gflg++;
		return(c);
	}
	c = readc();
 	if (ec != '\'' && e.iop->task != XGRAVE) {
		if(c == '\\') {
			c = readc();
			if (c == '\n' && ec != '\"')
				return(my_getc(ec));
			c |= QUOTE;
		}
	}
	return(c);
}

static void
unget(c)
int c;
{
	if (e.iop >= e.iobase)
		e.iop->peekc = c;
}

static int
eofc()

{
  return e.iop < e.iobase || (e.iop->peekc == 0 && e.iop->prev == 0);
}

static int
readc()
{
	register int c;

	for (; e.iop >= e.iobase; e.iop--)
		if ((c = e.iop->peekc) != '\0') {
			e.iop->peekc = 0;
			return(c);
		}
		else {
		    if (e.iop->prev != 0) {
		        if ((c = (*e.iop->iofn)(e.iop->argp, e.iop)) != '\0') {
			        if (c == -1) {
				        e.iop++;
				        continue;
			        }
			        if (e.iop == iostack)
				        ioecho(c);
			        return(e.iop->prev = c);
		        }
		        else if (e.iop->task == XIO && e.iop->prev != '\n') {
			        e.iop->prev = 0;
				if (e.iop == iostack)
					ioecho('\n');
			        return '\n';
		        }
		    }
		    if (e.iop->task == XIO) {
			if (multiline)
			    return e.iop->prev = 0;
			if (interactive && e.iop == iostack+1) {
#ifdef CONFIG_FEATURE_COMMAND_EDITING
			    current_prompt=prompt->value;
#else
			    prs(prompt->value);
#endif
			}
		    }
		}
	if (e.iop >= iostack)
		return(0);
	leave();
	/* NOTREACHED */
	return(0);
}

static void
ioecho(c)
char c;
{
	if (flag['v'])
		write(2, &c, sizeof c);
}

static void
pushio(argp, fn)
struct ioarg *argp;
int (*fn)();
{
	if (++e.iop >= &iostack[NPUSH]) {
		e.iop--;
		err("Shell input nested too deeply");
		gflg++;
		return;
	}
	e.iop->iofn = fn;

	if (argp->afid != AFID_NOBUF)
	  e.iop->argp = argp;
	else {
	  e.iop->argp  = ioargstack + (e.iop - iostack);
	  *e.iop->argp = *argp;
	  e.iop->argp->afbuf = e.iop == &iostack[0] ? &mainbuf : &sharedbuf;
	  if (isatty(e.iop->argp->afile) == 0 &&
	      (e.iop == &iostack[0] ||
	       lseek(e.iop->argp->afile, 0L, 1) != -1)) {
	    if (++bufid == AFID_NOBUF)
	      bufid = AFID_ID;
	    e.iop->argp->afid  = bufid;
	  }
	}

	e.iop->prev  = ~'\n';
	e.iop->peekc = 0;
	e.iop->xchar = 0;
	e.iop->nlcount = 0;
	if (fn == filechar || fn == linechar)
		e.iop->task = XIO;
	else if (fn == gravechar || fn == qgravechar)
		e.iop->task = XGRAVE;
	else
		e.iop->task = XOTHER;
}

static struct io *
setbase(ip)
struct io *ip;
{
	register struct io *xp;

	xp = e.iobase;
	e.iobase = ip;
	return(xp);
}

/*
 * Input generating functions
 */

/*
 * Produce the characters of a string, then a newline, then EOF.
 */
static int
nlchar(ap)
register struct ioarg *ap;
{
	register int c;

	if (ap->aword == NULL)
		return(0);
	if ((c = *ap->aword++) == 0) {
		ap->aword = NULL;
		return('\n');
	}
	return(c);
}

/*
 * Given a list of words, produce the characters
 * in them, with a space after each word.
 */
static int
wdchar(ap)
register struct ioarg *ap;
{
	register char c;
	register char **wl;

	if ((wl = ap->awordlist) == NULL)
		return(0);
	if (*wl != NULL) {
		if ((c = *(*wl)++) != 0)
			return(c & 0177);
		ap->awordlist++;
		return(' ');
	}
	ap->awordlist = NULL;
	return('\n');
}

/*
 * Return the characters of a list of words,
 * producing a space between them.
 */
static int
dolchar(ap)
register struct ioarg *ap;
{
	register char *wp;

	if ((wp = *ap->awordlist++) != NULL) {
		PUSHIO(aword, wp, *ap->awordlist == NULL? strchar: xxchar);
		return(-1);
	}
	return(0);
}

static int
xxchar(ap)
register struct ioarg *ap;
{
	register int c;

	if (ap->aword == NULL)
		return(0);
	if ((c = *ap->aword++) == '\0') {
		ap->aword = NULL;
		return(' ');
	}
	return(c);
}

/*
 * Produce the characters from a single word (string).
 */
static int
strchar(ap)
register struct ioarg *ap;
{
	register int c;

	if (ap->aword == NULL || (c = *ap->aword++) == 0)
		return(0);
	return(c);
}

/*
 * Produce quoted characters from a single word (string).
 */
static int
qstrchar(ap)
register struct ioarg *ap;
{
	register int c;

	if (ap->aword == NULL || (c = *ap->aword++) == 0)
		return(0);
	return(c|QUOTE);
}

/*
 * Return the characters from a file.
 */
static int
filechar(ap)
register struct ioarg *ap;
{
	register int i;
	char c;
	struct iobuf *bp = ap->afbuf;

	if (ap->afid != AFID_NOBUF) {
	  if ((i = ap->afid != bp->id) || bp->bufp == bp->ebufp) {
	    if (i)
	      lseek(ap->afile, ap->afpos, 0);
	    i = safe_read(ap->afile, bp->buf, sizeof(bp->buf));
	    if (i <= 0) {
	      closef(ap->afile);
	      return 0;
	    }
	    bp->id = ap->afid;
	    bp->ebufp = (bp->bufp  = bp->buf) + i;
	  }
	  ap->afpos++;
	  return *bp->bufp++ & 0177;
	}

#ifdef CONFIG_FEATURE_COMMAND_EDITING
	if (interactive) {
	    static char mycommand[BUFSIZ];
	    static int position = 0, size = 0;

	    while (size == 0 || position >= size) {
		cmdedit_read_input(current_prompt, mycommand);
		size = strlen(mycommand);
		position = 0;
	    }
	    c = mycommand[position];
	    position++;
	    return(c);
	} else 
#endif
	{
		i = safe_read(ap->afile, &c, sizeof(c));
		return(i == sizeof(c)? c&0177: (closef(ap->afile), 0));
	}
}

/*
 * Return the characters from a here temp file.
 */
static int
herechar(ap)
register struct ioarg *ap;
{
	char c;


	if (read(ap->afile, &c, sizeof(c)) != sizeof(c)) {
		close(ap->afile);
		c = 0;
	}
	return (c);

}

/*
 * Return the characters produced by a process (`...`).
 * Quote them if required, and remove any trailing newline characters.
 */
static int
gravechar(ap, iop)
struct ioarg *ap;
struct io *iop;
{
	register int c;

	if ((c = qgravechar(ap, iop)&~QUOTE) == '\n')
		c = ' ';
	return(c);
}

static int
qgravechar(ap, iop)
register struct ioarg *ap;
struct io *iop;
{
	register int c;

	if (iop->xchar) {
		if (iop->nlcount) {
			iop->nlcount--;
			return('\n'|QUOTE);
		}
		c = iop->xchar;
		iop->xchar = 0;
	} else if ((c = filechar(ap)) == '\n') {
		iop->nlcount = 1;
		while ((c = filechar(ap)) == '\n')
			iop->nlcount++;
		iop->xchar = c;
		if (c == 0)
			return(c);
		iop->nlcount--;
		c = '\n';
	}
	return(c!=0? c|QUOTE: 0);
}

/*
 * Return a single command (usually the first line) from a file.
 */
static int
linechar(ap)
register struct ioarg *ap;
{
	register int c;

	if ((c = filechar(ap)) == '\n') {
		if (!multiline) {
			closef(ap->afile);
			ap->afile = -1;	/* illegal value */
		}
	}
	return(c);
}

static void
prs(s)
register char *s;
{
	if (*s)
		write(2, s, strlen(s));
}

static void
prn(u)
unsigned u;
{
	prs(itoa(u, 0));
}

static void
closef(i)
register int i;
{
	if (i > 2)
		close(i);
}

static void
closeall()
{
	register int u;

	for (u=NUFILE; u<NOFILE;)
		close(u++);
}

/*
 * remap fd into Shell's fd space
 */
static int
remap(fd)
register int fd;
{
	register int i;
	int map[NOFILE];

	if (fd < e.iofd) {
		for (i=0; i<NOFILE; i++)
			map[i] = 0;
		do {
			map[fd] = 1;
			fd = dup(fd);
		} while (fd >= 0 && fd < e.iofd);
		for (i=0; i<NOFILE; i++)
			if (map[i])
				close(i);
		if (fd < 0)
			err("too many files open in shell");
	}
	return(fd);
}

static int
openpipe(pv)
register int *pv;
{
	register int i;

	if ((i = pipe(pv)) < 0)
		err("can't create pipe - try again");
	return(i);
}

static void
closepipe(pv)
register int *pv;
{
	if (pv != NULL) {
		close(*pv++);
		close(*pv);
	}
}

/* -------- here.c -------- */

/*
 * here documents
 */

static void
markhere(s, iop)
register char *s;
struct ioword *iop;
{
	register struct here *h, *lh;

	h = (struct here *) space(sizeof(struct here));
	if (h == 0)
		return;
	h->h_tag = evalstr(s, DOSUB);
	if (h->h_tag == 0)
		return;
	h->h_iop = iop;
	iop->io_name = 0;
	h->h_next = NULL;
	if (inhere == 0)
		inhere = h;
	else
		for (lh = inhere; lh!=NULL; lh = lh->h_next)
			if (lh->h_next == 0) {
				lh->h_next = h;
				break;
			}
	iop->io_flag |= IOHERE|IOXHERE;
	for (s = h->h_tag; *s; s++)
		if (*s & QUOTE) {
			iop->io_flag &= ~ IOXHERE;
			*s &= ~ QUOTE;
		}
	h->h_dosub = iop->io_flag & IOXHERE;
}

static void
gethere()
{
	register struct here *h, *hp;

	/* Scan here files first leaving inhere list in place */
	for (hp = h = inhere; h != NULL; hp = h, h = h->h_next)
	  readhere(&h->h_iop->io_name, h->h_tag, h->h_dosub? 0: '\'');

	/* Make inhere list active - keep list intact for scraphere */
	if (hp != NULL) {
	  hp->h_next = acthere;
	  acthere    = inhere;
	  inhere     = NULL;
	}
}

static void
readhere(name, s, ec)
char **name;
register char *s;
int ec;
{
	int tf;
	char tname[30] = ".msh_XXXXXX";
	register int c;
	jmp_buf ev;
	char myline [LINELIM+1];
	char *thenext;

	tf = mkstemp(tname);
	if (tf < 0)
		return;
	*name = strsave(tname, areanum);
	if (newenv(setjmp(errpt = ev)) != 0)
		unlink(tname);
	else {
		pushio(e.iop->argp, e.iop->iofn);
		e.iobase = e.iop;
		for (;;) {
		    if (interactive && e.iop <= iostack) {
#ifdef CONFIG_FEATURE_COMMAND_EDITING
			    current_prompt=cprompt->value;
#else
			    prs(cprompt->value);
#endif
			}
			thenext = myline;
			while ((c = my_getc(ec)) != '\n' && c) {
				if (ec == '\'')
					c &= ~ QUOTE;
				if (thenext >= &myline[LINELIM]) {
					c = 0;
					break;
				}
				*thenext++ = c;
			}
			*thenext = 0;
			if (strcmp(s, myline) == 0 || c == 0)
				break;
			*thenext++ = '\n';
			write (tf, myline, (int)(thenext-myline));
		}
		if (c == 0) {
			prs("here document `"); prs(s); err("' unclosed");
		}
		quitenv();
	}
	close(tf);
}

/*
 * open here temp file.
 * if unquoted here, expand here temp file into second temp file.
 */
static int
herein(hname, xdoll)
char *hname;
int xdoll;
{
	register int hf;
	int tf;

#if __GNUC__
	/* Avoid longjmp clobbering */
	(void) &tf;
#endif
	if (hname == 0)
		return(-1);
	hf = open(hname, 0);
	if (hf < 0)
		return (-1);
	if (xdoll) {
		char c;
		char tname[30] = ".msh_XXXXXX";
		jmp_buf ev;
	
		tf = mkstemp(tname);
		if (tf < 0)
			return (-1);
		if (newenv(setjmp(errpt = ev)) == 0) {
			PUSHIO(afile, hf, herechar);
			setbase(e.iop);
			while ((c = subgetc(0, 0)) != 0) {
				c &= ~ QUOTE;
				write(tf, &c, sizeof c);
			}
			quitenv();
		} else
			unlink(tname);
		close(tf);
		tf = open(tname, 0);
		unlink(tname);
		return (tf);
	} else
		return (hf);
}

static void
scraphere()
{
	register struct here *h;

	for (h = inhere; h != NULL; h = h->h_next) {
		if (h->h_iop && h->h_iop->io_name)
		  unlink(h->h_iop->io_name);
	}
	inhere = NULL;
}

/* unlink here temp files before a freearea(area) */
static void
freehere(area)
int area;
{
	register struct here *h, *hl;

	hl = NULL;
	for (h = acthere; h != NULL; h = h->h_next)
		if (getarea((char *) h) >= area) {
			if (h->h_iop->io_name != NULL)
				unlink(h->h_iop->io_name);
			if (hl == NULL)
				acthere = h->h_next;
			else
				hl->h_next = h->h_next;
		} else
			hl = h;
}



/*
 * Copyright (c) 1987,1997, Prentice Hall
 * All rights reserved.
 * 
 * Redistribution and use of the MINIX operating system in source and
 * binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 
 * Neither the name of Prentice Hall nor the names of the software
 * authors or contributors may be used to endorse or promote
 * products derived from this software without specific prior
 * written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS, AUTHORS, AND
 * CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PRENTICE HALL OR ANY AUTHORS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


